package com.hz.pm.api.common.enumeration;

import lombok.AllArgsConstructor;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * ProjectProcessType
 * </p>
 *
 * @author WendyYang
 * @since 14:25 2024/2/2
 */
@Getter
@AllArgsConstructor
public enum ProjectProcessType {

    /**
     * 项目流程配置 阶段枚举
     */
    ACCEPTANCE_DECLARATION_APPROVAL_PROCESS(5, "验收申报审批流程"),
    PROJECT_RECORD_APPROVAL_PROCESS(8, "立项备案审批流程"),
    XC_APPROVAL_PROCESS(9, "信创审批流程"),
    SELF_TEST(10, "系统自测审批流程"),
    ADAPTION(11, "开工文件审批流程"),
    TEST_VALID(12, "监理核实验证审批流程"),
    COMPLIANCE_REVIEW(13, "合规性审查审批流程"),
    PROJECT_REVIEW(14, "项目复核审批流程"),
    PROJECT_STOPPED(15, "项目终止审批流程"),
    PROJECT_CHANGE(16, "项目变更审批流程"),
    PURCHASE_NOTICE(17, "增加标项复核流程");

    private final Integer code;
    private final String desc;

    public boolean eq(Integer code) {
        return this.code.equals(code);
    }

    public static String getDesc(Integer code) {
        if (Objects.isNull(code)) {
            return StringUtils.EMPTY;
        }
        for (ProjectProcessType t : ProjectProcessType.values()) {
            if (code.equals(t.getCode())) {
                return t.desc;
            }
        }
        return StringUtils.EMPTY;
    }

    private static Optional<ProjectProcessType> get(Integer code) {
        return Arrays.stream(values()).filter(w -> w.getCode().equals(code)).findFirst();
    }

    public static ProjectProcessType getNoNull(Integer code) {
        return get(code).orElseThrow(() -> new IllegalArgumentException("流程实例类型"));
    }

}
