package com.hz.pm.api.common.statemachine.action;

import cn.hutool.core.lang.Assert;
import com.hz.pm.api.common.model.constant.StateMachineConst;
import com.hz.pm.api.common.statemachine.builder.impl.ProjectStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;


/**
 * 项目申报状态机action集合类
 *
 * @author CMM
 * @since 2023/02/08 13:48
 */
@Slf4j
@SuppressWarnings("all")
@WithStateMachine(id = ProjectStateMachineBuilderImpl.MACHINE_ID)
public class ProjectStateChangeAction {

    public static Project getProject(Message<ProjectStateChangeEvent> message) {
        Project project = (Project) message.getHeaders().get(StateMachineConst.MessageHeader.PROJECT_DECLARE);
        Assert.notNull(project, "未获取到需要状态变更的项目信息");
        return project;
    }

    public static <T> T getExtraArgs(Message<ProjectStateChangeEvent> message) {
        T extraArgs = (T) message.getHeaders().get(StateMachineConst.MessageHeader.EXTRA_ARGS);
        Assert.notNull(extraArgs, "未获取到需要状态变更的附件参数");
        return extraArgs;
    }


    @OnTransition(source = "TO_BE_DECLARED", target = "ON_COMPLIANCE_REVIEW")
    public void PROJECT_APPLICATION_SUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStage(ProjectStatus.NOT_APPROVED.getCode());
        project.setStatus(ProjectStatus.ON_COMPLIANCE_REVIEW.getCode());
    }

    @OnTransition(source = "ON_COMPLIANCE_REVIEW", target = "COMPLIANCE_REVIEW_FAILED")
    public void COMPLIANCE_REVIEW_FAILED(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.COMPLIANCE_REVIEW_FAILED.getCode());
    }

    @OnTransition(source = "COMPLIANCE_REVIEW_FAILED", target = "ON_COMPLIANCE_REVIEW")
    public void COMPLIANCE_REVIEW_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_COMPLIANCE_REVIEW.getCode());
    }

    @OnTransition(source = "ON_COMPLIANCE_REVIEW", target = "WITHOUT_EXPERT_REVIEW")
    public void COMPLIANCE_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.WITHOUT_EXPERT_REVIEW.getCode());
    }

    @OnTransition(source = "WITHOUT_EXPERT_REVIEW", target = "ON_EXPERT_REVIEW")
    public void EXPERT_REVIEW_SUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_EXPERT_REVIEW.getCode());
    }

    @OnTransition(source = "EXPERT_REVIEW_AFTER_MODIFY", target = "WITHOUT_EXPERT_REVIEW")
    public void EXPERT_REVIEW_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.WITHOUT_EXPERT_REVIEW.getCode());
    }

    @OnTransition(source = "ON_EXPERT_REVIEW", target = "WITHOUT_EXPERT_REVIEW")
    public void EXPERT_REVIEW_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.WITHOUT_EXPERT_REVIEW.getCode());
    }

    @OnTransition(source = "ON_EXPERT_REVIEW", target = "EXPERT_REVIEW_FAILED")
    public void EXPERT_REVIEW_FAILED(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.EXPERT_REVIEW_FAILED.getCode());
    }

    @OnTransition(source = "ON_EXPERT_REVIEW", target = "EXPERT_REVIEW_AFTER_MODIFY")
    public void EXPERT_REVIEW_AFTER_MODIFY(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY.getCode());
    }

    @OnTransition(source = "ON_EXPERT_REVIEW", target = "WITHOUT_PROJECT_REVIEW")
    public void EXPERT_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.WITHOUT_PROJECT_REVIEW.getCode());
    }

    @OnTransition(source = "WITHOUT_PROJECT_REVIEW", target = "ON_PROJECT_REVIEW")
    public void PROJECT_REVIEW_SUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_PROJECT_REVIEW.getCode());
    }

    @OnTransition(source = "PROJECT_REVIEW_FAILED", target = "ON_PROJECT_REVIEW")
    public void PROJECT_REVIEW_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_PROJECT_REVIEW.getCode());
    }

    @OnTransition(source = "ON_PROJECT_REVIEW", target = "PROJECT_REVIEW_FAILED")
    public void PROJECT_REVIEW_FAILED(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.PROJECT_REVIEW_FAILED.getCode());
    }

    @OnTransition(source = "ON_PROJECT_REVIEW", target = "ON_ANNUAL_PLAN")
    public void PROJECT_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_ANNUAL_PLAN.getCode());
    }

    @OnTransition(source = "ON_ANNUAL_PLAN", target = "DECLARED_APPROVED_TO_BE_RECORD")
    public void ANNUAL_PLAN_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.getCode());
    }

    //==================================================================================================================

    @OnTransition(source = "DECLARED_APPROVED_TO_BE_RECORD", target = "DECLARED_APPROVED_RECORD_AUDITING")
    public void DECLARED_RECORD_SUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待立项批复批复后，项目一级状态变更为已立项
        project.setStatus(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING.getCode());
    }

    @OnTransition(source = "DECLARED_APPROVED_RECORD_FAILED", target = "DECLARED_APPROVED_RECORD_AUDITING")
    public void DECLARED_RECORD_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING.getCode());
    }

    @OnTransition(source = "DECLARED_APPROVED_RECORD_AUDITING", target = "TO_BE_PURCHASED")
    public void DECLARED_RECORD_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待立项批复批复后，项目一级状态变更为已立项
        project.setStage(ProjectStatus.PROJECT_APPROVED.getCode());
        project.setStatus(ProjectStatus.TO_BE_PURCHASED.getCode());
    }

    @OnTransition(source = "DECLARED_APPROVED_RECORD_AUDITING", target = "DECLARED_APPROVED_RECORD_FAILED")
    public void DECLARED_RECORD_FAILED(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.DECLARED_APPROVED_RECORD_FAILED.getCode());
    }

    //==================================================================================================================

    @OnTransition(source = "TO_BE_APPROVED", target = "TO_BE_PURCHASED")
    public void PROJECT_APPROVAL(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待立项批复批复后，项目一级状态变更为已立项
        project.setStage(ProjectStatus.PROJECT_APPROVED.getCode());
        project.setStatus(ProjectStatus.TO_BE_PURCHASED.getCode());
    }

    @OnTransition(source = "TO_BE_PURCHASED", target = "ON_PURCHASING")
    public void SUBMIT_PURCHASE_NOTICE(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_PURCHASING.getCode());
    }

    @OnTransition(source = "ON_PURCHASING", target = "ON_ADAPTING")
    public void SUBMIT_PURCHASE_CONTRACT_RECORD(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_ADAPTING.getCode());
    }

    @OnTransition(source = "ON_PURCHASING", target = "ACCEPTED")
    public void SUBMIT_PURCHASE_CONTRACT_RECORD_TO_FIRST_ACCEPT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ACCEPTED.getCode());
    }

    @OnTransition(source = "ON_ADAPTING", target = "TO_BE_FIRST_INSPECTED")
    public void SUBMIT_PURCHASE_ORG_CONFIRM(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.TO_BE_FIRST_INSPECTED.getCode());
    }

    @OnTransition(source = "TO_BE_FIRST_INSPECTED", target = "ON_PILOT_RUNNING")
    public void SUBMIT_FIRST_INSPECTED_FILES(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_PILOT_RUNNING.getCode());
    }

    @OnTransition(source = "ON_PILOT_RUNNING", target = "ON_FINALLY_INSPECTED")
    public void FINAL_ACCEPTANCE_APPLICATION(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_FINALLY_INSPECTED.getCode());
    }

    @OnTransition(source = "ON_FINALLY_INSPECTED", target = "ARCHIVED")
    public void FINAL_ACCEPTANCE_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 终验审核通过后，项目一级状态变更为已归档
        project.setStage(ProjectStatus.ARCHIVED.getCode());
        project.setStatus(ProjectStatus.ACCEPTED.getCode());
    }

    @OnTransition(source = "ON_FINALLY_INSPECTED", target = "FINALLY_INSPECTED_FAILED")
    public void FINAL_ACCEPTANCE_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.FINALLY_INSPECTED_FAILED.getCode());
    }

    @OnTransition(source = "FINALLY_INSPECTED_FAILED", target = "ON_FINALLY_INSPECTED")
    public void FINAL_RE_ACCEPTANCE_APPLICATION(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatus.ON_FINALLY_INSPECTED.getCode());
    }

}
