package com.hz.pm.api.common.statemachine.action;

import com.hz.pm.api.common.statemachine.builder.impl.XcfhxStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderXcfhxApplyStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

import static com.hz.pm.api.common.statemachine.action.TenderStateChangeActionUtil.getPurchaseInfo;

/**
 * <p>
 * XcfhxStateChangeAction
 * </p>
 *
 * @author WendyYang
 * @since 11:40 2024/4/8
 */
@Slf4j
@WithStateMachine(id = XcfhxStateMachineBuilderImpl.MACHINE_ID)
public class XcfhxStateChangeAction {

    /**
     * 信创符合性申请
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_XCFHX_APPLY", target = "ON_XCFHX_APPLY")
    public void SUBMIT_XCFHX_APPLY(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderXcfhxApplyStatus.ON_XCFHX_APPLY.getCode());
        purchase.setXcfhxApplyStatus(TenderXcfhxApplyStatus.ON_XCFHX_APPLY.getCode());
    }

    /**
     * 信创符合性申请通过
     *
     * @param message \
     */
    @OnTransition(source = "ON_XCFHX_APPLY", target = "XCFHX_APPLY_PASSED")
    public void XCFHX_APPLY_PASSED(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY.getCode());
        purchase.setXcfhxApplyStatus(TenderXcfhxApplyStatus.XCFHX_APPLY_PASSED.getCode());
    }

    /**
     * 信创符合性申请失败
     *
     * @param message \
     */
    @OnTransition(source = "ON_XCFHX_APPLY", target = "XCFHX_APPLY_FAILED")
    public void XCFHX_APPLY_FAILED(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderXcfhxApplyStatus.XCFHX_APPLY_FAILED.getCode());
        purchase.setXcfhxApplyStatus(TenderXcfhxApplyStatus.XCFHX_APPLY_FAILED.getCode());
    }

    /**
     * 重新提交信创符合性申请
     *
     * @param message \
     */
    @OnTransition(source = "XCFHX_APPLY_FAILED", target = "ON_XCFHX_APPLY")
    public void RESUBMIT_XCFHX_APPLY(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderXcfhxApplyStatus.ON_XCFHX_APPLY.getCode());
        purchase.setXcfhxApplyStatus(TenderXcfhxApplyStatus.ON_XCFHX_APPLY.getCode());
    }

}
