package com.hz.pm.api.common.statemachine.factory;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.statemachine.action.ProjectStateChangeAction;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.meeting.entity.enumeration.ExpertReviewResultEnum;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import lombok.AllArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.statemachine.StateContext;
import org.springframework.statemachine.guard.Guard;

import java.util.List;

/**
 * <p>
 * ProjectGuardFactory
 * </p>
 *
 * @author WendyYang
 * @since 11:28 2024/8/22
 */
@Configuration
public class ProjectGuardFactory {

    private final IPurchaseService purchaseService;

    public ProjectGuardFactory(IPurchaseService purchaseService) {
        this.purchaseService = purchaseService;
    }

    @AllArgsConstructor
    public static class ExpertReviewToFailed implements Guard<ProjectStatus, ProjectStateChangeEvent> {

        @Override
        public boolean evaluate(StateContext<ProjectStatus, ProjectStateChangeEvent> ctx) {
            Integer reviewResult = ProjectStateChangeAction.getExtraArgs(ctx.getMessage());
            return ExpertReviewResultEnum.FAILED.eq(reviewResult);
        }

    }

    @AllArgsConstructor
    public static class ExpertReviewToModify implements Guard<ProjectStatus, ProjectStateChangeEvent> {

        @Override
        public boolean evaluate(StateContext<ProjectStatus, ProjectStateChangeEvent> ctx) {
            Integer reviewResult = ProjectStateChangeAction.getExtraArgs(ctx.getMessage());
            return ExpertReviewResultEnum.REVIEW_AFTER_MODIFY.eq(reviewResult);
        }

    }

    @AllArgsConstructor
    public static class ProjectPurchaseToAcceptGuard implements Guard<ProjectStatus, ProjectStateChangeEvent> {

        private IPurchaseService purchaseService;

        @Override
        public boolean evaluate(StateContext<ProjectStatus, ProjectStateChangeEvent> ctx) {
            return !canTurnToAdapt(ctx, purchaseService);
        }

    }

    @AllArgsConstructor
    public static class ProjectPurchaseToAdaptGuard implements Guard<ProjectStatus, ProjectStateChangeEvent> {

        private IPurchaseService purchaseService;

        @Override
        public boolean evaluate(StateContext<ProjectStatus, ProjectStateChangeEvent> ctx) {
            return canTurnToAdapt(ctx, purchaseService);
        }
    }


    private static boolean canTurnToAdapt(StateContext<ProjectStatus, ProjectStateChangeEvent> ctx,
                                          IPurchaseService purchaseService) {
        Project project = ProjectStateChangeAction.getProject(ctx.getMessage());
        Wrapper<Purchase> query = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getId, Purchase::getBidType)
                .eq(Purchase::getProjectId, project.getId());
        List<Purchase> purchases = purchaseService.list(query);
        return CollUtil.anyMatch(purchases, w -> BidTypeEnum.BUILD_APP.eq(w.getBidType()));
    }


    @Bean
    public ProjectPurchaseToAcceptGuard projectPurchaseToAcceptGuard() {
        return new ProjectPurchaseToAcceptGuard(purchaseService);
    }

    @Bean
    public ProjectPurchaseToAdaptGuard projectPurchaseToAdaptGuard() {
        return new ProjectPurchaseToAdaptGuard(purchaseService);
    }

}
