package com.hz.pm.api.common.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Objects;
import java.util.function.Function;

/**
 * <p>
 * DecimalUtil
 * </p>
 *
 * @author WendyYang
 * @since 18:53 2024/2/21
 */
public class DecimalUtil {

    private DecimalUtil() {
    }

    public static boolean noNullAndGtZero(BigDecimal num) {
        return num != null && num.compareTo(BigDecimal.ZERO) > 0;
    }

    @SafeVarargs
    public static <T> BigDecimal sum(Collection<T> objs, Function<T, BigDecimal>... functions) {
        if (CollUtil.isEmpty(objs) || ArrayUtil.isEmpty(functions)) {
            return BigDecimal.ZERO;
        }
        return objs.stream()
                .map(w -> Arrays.stream(functions)
                        .map(f -> f.apply(w))
                        .filter(Objects::nonNull)
                        .reduce(BigDecimal::add)
                        .orElse(BigDecimal.ZERO))
                .reduce(BigDecimal::add)
                .orElse(BigDecimal.ZERO);

    }

    public static <T> BigDecimal avg(Collection<T> objs, Function<T, BigDecimal> function) {
        if (CollUtil.isEmpty(objs)) {
            return BigDecimal.ZERO;
        }
        BigDecimal size = BigDecimal.valueOf(objs.size());
        return objs.stream()
                .map(function)
                .reduce(BigDecimal::add)
                .orElse(BigDecimal.ZERO)
                .divide(size, RoundingMode.HALF_UP);
    }

    public static <T> BigDecimal max(Collection<T> objs, Function<T, BigDecimal> function) {
        if (CollUtil.isEmpty(objs)) {
            return BigDecimal.ZERO;
        }
        return objs.stream().map(function).max(BigDecimal::compareTo).orElse(BigDecimal.ZERO);
    }

    public static <T> BigDecimal min(Collection<T> objs, Function<T, BigDecimal> function) {
        if (CollUtil.isEmpty(objs)) {
            return BigDecimal.ZERO;
        }
        return objs.stream().map(function).min(BigDecimal::compareTo).orElse(BigDecimal.ZERO);
    }

    public static <T> T objByMax(Collection<T> objs, Function<T, BigDecimal> function) {
        if (CollUtil.isEmpty(objs)) {
            return null;
        }
        return objs.stream().max(Comparator.comparing(function)).orElse(null);
    }

    public static <T> T objByMin(Collection<T> objs, Function<T, BigDecimal> function) {
        if (CollUtil.isEmpty(objs)) {
            return null;
        }
        return objs.stream().min(Comparator.comparing(function)).orElse(null);
    }

    public static BigDecimal sum(BigDecimal... nums) {
        if (ArrayUtil.isEmpty(nums)) {
            return BigDecimal.ZERO;
        }
        return Arrays.stream(nums).filter(Objects::nonNull).reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
    }

}
