package com.hz.pm.api.common.util;

import cn.hutool.core.date.StopWatch;
import cn.hutool.extra.spring.SpringUtil;
import com.hz.pm.api.common.config.ThreadPoolUtilProperties;
import com.hz.pm.api.common.config.ThreadPoolUtilProperties.ThreadPoolProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.task.TaskExecutor;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.security.task.DelegatingSecurityContextTaskExecutor;

import java.util.concurrent.ThreadPoolExecutor;

/**
 * <p>
 * ThreadPoolUtil
 * </p>
 *
 * @author WendyYang
 * @since 2023/10/27
 **/
@Slf4j
public class ThreadPoolUtil {

    private ThreadPoolUtil() {
    }

    public static final TaskExecutor REQUEST;
    public static final ThreadPoolTaskScheduler SCHEDULER;

    static {
        StopWatch watch = new StopWatch();
        watch.start();
        ThreadPoolUtilProperties properties = SpringUtil.getBean(ThreadPoolUtilProperties.class);
        ThreadPoolProperties requestProps = properties.getRequest();
        ThreadPoolTaskExecutor requestExecutor = new MDCThreadPoolTaskExecutor();
        requestExecutor.setCorePoolSize(requestProps.getCorePoolSize());
        requestExecutor.setMaxPoolSize(requestProps.getMaxPoolSize());
        requestExecutor.setQueueCapacity(requestProps.getQueueCapacity());
        requestExecutor.setThreadPriority(Thread.NORM_PRIORITY);
        requestExecutor.setAllowCoreThreadTimeOut(requestProps.getAllowCoreThreadTimeOut());
        requestExecutor.setWaitForTasksToCompleteOnShutdown(requestProps.getWaitForTasksToCompleteOnShutdown());
        requestExecutor.setKeepAliveSeconds(requestProps.getKeepAliveSeconds());
        requestExecutor.setThreadNamePrefix(requestProps.getThreadNamePrefix());
        requestExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        requestExecutor.initialize();
        REQUEST = new DelegatingSecurityContextTaskExecutor(requestExecutor);
        ThreadPoolProperties schedulerProps = properties.getScheduler();
        SCHEDULER = new MDCThreadPoolTaskScheduler();
        SCHEDULER.setThreadNamePrefix(schedulerProps.getThreadNamePrefix());
        SCHEDULER.setPoolSize(schedulerProps.getCorePoolSize());
        SCHEDULER.setThreadPriority(Thread.NORM_PRIORITY - 1);
        SCHEDULER.setWaitForTasksToCompleteOnShutdown(schedulerProps.getWaitForTasksToCompleteOnShutdown());
        SCHEDULER.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        SCHEDULER.initialize();
        watch.stop();
        log.info("线程池初始化耗时: {} ms", watch.getTotalTimeMillis());
    }

}
