package com.hz.pm.api.external.controller;

import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.MhFileClient;
import com.hz.pm.api.external.model.dto.MhDictDTO;
import com.hz.pm.api.external.model.dto.MhFileInfoDTO;
import com.hz.pm.api.external.model.dto.MhXcfhxReportListDTO;
import com.hz.pm.api.external.model.enumeration.MhDictType;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * MhDictController
 * </p>
 *
 * @author WendyYang
 * @since 10:28 2024/3/26
 */
@RestController
@Api(tags = "信产接口")
@RequiredArgsConstructor
@RequestMapping("/api/v1/mh")
public class MhApiController {

    private final MhApiClient mhApiClient;
    private final MhFileClient mhFileClient;

    @GetMapping("/dict/listByType/{dictType}")
    @ApiOperation("根据字典类型获取字典信息")
    public List<MhDictDTO> dictListByType(@PathVariable MhDictType dictType) {
        return mhApiClient.dictListByType(dictType);
    }

    @GetMapping("/file/download/{fileId}")
    @WebLog("下载信创平台文件")
    @ApiOperation("下载信创平台文件")
    public void download(@PathVariable String fileId, HttpServletResponse response) {
        mhFileClient.download(fileId, response);
    }

    @GetMapping("/anonymous/file/download/{fileId}")
    @WebLog("下载信创平台文件")
    @ApiOperation("下载信创平台文件")
    public void anonymousDownload(@PathVariable String fileId, HttpServletResponse response) {
        mhFileClient.download(fileId, response);
    }

    @GetMapping("/file/listFileInfo")
    @ApiOperation("获取文件详情")
    public List<MhFileInfoDTO> listFileInfo(@RequestParam String fileIds) {
        return mhFileClient.listFileInfo(fileIds);
    }

    @GetMapping("/pageXcfhxReport")
    @ApiOperation("信创符合性测评报告列表")
    public List<MhXcfhxReportListDTO> pageXcfhxReport() {
        return mhApiClient.pageXcfhxTestReport();
    }

}


