package com.hz.pm.api.external.controller;

import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.manage.SyncMhUserOrgManage;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.wflow.bean.entity.WflowModels;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * <p>
 * MhSyncController
 * </p>
 *
 * @author WendyYang
 * @since 09:56 2023/12/22
 */
@Api(hidden = true, tags = "信产数据同步")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/mh/sync")
public class MhSyncController {

    private final SyncMhUserOrgManage syncMhUserOrgManage;
    private final ProcessModelManage processModelManage;

    @GetMapping("/wflow/{type}")
    @ApiOperation("获取当前用户的流程模型")
    public WflowModels syncWflow(@PathVariable Integer type) {
        return processModelManage.getWflowModelsNoNull(ProjectProcessType.getNoNull(type),
                LoginUserUtil.userDetail().getMhUnitId());
    }

    @GetMapping("/users")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    @ApiOperation("同步用户信息")
    public void getUsers(@RequestParam(value = "syncTime", required = false)
                         @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") LocalDateTime syncTime) {
        syncMhUserOrgManage.syncUsers(syncTime);
    }

    @GetMapping("/projects")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    @ApiOperation("同步项目信息")
    public void getUsers() {
        syncMhUserOrgManage.syncMhProjects();
    }

    @GetMapping("/units")
    @ApiOperation("同步单位信息")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    public void getUnits() {
        syncMhUserOrgManage.syncUnits();
    }

    @GetMapping("/experts")
    @ApiOperation("同步专家信息")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    public void getExperts(@RequestParam(value = "syncTime", required = false)
                           @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") LocalDateTime syncTime) {
        syncMhUserOrgManage.syncExperts(syncTime);
    }

    @GetMapping("/company")
    @ApiOperation("同步企业信息")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    public void getExperts() {
        syncMhUserOrgManage.syncCompany();
    }

}
