package com.hz.pm.api.meeting.entity.enumeration;

import cn.hutool.core.util.StrUtil;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * 会议评审类型
 * </p>
 *
 * @author WendyYang
 * @since 16:32 2023/3/15
 */
@Getter
@AllArgsConstructor
public enum MeetingReviewTypeEnum {

    PROJECT_REVIEW("项目评审", "6"),

    EXPERT_REVIEW("专家论证", "7"),

    OTHER("其他会议", "99");

    private final String value;
    private final String code;

    public boolean eq(String code) {
        return this.getCode().equals(code);
    }

    public static MeetingReviewTypeEnum getNoNull(String code) {
        return get(code).orElseThrow(() -> new IllegalArgumentException("无效的会议评审类型编码"));
    }

    public static Optional<MeetingReviewTypeEnum> get(String code) {
        return Arrays.stream(values())
                .filter(w -> w.getCode().equals(code))
                .findFirst();
    }

    public static String getValue(String code) {
        return get(code)
                .flatMap(w -> Optional.of(w.getValue()))
                .orElse(StrUtil.EMPTY);
    }

}
