package com.hz.pm.api.open.controller;

import com.hz.pm.api.open.manage.OpenApiFileManage;
import com.hz.pm.api.open.model.po.StrFileIdListReq;
import com.hz.pm.api.open.model.po.StrFileIdReq;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.file.entity.vo.result.AttachFileVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * OpenApiFileController
 * </p>
 *
 * @author WendyYang
 * @since 10:44 2024/12/9
 */
@Api(tags = "文件管理（供信产使用）")
@RestController
@RequestMapping("/open/api/file")
@RequiredArgsConstructor
public class OpenApiFileController {

    private final OpenApiFileManage openApiFileManage;

    @GetMapping("/initStrId")
    @ApiOperation(hidden = true, value = "初始化文件strId")
    public void initStrId() {
        Assert.isTrue(LoginUserUtil.isLogin(), "用户未登录");
        openApiFileManage.initStrId();
    }

    @PostMapping("/listFileInfo")
    @ApiOperation("获取文件基本信息")
    public List<AttachFileVo> listFileBasicInfo(@Valid @RequestBody StrFileIdListReq req) {
        return openApiFileManage.listFileInfo(req);
    }

    @PostMapping("/download")
    @ApiOperation("文件下载")
    public void download(@Valid @RequestBody StrFileIdReq req, HttpServletResponse response) throws Exception {
        openApiFileManage.download(req, response);
    }

}
