package com.hz.pm.api.open.controller;


import com.hz.pm.api.open.manage.OpenApiMeetingExpertManage;
import com.hz.pm.api.open.model.po.PageExpertJudgePO;
import com.hz.pm.api.open.model.po.ReqMeetingExpertInfoPO;
import com.hz.pm.api.open.model.po.ReqMeetingExpertSignPO;
import com.hz.pm.api.open.model.vo.ExpertJudgeToMhVO;
import com.hz.pm.api.open.model.vo.MeetingExpertToMhDTO;
import com.ningdatech.basic.model.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 会议专家信息外部调用接口
 * </p>
 *
 * @author CMM
 */
@Slf4j
@Validated
@Api(tags = "专家会议管理（供信产使用）")
@RestController
@RequestMapping("/open/api/meeting-expert-info")
@RequiredArgsConstructor
public class OpenApiMeetingController {

    private final OpenApiMeetingExpertManage openMeetingExpertManage;

    @PostMapping("/list")
    @ApiOperation("获取已抽取完成会议确定参加会议且未签到的专家信息列表")
    public List<MeetingExpertToMhDTO> infoList(@Valid @RequestBody ReqMeetingExpertInfoPO po) {
        return openMeetingExpertManage.infoList(po);
    }

    @PostMapping("/sign")
    @ApiOperation("获取已抽取完成会议确定参加会议且未签到的专家信息列表")
    public String sign(@Valid @RequestBody ReqMeetingExpertSignPO po) {
        return openMeetingExpertManage.sign(po);
    }

    @PostMapping("/pageExpertJudges")
    @ApiOperation("获取专家评价列表")
    public PageVo<ExpertJudgeToMhVO> pageExpertJudges(@RequestBody @Valid PageExpertJudgePO po) {
        return openMeetingExpertManage.pageExpertJudges(po);
    }

}
