package com.hz.pm.api.open.manage;

import cn.hutool.core.lang.UUID;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.open.model.po.StrFileIdListReq;
import com.hz.pm.api.open.model.po.StrFileIdReq;
import com.hz.pm.api.open.util.OpenApiCheckSignUtil;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.entity.vo.result.AttachFileVo;
import com.ningdatech.file.service.FileService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * OpenApiFileManage
 * </p>
 *
 * @author WendyYang
 * @since 10:46 2024/12/9
 */
@Component
@RequiredArgsConstructor
public class OpenApiFileManage {

    private final FileService fileService;
    private final OpenApiCheckSignUtil openApiCheckSignUtil;

    public void initStrId() {
        List<File> files = fileService.list(Wrappers.lambdaQuery(File.class)
                .select(File::getId, File::getStrId));
        files.forEach(w -> w.setStrId(UUID.randomUUID().toString(true)));
        fileService.updateBatchById(files);
    }

    public List<AttachFileVo> listFileInfo(StrFileIdListReq req) {
        openApiCheckSignUtil.checkSign(req.getKey());
        return fileService.getByStrIds(req.getStrFileIds());
    }

    public void download(StrFileIdReq req, HttpServletResponse response) throws Exception {
        openApiCheckSignUtil.checkSign(req.getKey());
        fileService.download(req.getStrFileId(), response);
    }

}
