package com.hz.pm.api.open.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.expert.entity.ExpertUserFullInfo;
import com.hz.pm.api.expert.model.dto.DictionaryVO;
import com.hz.pm.api.expert.service.IExpertUserFullInfoService;
import com.hz.pm.api.meeting.entity.domain.Meeting;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.entity.domain.MeetingExpertJudge;
import com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.hz.pm.api.meeting.entity.enumeration.MeetingStatusEnum;
import com.hz.pm.api.meeting.service.IMeetingExpertJudgeService;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.hz.pm.api.meeting.service.IMeetingService;
import com.hz.pm.api.meta.constant.ExpertDictTypeEnum;
import com.hz.pm.api.meta.helper.DictionaryCache;
import com.hz.pm.api.meta.model.dto.DictionaryDTO;
import com.hz.pm.api.meta.model.entity.ExpertDictionary;
import com.hz.pm.api.meta.service.IExpertDictionaryService;
import com.hz.pm.api.open.model.po.PageExpertJudgePO;
import com.hz.pm.api.open.model.po.ReqMeetingExpertInfoPO;
import com.hz.pm.api.open.model.po.ReqMeetingExpertSignPO;
import com.hz.pm.api.open.model.vo.ExpertInfoVO;
import com.hz.pm.api.open.model.vo.ExpertJudgeToMhVO;
import com.hz.pm.api.open.model.vo.MeetingExpertToMhDTO;
import com.hz.pm.api.open.util.OpenApiCheckSignUtil;
import com.hz.pm.api.user.model.entity.UserInfo;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.service.IUserInfoService;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author CMM
 * @since 2024/05/10 17:18
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class OpenApiMeetingExpertManage {


    private final OpenApiCheckSignUtil openApiCheckSignUtil;
    private final IMeetingService meetingService;
    private final IMeetingExpertService meetingExpertService;
    private final UserInfoHelper userInfoHelper;
    private final IUserInfoService userInfoService;
    private final IExpertDictionaryService iExpertDictionaryService;
    private final DictionaryCache dictionaryCache;
    private final IExpertUserFullInfoService expertUserFullInfoService;
    private final IMeetingExpertJudgeService meetingExpertJudgeService;

    public List<MeetingExpertToMhDTO> infoList(ReqMeetingExpertInfoPO po) {
        String key = po.getKey();
        // 校验传入的签名
        openApiCheckSignUtil.checkSign(key);
        // 获取已经抽取(指定)完成 确认参加会议并且未签到的专家信息
        List<Meeting> meetingList = meetingService.list(Wrappers.lambdaQuery(Meeting.class)
                .eq(Meeting::getConfirmedRoster, Boolean.TRUE)
                .ne(Meeting::getStatus, MeetingStatusEnum.CANCEL.getCode()));
        if (CollUtil.isEmpty(meetingList)) {
            return Collections.emptyList();
        }
        List<MeetingExpertToMhDTO> result = Lists.newArrayList();
        // 获取已抽取完成会议中同意参加但未签到的专家信息
        List<Long> meetingIds = meetingList.stream().map(Meeting::getId).collect(Collectors.toList());
        Map<Long, List<MeetingExpert>> meetingExpertMap = meetingExpertService.list(Wrappers.lambdaQuery(MeetingExpert.class)
                        .in(MeetingExpert::getMeetingId, meetingIds)
                        .eq(MeetingExpert::getConfirmedRoster, Boolean.TRUE)
                        .eq(MeetingExpert::getStatus, ExpertAttendStatusEnum.AGREED.getCode())
                        .eq(MeetingExpert::getSignStatus, Boolean.FALSE))
                .stream().collect(Collectors.groupingBy(MeetingExpert::getMeetingId));
        Map<Long, Meeting> meetingMap = CollUtils.listToMap(meetingList, Meeting::getId);
        List<MeetingExpert> experts = Lists.newArrayList();
        for (Map.Entry<Long, List<MeetingExpert>> listEntry : meetingExpertMap.entrySet()) {
            List<MeetingExpert> value = listEntry.getValue();
            experts.addAll(value);
        }

        // 获取未签到的专家信息
        List<MeetingExpert> expertList = experts.stream().filter(m -> Boolean.FALSE.equals(m.getSignStatus())).collect(Collectors.toList());
        Set<Long> userIds = expertList.stream().map(MeetingExpert::getExpertId).collect(Collectors.toSet());
        List<UserInfo> userInfoList = userInfoService.list(Wrappers.lambdaQuery(UserInfo.class).in(UserInfo::getId, userIds));
        List<Long> userIdList = new ArrayList<>(userIds);
        List<ExpertUserFullInfo> expertUserFullInfos = expertUserFullInfoService.listByUserIds(userIdList);
        Map<Long, ExpertUserFullInfo> userFullInfoMap = MapUtil.newHashMap();
        if (CollUtil.isNotEmpty(expertUserFullInfos)) {
            userFullInfoMap = expertUserFullInfos.stream().collect(Collectors.toMap(ExpertUserFullInfo::getUserId, v -> v));
        }
        Set<String> employeeSet = Sets.newHashSet();
        userInfoList = userInfoList.stream().filter(u -> employeeSet.add(u.getEmployeeCode())).collect(Collectors.toList());
        List<UserFullInfoDTO> userFullInfos = userInfoHelper.getUserFullInfos(userInfoList);
        Set<Long> userIdSet = Sets.newHashSet();
        userFullInfos = userFullInfos.stream().filter(u -> userIdSet.add(u.getUserId())).collect(Collectors.toList());
        Map<Long, UserFullInfoDTO> infoMap = userFullInfos.stream().collect(Collectors.toMap(UserFullInfoDTO::getUserId, v -> v));

        List<ExpertDictionary> expertDictionaryList = iExpertDictionaryService.list(Wrappers.lambdaQuery(ExpertDictionary.class)
                .eq(ExpertDictionary::getExpertInfoField, ExpertDictTypeEnum.EXPERT_TYPE.getKey())
                .in(ExpertDictionary::getUserId, userIdSet));
        Map<Long, List<ExpertDictionary>> dictMap = expertDictionaryList.stream().collect(Collectors.groupingBy(ExpertDictionary::getUserId));

        for (Map.Entry<Long, List<MeetingExpert>> entry : meetingExpertMap.entrySet()) {
            MeetingExpertToMhDTO vo = new MeetingExpertToMhDTO();
            Long meetingId = entry.getKey();
            Meeting meeting = meetingMap.get(meetingId);
            if (Objects.nonNull(meeting)) {
                vo.setMeetingId(meeting.getId());
                vo.setStartTime(meeting.getStartTime());
                vo.setEndTime(meeting.getEndTime());
                vo.setMeetingName(meeting.getName());
                vo.setMeetingAddress(meeting.getMeetingAddress());
                vo.setConnecter(meeting.getConnecter());
                vo.setContact(meeting.getContact());
            }
            List<MeetingExpert> meetingExperts = entry.getValue();
            if (CollUtil.isNotEmpty(meetingExperts)) {
                // 获取未签到的专家信息
                List<MeetingExpert> expertUserList = meetingExperts.stream().filter(m -> Boolean.FALSE.equals(m.getSignStatus())).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(expertUserList)) {
                    Map<Long, ExpertUserFullInfo> finalUserFullInfoMap = userFullInfoMap;
                    List<ExpertInfoVO> expertInfoVOList = expertUserList.stream().map(m -> {
                        ExpertInfoVO expertInfoVO = new ExpertInfoVO();
                        expertInfoVO.setUserId(m.getExpertId());
                        expertInfoVO.setName(m.getExpertName());
                        expertInfoVO.setPhoneNo(m.getMobile());
                        UserFullInfoDTO userFullInfoDTO = infoMap.get(m.getExpertId());
                        if (Objects.nonNull(userFullInfoDTO)) {
                            expertInfoVO.setMhUnitId(userFullInfoDTO.getMhUnitId());
                            expertInfoVO.setMhUnitName(userFullInfoDTO.getMhUnitName());
                        }
                        ExpertUserFullInfo expertUserFullInfo = finalUserFullInfoMap.get(m.getExpertId());
                        if (Objects.nonNull(expertUserFullInfo)) {
                            expertInfoVO.setExpertId(expertUserFullInfo.getId());
                        }
                        List<ExpertDictionary> dictionaries = dictMap.get(m.getExpertId());
                        if (CollUtil.isNotEmpty(dictionaries)) {
                            List<DictionaryVO> dictionaryFieldInfos = dictionaries.stream().map(r -> {
                                DictionaryVO dictionary = new DictionaryVO();
                                dictionary.setDictionaryType(r.getExpertInfoField());
                                dictionary.setDictionaryCode(r.getDictionaryCode());
                                Optional<DictionaryDTO> dictionaryDTO = dictionaryCache.getByCode(r.getDictionaryCode());
                                dictionaryDTO.ifPresent(dictDTO -> dictionary.setDictionaryName(dictDTO.getName()));
                                return dictionary;
                            }).collect(Collectors.toList());
                            expertInfoVO.setExpertType(dictionaryFieldInfos);
                        }
                        return expertInfoVO;
                    }).collect(Collectors.toList());
                    vo.setExpertInfoList(expertInfoVOList);
                }
            }
            result.add(vo);
        }
        return result;
    }


    public String sign(ReqMeetingExpertSignPO po) {
        String key = po.getKey();
        // 校验传入的签名
        openApiCheckSignUtil.checkSign(key);
        Long meetingId = po.getMeetingId();
        Long userId = po.getUserId();
        LocalDateTime signTime = po.getSignTime();
        String signAddress = po.getSignAddress();
        // 获取会议专家信息
        MeetingExpert meetingExpert = meetingExpertService.getByMeetingIdAndExpertId(meetingId, userId);
        // 更新专家签到信息
        if (Objects.nonNull(meetingExpert)) {
            meetingExpert.setSignStatus(true);
            meetingExpert.setSignTime(signTime);
            meetingExpert.setSignAddress(signAddress);
            meetingExpertService.updateById(meetingExpert);
            return "success";
        }
        return "fail";
    }

    public PageVo<ExpertJudgeToMhVO> pageExpertJudges(PageExpertJudgePO po) {
        openApiCheckSignUtil.checkSign(po.getKey());
        LambdaQueryWrapper<MeetingExpertJudge> query = Wrappers.lambdaQuery(MeetingExpertJudge.class)
                .eq(po.getMeetingId() != null, MeetingExpertJudge::getMeetingId, po.getMeetingId())
                .orderByDesc(MeetingExpertJudge::getCreateOn);
        Map<Long, ExpertUserFullInfo> expertUserMap = new HashMap<>();
        if (StrUtil.isNotBlank(po.getMhExpertId())) {
            ExpertUserFullInfo expertUser = expertUserFullInfoService.getByMhExpertId(po.getMhExpertId());
            if (expertUser == null) {
                return PageVo.empty();
            }
            query.eq(MeetingExpertJudge::getExpertId, expertUser.getUserId());
            expertUserMap.put(expertUser.getUserId(), expertUser);
        }
        Page<MeetingExpertJudge> page = meetingExpertJudgeService.page(po.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        if (StrUtil.isBlank(po.getMhExpertId())) {
            List<Long> expertIds = CollUtils.fieldList(page.getRecords(), MeetingExpertJudge::getExpertId);
            List<ExpertUserFullInfo> expertUsers = expertUserFullInfoService.listByUserIds(expertIds);
            for (ExpertUserFullInfo expertUser : expertUsers) {
                expertUserMap.put(expertUser.getUserId(), expertUser);
            }
        }
        List<ExpertJudgeToMhVO> data = page.getRecords().stream().map(w -> {
            ExpertUserFullInfo expertUser = expertUserMap.get(w.getExpertId());
            return ExpertJudgeToMhVO.builder()
                    .expertJudgeId(w.getId())
                    .mhExpertId(expertUser.getMhExpertId())
                    .meetingId(w.getMeetingId())
                    .userId(w.getExpertId())
                    .attended(w.getAttended())
                    .performance(w.getPerformance())
                    .advised(w.getAdvised())
                    .leaveEarly(w.getLeaveEarly())
                    .brokeRule(w.getBrokeRule())
                    .brokeRuleContent(w.getBrokeRuleContent())
                    .createOn(w.getCreateOn())
                    .score(w.getScore())
                    .build();
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

}
