package com.hz.pm.api.open.util;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * <p>
 * OpenApiCheckSignUtil
 * </p>
 *
 * @author WendyYang
 * @since 10:45 2024/11/6
 */
@Slf4j
@Component
public class OpenApiCheckSignUtil {

    @Value("${hz-pm.interfaceKey}")
    private String interfaceKey;

    /**
     * 将字节数组转换为十六进制字符串
     *
     * @param bytes 字节数组
     * @return 十六进制字符串
     */
    private static String bytesToHexString(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            String hex = Integer.toHexString(b & 0xFF);
            if (hex.length() == 1) {
                sb.append("0");
            }
            sb.append(hex);
        }
        return sb.toString();
    }

    public void checkSign(String targetSign) {
        if (StrUtil.isBlank(targetSign)) {
            throw new BizException("签名错误");
        }
        StringBuilder sb = new StringBuilder();
        sb.append("key=").append(interfaceKey);
        // 使用SHA-256进行加密
        String sign = null;
        try {
            MessageDigest md = MessageDigest.getInstance("SHA-256");
            byte[] bytes = md.digest(sb.toString().getBytes());
            sign = bytesToHexString(bytes);
        } catch (NoSuchAlgorithmException e) {
            log.error("签名校验错误：", e);
        }
        if (sign != null && sign.length() >= 8 && StrUtil.endWith(sign, targetSign)) {
            return;
        }
        throw new BizException("签名错误");
    }

}
