package com.hz.pm.api.performance.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.performance.enumration.PerformanceTemplateTypeEnum;
import com.hz.pm.api.performance.helper.TemplateDetailBuildHelper;
import com.hz.pm.api.performance.model.dto.PerformanceIndicatorAppIndexSaveDTO;
import com.hz.pm.api.performance.model.dto.PerformanceIndicatorProjectIndexSaveDTO;
import com.hz.pm.api.performance.model.dto.PerformanceIndicatorProjectTemplateSaveDTO;
import com.hz.pm.api.performance.model.dto.ProjectTemplateDetailDTO;
import com.hz.pm.api.performance.model.entity.PerformanceAppraisalAppIndicator;
import com.hz.pm.api.performance.model.entity.PerformanceAppraisalProjectIndicator;
import com.hz.pm.api.performance.model.entity.PerformanceIndicatorProjectTemplate;
import com.hz.pm.api.performance.model.entity.PerformanceIndicatorProjectTemplateDetail;
import com.hz.pm.api.performance.model.req.PerformanceAppraisalListReq;
import com.hz.pm.api.performance.model.vo.AppIndexVO;
import com.hz.pm.api.performance.model.vo.ProjectIndexTemplateVO;
import com.hz.pm.api.performance.model.vo.ProjectIndexVO;
import com.hz.pm.api.performance.model.vo.ProjectTemplateDetailVO;
import com.hz.pm.api.performance.service.IPerformanceAppraisalAppIndicatorService;
import com.hz.pm.api.performance.service.IPerformanceAppraisalProjectIndicatorService;
import com.hz.pm.api.performance.service.IPerformanceIndicatorProjectTemplateDetailService;
import com.hz.pm.api.performance.service.IPerformanceIndicatorProjectTemplateService;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.StrPool;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname IndicatorTemplateManage
 * @Description
 * @Date 2023/6/26 14:15
 * @Author PoffyZhang
 */
@Slf4j
@Component
@AllArgsConstructor
public class IndicatorConfigManage {
    private final IPerformanceIndicatorProjectTemplateService indicatorProjectTemplateService;
    private final IPerformanceIndicatorProjectTemplateDetailService indicatorProjectTemplateDetailService;
    private final IPerformanceAppraisalProjectIndicatorService projectIndicatorService;
    private final IPerformanceAppraisalAppIndicatorService appIndicatorService;
    private final TemplateDetailBuildHelper templateDetailBuildHelper;

    /**
     * 模板列表
     *
     * @param req
     * @return
     */
    public PageVo<ProjectIndexTemplateVO> projectIndexTemplateList(PerformanceAppraisalListReq req) {
        Page<PerformanceIndicatorProjectTemplate> page = req.page();
        LambdaQueryWrapper<PerformanceIndicatorProjectTemplate> wrapper = Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplate.class)
                .like(StrUtils.isNotBlank(req.getTemplateName()), PerformanceIndicatorProjectTemplate::getName, req.getTemplateName())
                .orderByDesc(PerformanceIndicatorProjectTemplate::getCreateOn);
        indicatorProjectTemplateService.page(page, wrapper);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }

        List<ProjectIndexTemplateVO> res = page.getRecords().stream()
                .map(p -> BeanUtil.copyProperties(p, ProjectIndexTemplateVO.class))
                .collect(Collectors.toList());

        return PageVo.of(res, page.getTotal());
    }

    /**
     * 查询项目模板详情
     *
     * @param id \
     * @return \
     */
    public ProjectIndexTemplateVO projectDetail(Long id) {
        PerformanceIndicatorProjectTemplate template = indicatorProjectTemplateService.getById(id);
        if (Objects.isNull(template)) {
            return null;
        }
        ProjectIndexTemplateVO vo = BeanUtil.copyProperties(template,
                ProjectIndexTemplateVO.class);
        String projectTagIds = template.getProjectTagIds();
        if (StringUtils.isNotBlank(projectTagIds)) {
            vo.setProjectTagIds(BizUtils.splitToLong(projectTagIds));
        }
        // 获取模版绩效指标详情
        List<PerformanceIndicatorProjectTemplateDetail> templateDetails = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, id)
                .eq(PerformanceIndicatorProjectTemplateDetail::getIsAdditional, Boolean.FALSE));
        // 获取模版附加绩效指标详情
        List<PerformanceIndicatorProjectTemplateDetail> additionalTemplateDetails = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, id)
                .eq(PerformanceIndicatorProjectTemplateDetail::getIsAdditional, Boolean.TRUE));
        if (CollUtil.isNotEmpty(templateDetails)) {
            List<ProjectTemplateDetailVO> templateDetailVos = templateDetailBuildHelper.buildTemplateDetail(templateDetails, null, null);
            vo.setTemplateDetails(templateDetailVos);
        }

        if (CollUtil.isNotEmpty(additionalTemplateDetails)) {
            List<ProjectTemplateDetailVO> additionalTemplateDetailVos = templateDetailBuildHelper.buildTemplateDetail(additionalTemplateDetails, null, null);
            vo.setAdditionalIndexDetails(additionalTemplateDetailVos);
        }
        return vo;
    }

    /**
     * 项目指标 模板保存
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String projectTemplateSave(PerformanceIndicatorProjectTemplateSaveDTO dto) {
        UserInfoDetails user = LoginUserUtil.userDetail();
        PerformanceIndicatorProjectTemplate template = BeanUtil.copyProperties(dto, PerformanceIndicatorProjectTemplate.class);
        List<Long> projectTagIds = dto.getProjectTagIds();
        List<String> tagIdList = projectTagIds.stream().map(String::valueOf).collect(Collectors.toList());
        template.setProjectTagIds(String.join(StrPool.COMMA, tagIdList));
        if (Objects.isNull(dto.getStatus())) {
            // 默认模版禁用
            template.setStatus(0);
        }
        if (Objects.nonNull(dto.getId())) {
            PerformanceIndicatorProjectTemplate old = indicatorProjectTemplateService.getById(dto.getId());
            Assert.notNull(old, "该模板不存在!");
            template.setId(old.getId());
        } else {
            template.setId(null);
            template.setCreateOn(LocalDateTime.now());
            template.setCreateBy(user.getUsername());
        }
        template.setUpdateOn(LocalDateTime.now());
        template.setUpdateBy(user.getUsername());
        if (indicatorProjectTemplateService.saveOrUpdate(template)) {
            //保存 模板指标的详情
            if (CollUtil.isNotEmpty(dto.getTemplateDetails())) {
                //先删除
                indicatorProjectTemplateDetailService.remove(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                        .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, dto.getId())
                        .eq(PerformanceIndicatorProjectTemplateDetail::getIsAdditional, Boolean.FALSE));
                analysisTemplateDetails(dto.getTemplateDetails(), template.getId());
                //校检 所有3级指标（不包括附加指标） 总分是不是100
                List<PerformanceIndicatorProjectTemplateDetail> indexList = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                        .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, template.getId())
                        .eq(PerformanceIndicatorProjectTemplateDetail::getIsAdditional, Boolean.FALSE)
                        .eq(PerformanceIndicatorProjectTemplateDetail::getType, PerformanceTemplateTypeEnum.THIRD_INDEX.getCode()));
                if (CollUtil.isNotEmpty(indexList)) {
                    BigDecimal sum = indexList.stream()
                            .map(PerformanceIndicatorProjectTemplateDetail::getIndexScore)
                            .filter(Objects::nonNull)
                            .reduce(BigDecimal.ZERO, BigDecimal::add);
                    boolean score = (BigDecimal.valueOf(100)).equals(sum);
                    VUtils.isTrue(Boolean.FALSE.equals(score)).throwMessage("总分不是100 操作失败!");
                }
            }
            // 保存 模版附加指标详情
            if (CollUtil.isNotEmpty(dto.getAdditionalIndexDetails())) {
                indicatorProjectTemplateDetailService.remove(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                        .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, dto.getId())
                        .eq(PerformanceIndicatorProjectTemplateDetail::getIsAdditional, Boolean.TRUE));
                analysisAdditionalTemplateDetails(dto.getAdditionalIndexDetails(), template.getId());
            }
            return BizConst.SAVE_SUCCESS;
        }
        throw new BizException(BizConst.SAVE_FAIL);
    }

    /**
     * 解析 项目考核指标
     *
     * @param templateDetails
     * @return
     */
    public void analysisTemplateDetails(
            List<ProjectTemplateDetailDTO> templateDetails, Long templateId) {
        if (CollUtil.isEmpty(templateDetails)) {
            return;
        }
        analysisTemplateDetailsInner(templateDetails, templateId, null);
    }

    /**
     * 解析 项目附加考核指标
     *
     * @param additionalTemplateDetails
     * @return
     */
    public void analysisAdditionalTemplateDetails(
            List<ProjectTemplateDetailDTO> additionalTemplateDetails, Long templateId) {
        if (CollUtil.isEmpty(additionalTemplateDetails)) {
            return;
        }
        analysisAdditionalTemplateDetailsInner(additionalTemplateDetails, templateId, null);
    }

    public void analysisTemplateDetailsInner(List<ProjectTemplateDetailDTO> templateDetails,
                                             Long templateId, Long parentId) {
        if (CollUtil.isEmpty(templateDetails)) {
            return;
        }
        UserInfoDetails user = LoginUserUtil.userDetail();
        for (ProjectTemplateDetailDTO detail : templateDetails) {
            //计算阶段的
            PerformanceIndicatorProjectTemplateDetail templateDetail = BeanUtil.copyProperties(detail, PerformanceIndicatorProjectTemplateDetail.class);
            templateDetail.setCreateBy(user.getUsername());
            templateDetail.setCreateOn(LocalDateTime.now());
            templateDetail.setUpdateBy(user.getUsername());
            templateDetail.setUpdateOn(LocalDateTime.now());
            templateDetail.setTemplateId(templateId);
            // 一级指标的父ID为null
            templateDetail.setParentId(parentId);
            templateDetail.setIsAdditional(Boolean.FALSE);

            indicatorProjectTemplateDetailService.save(templateDetail);

            if (CollUtil.isNotEmpty(detail.getChildren())) {
                analysisTemplateDetailsInner(detail.getChildren(), templateId, templateDetail.getId());
            }
        }
    }

    public void analysisAdditionalTemplateDetailsInner(List<ProjectTemplateDetailDTO> templateDetails,
                                                       Long templateId, Long parentId) {
        if (CollUtil.isEmpty(templateDetails)) {
            return;
        }
        UserInfoDetails user = LoginUserUtil.userDetail();
        for (ProjectTemplateDetailDTO detail : templateDetails) {
            //计算阶段的
            PerformanceIndicatorProjectTemplateDetail templateDetail = BeanUtil.copyProperties(detail, PerformanceIndicatorProjectTemplateDetail.class);
            templateDetail.setCreateBy(user.getUsername());
            templateDetail.setCreateOn(LocalDateTime.now());
            templateDetail.setUpdateBy(user.getUsername());
            templateDetail.setUpdateOn(LocalDateTime.now());
            templateDetail.setTemplateId(templateId);
            // 标记为附加指标
            templateDetail.setIsAdditional(Boolean.TRUE);
            // 一级指标的父ID为null
            templateDetail.setParentId(parentId);

            indicatorProjectTemplateDetailService.save(templateDetail);

            if (CollUtil.isNotEmpty(detail.getChildren())) {
                analysisAdditionalTemplateDetailsInner(detail.getChildren(), templateId, templateDetail.getId());
            }
        }
    }

    /**
     * 删除项目模板
     *
     * @param id
     * @return
     */
    public String projectTemplateDelete(Long id) {
        PerformanceIndicatorProjectTemplate projectTemplate = indicatorProjectTemplateService.getById(id);
        VUtils.isTrue(Objects.isNull(projectTemplate)).throwMessage("该模板不存在!");

        if (CommonEnum.YES.getCode().equals(projectTemplate.getStatus())) {
            throw new BizException("操作失败 该模板 是启用状态!");
        }

        if (indicatorProjectTemplateService.removeById(id)) {
            indicatorProjectTemplateDetailService
                    .remove(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                            .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, id));
            return "删除成功";
        }
        return "操作失败";
    }

    public Boolean enOrDisProjectIndexTemplate(Long templateId, boolean active) {
        PerformanceIndicatorProjectTemplate projectTemplate = indicatorProjectTemplateService.getById(templateId);
        if (Boolean.TRUE.equals(active)) {
            // 根据创建模板时选择的项目类型、预算年度、项目金额、项目标签
            // 判断模板是否可以启用(同一套组合只能启用一套模板)
            PerformanceIndicatorProjectTemplate template = indicatorProjectTemplateService.getOne(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplate.class)
                    .eq(PerformanceIndicatorProjectTemplate::getProjectType, projectTemplate.getProjectType())
                    .eq(PerformanceIndicatorProjectTemplate::getProjectYear, projectTemplate.getProjectYear())
                    .eq(PerformanceIndicatorProjectTemplate::getAmountRange, projectTemplate.getAmountRange())
                    .eq(PerformanceIndicatorProjectTemplate::getProjectTagIds, projectTemplate.getProjectTagIds())
                    .eq(PerformanceIndicatorProjectTemplate::getStatus, CommonEnum.YES.getCode()));
            if (Objects.nonNull(template)) {
                throw new BizException("相同配置的模板已存在,当前模板无法启用!");
            }
            projectTemplate.setStatus(CommonEnum.YES.getCode());
        } else {
            projectTemplate.setStatus(CommonEnum.NO.getCode());
        }
        return indicatorProjectTemplateService.updateById(projectTemplate);
    }

    @Transactional(rollbackFor = Exception.class)
    public String projectTemplateCopy(Long templateId) {
        UserInfoDetails user = LoginUserUtil.userDetail();
        PerformanceIndicatorProjectTemplate projectTemplate = indicatorProjectTemplateService.getById(templateId);
        PerformanceIndicatorProjectTemplate newTemplate = new PerformanceIndicatorProjectTemplate();
        BeanUtils.copyProperties(projectTemplate, newTemplate);
        // 复制后，项目模版名称变更为xxx副本，状态为禁用
        newTemplate.setName(projectTemplate.getName() + CommonConst.COPY);
        newTemplate.setStatus(CommonEnum.NO.getCode());
        newTemplate.setCreateOn(LocalDateTime.now());
        newTemplate.setCreateBy(user.getUsername());
        newTemplate.setUpdateOn(LocalDateTime.now());
        newTemplate.setUpdateBy(user.getUsername());
        // 副本ID要置空
        newTemplate.setId(null);
        if (indicatorProjectTemplateService.save(newTemplate)) {
            // 新增指标详情
            List<PerformanceIndicatorProjectTemplateDetail> projectTemplateDetails = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                    .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, templateId));
            Map<Long, PerformanceIndicatorProjectTemplateDetail> detailMap = projectTemplateDetails.stream().collect(Collectors.toMap(PerformanceIndicatorProjectTemplateDetail::getId, p -> p));
            // 按层级逐个新增并保存各指标
            // 从三级指标开始
            List<PerformanceIndicatorProjectTemplateDetail> thirdList = projectTemplateDetails.stream()
                    .filter(p -> PerformanceTemplateTypeEnum.THIRD_INDEX.getCode().equals(p.getType()))
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(thirdList)) {
                throw new BizException("当前模板未配置三级指!");
            }
            List<PerformanceIndicatorProjectTemplateDetail> templateDetails = Lists.newArrayList();
            for (PerformanceIndicatorProjectTemplateDetail templateDetail : thirdList) {
                PerformanceIndicatorProjectTemplateDetail newThirdTemplateDetail = new PerformanceIndicatorProjectTemplateDetail();
                BeanUtils.copyProperties(templateDetail, newThirdTemplateDetail);
                newThirdTemplateDetail.setTemplateId(newTemplate.getId());
                newThirdTemplateDetail.setCreateOn(LocalDateTime.now());
                newThirdTemplateDetail.setCreateBy(user.getUsername());
                newThirdTemplateDetail.setUpdateOn(LocalDateTime.now());
                newThirdTemplateDetail.setUpdateBy(user.getUsername());
                // 新的指标ID、parentId要置空
                newThirdTemplateDetail.setId(null);
                newThirdTemplateDetail.setParentId(null);
                indicatorProjectTemplateDetailService.save(newThirdTemplateDetail);
                // 再保存关联的二级指标
                Long thirdParentId = templateDetail.getParentId();
                PerformanceIndicatorProjectTemplateDetail secondTemplateDetail = detailMap.get(thirdParentId);
                if (Objects.isNull(secondTemplateDetail)) {
                    throw new BizException("三级指标：" + templateDetail.getName() + "对应的二级指标不存在");
                }
                PerformanceIndicatorProjectTemplateDetail newSecondTemplateDetail = new PerformanceIndicatorProjectTemplateDetail();
                BeanUtils.copyProperties(secondTemplateDetail, newSecondTemplateDetail);
                newSecondTemplateDetail.setTemplateId(newTemplate.getId());
                newSecondTemplateDetail.setCreateOn(LocalDateTime.now());
                newSecondTemplateDetail.setCreateBy(user.getUsername());
                newSecondTemplateDetail.setUpdateOn(LocalDateTime.now());
                newSecondTemplateDetail.setUpdateBy(user.getUsername());
                // 新的指标Id、parentId要置空
                newSecondTemplateDetail.setId(null);
                newSecondTemplateDetail.setParentId(null);
                indicatorProjectTemplateDetailService.save(newSecondTemplateDetail);
                // 关联三级指标
                newThirdTemplateDetail.setParentId(newSecondTemplateDetail.getId());
                templateDetails.add(newThirdTemplateDetail);
                // 再保存关联的一级指标
                Long secondParentId = secondTemplateDetail.getParentId();
                PerformanceIndicatorProjectTemplateDetail firstTemplateDetail = detailMap.get(secondParentId);
                if (Objects.isNull(firstTemplateDetail)) {
                    throw new BizException("二级指标：" + secondTemplateDetail.getName() + "对应的一级指标不存在");
                }
                PerformanceIndicatorProjectTemplateDetail newFirstTemplateDetail = new PerformanceIndicatorProjectTemplateDetail();
                BeanUtils.copyProperties(firstTemplateDetail, newFirstTemplateDetail);
                newFirstTemplateDetail.setTemplateId(newTemplate.getId());
                newFirstTemplateDetail.setCreateOn(LocalDateTime.now());
                newFirstTemplateDetail.setCreateBy(user.getUsername());
                newFirstTemplateDetail.setUpdateOn(LocalDateTime.now());
                newFirstTemplateDetail.setUpdateBy(user.getUsername());
                // 新的指标ID、parentId要置空
                newFirstTemplateDetail.setId(null);
                newFirstTemplateDetail.setParentId(null);
                indicatorProjectTemplateDetailService.save(newFirstTemplateDetail);
                // 关联二级指标
                newSecondTemplateDetail.setParentId(newFirstTemplateDetail.getId());
                templateDetails.add(newSecondTemplateDetail);
            }
            // 批量更新
            if (indicatorProjectTemplateDetailService.updateBatchById(templateDetails)) {
                return "模版复制成功";
            }
        }
        return "模版复制失败";
    }

    public PageVo<ProjectIndexVO> projectIndexList(PerformanceAppraisalListReq req) {
        Page<PerformanceAppraisalProjectIndicator> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisalProjectIndicator> wrapper = Wrappers.lambdaQuery(PerformanceAppraisalProjectIndicator.class)
                .like(StringUtils.isNotBlank(req.getIndexName()), PerformanceAppraisalProjectIndicator::getIndexName, req.getIndexName())
                .eq(Objects.nonNull(req.getIndexLevel()), PerformanceAppraisalProjectIndicator::getIndexLevel, req.getIndexLevel())
                .eq(StringUtils.isNotBlank(req.getRegionCode()), PerformanceAppraisalProjectIndicator::getRegionCode, req.getRegionCode())
                .orderByDesc(PerformanceAppraisalProjectIndicator::getCreateOn);
        projectIndicatorService.page(page, wrapper);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        List<ProjectIndexVO> result = page.getRecords().stream()
                .map(i -> BeanUtil.copyProperties(i, ProjectIndexVO.class))
                .collect(Collectors.toList());
        return PageVo.of(result, page.getTotal());
    }

    public String projectIndexSave(PerformanceIndicatorProjectIndexSaveDTO dto) {
        UserInfoDetails user = LoginUserUtil.userDetail();
        PerformanceAppraisalProjectIndicator projectIndicator = BeanUtil.copyProperties(dto, PerformanceAppraisalProjectIndicator.class);
        if (Objects.nonNull(dto.getId())) {
            PerformanceAppraisalProjectIndicator old = projectIndicatorService.getById(dto.getId());
            Assert.notNull(old, "该项目指标不存在!");
            projectIndicator.setId(old.getId());
        } else {
            projectIndicator.setId(null);
            projectIndicator.setCreateOn(LocalDateTime.now());
            projectIndicator.setCreateBy(user.getUsername());
        }
        projectIndicator.setUpdateOn(LocalDateTime.now());
        projectIndicator.setUpdateBy(user.getUsername());
        if (projectIndicatorService.saveOrUpdate(projectIndicator)) {
            return "指标新增成功";
        }
        return "指标新增失败";
    }

    public String projectIndexDelete(Long id) {
        PerformanceAppraisalProjectIndicator projectIndicator = projectIndicatorService.getById(id);
        VUtils.isTrue(Objects.isNull(projectIndicator)).throwMessage("该项目指标不存在!");
        if (projectIndicatorService.removeById(id)) {
            return "删除成功";
        }
        return "操作失败";
    }

    public ProjectIndexVO projectIndexDetail(Long id) {
        PerformanceAppraisalProjectIndicator projectIndex = projectIndicatorService.getById(id);
        if (Objects.isNull(projectIndex)) {
            return null;
        }
        return BeanUtil.copyProperties(projectIndex, ProjectIndexVO.class);
    }

    public PageVo<AppIndexVO> appIndexList(PerformanceAppraisalListReq req) {
        Page<PerformanceAppraisalAppIndicator> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisalAppIndicator> wrapper = Wrappers.lambdaQuery(PerformanceAppraisalAppIndicator.class)
                .like(StringUtils.isNotBlank(req.getIndexName()), PerformanceAppraisalAppIndicator::getIndexName, req.getIndexName())
                .orderBy(Boolean.FALSE, Boolean.TRUE, PerformanceAppraisalAppIndicator::getSort);
        appIndicatorService.page(page, wrapper);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        List<AppIndexVO> result = page.getRecords().stream()
                .map(i -> BeanUtil.copyProperties(i, AppIndexVO.class))
                .collect(Collectors.toList());
        return PageVo.of(result, page.getTotal());
    }

    public String appIndexSave(PerformanceIndicatorAppIndexSaveDTO dto) {
        UserInfoDetails user = LoginUserUtil.userDetail();
        PerformanceAppraisalAppIndicator appIndicator = BeanUtil.copyProperties(dto, PerformanceAppraisalAppIndicator.class);
        if (Objects.nonNull(dto.getId())) {
            PerformanceAppraisalAppIndicator old = appIndicatorService.getById(dto.getId());
            Assert.notNull(old, "该项目指标不存在!");
            appIndicator.setId(old.getId());
        } else {
            appIndicator.setId(null);
            appIndicator.setCreateOn(LocalDateTime.now());
            appIndicator.setCreateBy(user.getUsername());
        }
        appIndicator.setUpdateOn(LocalDateTime.now());
        appIndicator.setUpdateBy(user.getUsername());
        if (appIndicatorService.saveOrUpdate(appIndicator)) {
            return "指标新增成功";
        }
        return "指标新增失败";
    }

    public String appIndexDelete(Long id) {
        PerformanceAppraisalAppIndicator appIndicator = appIndicatorService.getById(id);
        VUtils.isTrue(Objects.isNull(appIndicator)).throwMessage("该应用指标不存在!");
        if (appIndicatorService.removeById(id)) {
            return "删除成功";
        }
        return "操作失败";
    }

    public AppIndexVO appIndexDetail(Long id) {
        PerformanceAppraisalAppIndicator appIndex = appIndicatorService.getById(id);
        if (Objects.isNull(appIndex)) {
            return null;
        }
        return BeanUtil.copyProperties(appIndex, AppIndexVO.class);
    }

    public Boolean enOrDisAppIndex(Long appId, Boolean state) {
        PerformanceAppraisalAppIndicator appIndicator = appIndicatorService.getById(appId);
        if (Boolean.TRUE.equals(state)) {
            appIndicator.setIsDisplay(Boolean.TRUE);
        } else {
            appIndicator.setIsDisplay(Boolean.FALSE);
        }
        return appIndicatorService.updateById(appIndicator);
    }
}
