package com.hz.pm.api.projectdeclared.chain;

import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.projectdeclared.chain.handler.FinalInspectedPassedHandler;
import com.hz.pm.api.projectdeclared.chain.handler.PurchaseOrgConfirmHandler;
import com.hz.pm.api.projectdeclared.chain.handler.SubmitFinalInspectedHandler;
import com.hz.pm.api.projectdeclared.chain.handler.SubmitFirstInspectedHandler;
import com.hz.pm.api.projectdeclared.chain.request.ProjectStatusRewriteRequest;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * ProjectStatusRewriteHandlerContext
 * </p>
 *
 * @author WendyYang
 * @since 18:20 2024/10/22
 */
@Component
@RequiredArgsConstructor
public class ProjectStatusRewriteHandlerContext {

    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final IProjectService projectService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    public void process(Purchase purchase, Project project, List<Purchase> purchases) {
        if (!BidTypeEnum.BUILD_APP.eq(purchase.getBidType())) {
            List<Long> buildAddBidIds = CollUtils.filter(purchases, w -> BidTypeEnum.BUILD_APP.eq(w.getBidType()), Purchase::getId);
            if (!buildAddBidIds.isEmpty()) {
                List<PurchaseStatusChange> changes = purchaseStatusChangeService.listByProjectCode(project.getProjectCode(), buildAddBidIds);
                Map<Long, List<PurchaseStatusChange>> changeMap = CollUtils.group(changes, PurchaseStatusChange::getBidId);
                PurchaseOrgConfirmHandler firstHandler = getFirstHandler();
                ProjectStatusRewriteRequest context = new ProjectStatusRewriteRequest();
                context.setProjectStateMachineUtil(projectStateMachineUtil);
                context.setProject(project);
                context.setBuildAddBidIds(buildAddBidIds);
                context.setPurchaseStatusChangeMap(changeMap);
                Integer beforeStatus = project.getStatus();
                firstHandler.handle(context);
                if (!beforeStatus.equals(project.getStatus())) {
                    projectService.updateById(project);
                }
            }
        }
    }

    private static PurchaseOrgConfirmHandler getFirstHandler() {
        FinalInspectedPassedHandler finalInspectedPassedHandler = new FinalInspectedPassedHandler(null);
        SubmitFinalInspectedHandler submitFinalInspectedHandler = new SubmitFinalInspectedHandler(finalInspectedPassedHandler);
        SubmitFirstInspectedHandler submitFirstInspectedHandler = new SubmitFirstInspectedHandler(submitFinalInspectedHandler);
        return new PurchaseOrgConfirmHandler(submitFirstInspectedHandler);
    }

}
