package com.hz.pm.api.projectdeclared.controller;

import com.hz.pm.api.meeting.entity.dto.ProjectReviewResultDTO;
import com.hz.pm.api.projectdeclared.manage.ProjectReviewManage;
import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectdeclared.model.vo.ExpertReviewProgressVO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.UploadConstructionPlanSealFileReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * ProjectReviewController
 * </p>
 *
 * @author WendyYang
 * @since 10:52 2024/4/17
 */
@Api(tags = "项目评审")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/projectReview")
public class ProjectReviewController {

    private final ProjectReviewManage projectReviewManage;

    @ApiOperation("提交项目评审申请")
    @PostMapping(value = {"/submit", "resubmit"})
    @WebLog("提交项目评审申请")
    public void projectReviewApply(@RequestBody DefaultDeclaredDTO req) {
        projectReviewManage.projectReviewApply(req);
    }

    @ApiOperation("上传项目建设方案盖章文件")
    @WebLog("上传项目建设方案盖章文件")
    @PostMapping("/uploadConstructionPlanSealFile")
    public void uploadConstructionPlanSealFile(@Valid @RequestBody UploadConstructionPlanSealFileReq req) {
        projectReviewManage.uploadConstructionPlanSealFile(req);
    }

    @ApiOperation("项目评审详情")
    @GetMapping("/detail/{projectId}")
    public ProjectReviewResultDTO projectReviewDetail(@PathVariable Long projectId) {
        return projectReviewManage.projectReviewDetail(projectId);
    }

    @ApiOperation("项目评审列表")
    @GetMapping("/pageReviewProject")
    public PageVo<ProjectLibListItemVO> pageReviewProject(ProjectListReq req) {
        return projectReviewManage.pageReviewProject(req);
    }

    @GetMapping("/progressStatistics")
    @ApiOperation("评审项目进度统计")
    public ExpertReviewProgressVO progressStatistics(ProjectListReq req) {
        return projectReviewManage.reviewProjectProgress(req);
    }

    @ApiOperation("专家评审列表")
    @GetMapping("/pageExpertReviewProject")
    public PageVo<ProjectLibListItemVO> pageExpertReviewProject(ProjectListReq req) {
        return projectReviewManage.pageExpertReviewProject(req);
    }

    @GetMapping("/expertReviewProgressStatistics")
    @ApiOperation("专家评审项目进度统计")
    public ExpertReviewProgressVO expertProgressStatistics(ProjectListReq req) {
        return projectReviewManage.expertReviewProjectProgress(req);
    }

    @ApiOperation("修改后重新提交")
    @WebLog("修改后重新提交")
    @PostMapping("/resubmitByModify")
    public void resubmitByModify(@RequestBody DefaultDeclaredDTO req) {
        projectReviewManage.resubmitByModify(req);
    }

}
