package com.hz.pm.api.projectdeclared.helper;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.external.MhFileClient;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.ApiResponse;
import com.ningdatech.file.controller.FileController;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;

/**
 * @Classname MhXcFhxFileHelper
 * @Description
 * @Date 2024/7/3 10:41
 * @Author PoffyZhang
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class MhXcfhxReportHelper {

    private final MhFileClient mhFileClient;
    private final FileController fileController;

    public FileResultVO getXcfhxFile(String reportFile, String fileName) {
        if (StrUtils.isBlank(reportFile)) {
            throw BizException.wrap("信创报告文件不能为空");
        }
        File tmpFile = mhFileClient.downloadToTmpFile(reportFile);
        try (FileInputStream fis = new FileInputStream(tmpFile)) {
            String newFileName;
            if (StrUtil.isNotBlank(fileName)) {
                newFileName = fileName + FileUtil.getSuffix(tmpFile);
            } else {
                newFileName = tmpFile.getName();
            }
            String mimeType = FileUtil.getMimeType(tmpFile.getPath());
            MockMultipartFile multipartFile = new MockMultipartFile(newFileName, newFileName, mimeType, fis);
            ApiResponse<FileResultVO> retFileInfo = fileController.upload(multipartFile, "default");
            return retFileInfo.getData();
        } catch (IOException e) {
            log.error("信创报告上传失败", e);
            throw BizException.wrap("信创符合性测评报告上传失败");
        } finally {
            FileUtil.del(tmpFile);
        }
    }

    public void download(String reportFile, HttpServletResponse response) {
        if (StrUtils.isBlank(reportFile)) {
            throw BizException.wrap("信创报告文件不能为空");
        }

        File file = mhFileClient.downloadToTmpFile(reportFile);
        // 设置响应的内容类型和头信息
        response.setContentType("application/pdf");
        String fileName;
        try {
            fileName = URLEncoder.encode(file.getName(), "utf-8");
        } catch (UnsupportedEncodingException e) {
            log.warn("文件名编码失败：{}", file.getName(), e);
            fileName = UUID.randomUUID().toString(true) + ".pdf";
        }
        response.setHeader("Content-Disposition", "attachment;filename=\"" + fileName + "\"");
        // 使用ServletOutputStream写文件内容
        try (InputStream in = Files.newInputStream(file.toPath());
             ServletOutputStream out = response.getOutputStream()) {
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = in.read(buffer)) != -1) {
                out.write(buffer, 0, bytesRead);
            }
        } catch (IOException e) {
            log.error("导出信创报告失败", e);
            throw BizException.wrap("导出信创测评报告失败");
        }
    }
}
