package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.model.constant.MhUnitConst;
import com.hz.pm.api.common.model.constant.TypeReferenceConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.common.util.ExcelExportStyle;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.external.model.enumeration.MhUnitTypeEnum;
import com.hz.pm.api.projectdeclared.model.dto.*;
import com.hz.pm.api.projectdeclared.model.entity.ProjectDraft;
import com.hz.pm.api.projectdeclared.model.enumerization.ProjectDraftTypeEnum;
import com.hz.pm.api.projectdeclared.model.vo.DeclaredProgressVO;
import com.hz.pm.api.projectdeclared.model.vo.ProjectDraftVO;
import com.hz.pm.api.projectdeclared.service.IProjectDraftService;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectPaymentPlanDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectApplication;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.staging.enums.MsgTypeEnum;
import com.hz.pm.api.sys.manage.NoticeManage;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.todocenter.constant.WorkNoticeConst;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS;

/**
 * <p>
 * DeclaredProjectManage
 * </p>
 *
 * @author WendyYang
 * @since 16:24 2024/3/29
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class DeclaredProjectManage {

    private final IProjectService projectService;
    private final ProjectLibManage projectLibManage;
    private final IProjectDraftService projectDraftService;
    private final ProcessInstanceService processService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final NoticeManage noticeManage;
    private final UserInfoHelper userInfoHelper;
    private final MhUnitCache mhUnitCache;
    private final IProjectStatusChangeService projectStatusChangeService;

    /**
     * 新项目 启动实例
     *
     * @param req \
     */
    @Transactional(rollbackFor = Exception.class)
    public synchronized void store(DefaultDeclaredDTO req) {
        UserInfoDetails user = LoginUserUtil.userDetailNotNull();
        ProjectDTO reqProj = req.getProjectInfo();
        reqProj.setBuildOrgCode(user.getMhUnitIdStr());
        reqProj.setBuildOrgName(user.getMhUnitName());
        //项目名称去重
        declaredProjectHelper.projectNameDuplicateCheck(reqProj);
        //判断申报金额 是否等于总的 判断年度支付金额 是否等于总金额
        declaredProjectHelper.checkAmount(reqProj);
        //写死 是否有主管单位
        reqProj.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        reqProj.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.eq(reqProj.getIsSuperOrg())) {
            reqProj.setSuperOrgCode(user.getMhUnitIdStr());
            reqProj.setSuperOrg(user.getMhUnitName());
        }

        projectLibManage.saveProjectNewVersion(reqProj, user.getUserId(), ProjectStatus.TO_BE_DECLARED);
    }

    /**
     * 新项目 启动实例
     *
     * @param req \
     * @return \
     */
    @Transactional(rollbackFor = Exception.class)
    public synchronized String startTheProcess(DefaultDeclaredDTO req) {
        UserInfoDetails user = LoginUserUtil.userDetailNotNull();

        WflowModels model = processModelManage.getWflowModelsNoNull(ProjectProcessType.COMPLIANCE_REVIEW,
                user.getMhUnitId());

        ProjectDTO reqProj = req.getProjectInfo();
        if (reqProj.getIsTemporaryAugment() == null) {
            reqProj.setIsTemporaryAugment(1);
        }
        reqProj.setBuildOrgCode(user.getMhUnitIdStr());
        reqProj.setBuildOrgName(user.getMhUnitName());

        //项目名称去重
        declaredProjectHelper.projectNameDuplicateCheck(reqProj);
        //判断申报金额 是否等于总的 判断年度支付金额 是否等于总金额
        declaredProjectHelper.checkAmount(reqProj);

        //写死 是否有主管单位
        reqProj.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        reqProj.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.eq(reqProj.getIsSuperOrg())) {
            reqProj.setSuperOrgCode(user.getMhUnitIdStr());
            reqProj.setSuperOrg(user.getMhUnitName());
        }

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(user));
        params.setProcessUsers(Collections.emptyMap());
        //放入条件判断的项目字段
        ProjectConditionDTO conditionDto = new ProjectConditionDTO();
        BeanUtils.copyProperties(req.getProjectInfo(), conditionDto);
        req.getFormData().putAll(JSON.parseObject(JSON.toJSONString(conditionDto), TypeReferenceConst.STR_OBJ_MAP));
        params.setFormData(req.getFormData());
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Project project = new Project();
        BeanUtils.copyProperties(reqProj, project);
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instanceId = processService.startProcessLs(model, params, orgModelMap);
        log.info("申报项目成功 【{}】", instanceId);

        //如果是重新提交的话 判断下 项目是否存在
        //保存项目相关
        Project buildProject = projectLibManage.saveProjectInDeclared(reqProj, instanceId, user.getUserId());
        if (Objects.nonNull(reqProj.getDraftId())) {
            //如果是草稿箱提交 删除对应的草稿箱
            projectDraftService.removeById(reqProj.getDraftId());
        }

        //发送给第一个审批人消息
        noticeManage.sendFirstUser(buildProject, model.getFormName(), instanceId,
                WorkNoticeConst.PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);

        return instanceId;
    }


    /**
     * 重新提交 启动实例
     *
     * @param req \
     * @return \
     */
    @Transactional(rollbackFor = Exception.class)
    public synchronized String reStartTheProcess(DefaultDeclaredDTO req) {
        ProjectDTO reqProj = req.getProjectInfo();
        Assert.notNull(reqProj.getId(), "提交失败，缺少项目ID!");
        UserInfoDetails user = LoginUserUtil.userDetailNotNull();
        reqProj.setBuildOrgCode(user.getMhUnitIdStr());
        reqProj.setBuildOrgName(user.getMhUnitName());

        Project projectInfo = projectService.getNoNull(reqProj.getId());

        //项目名称去重
        if (StringUtils.isNotBlank(reqProj.getProjectName()) &&
                !reqProj.getProjectName().equals(projectInfo.getProjectName())) {
            reqProj.setProjectCode(projectInfo.getProjectCode());
            declaredProjectHelper.projectNameDuplicateCheck(reqProj);
        }

        //写死 是否有主管单位
        projectInfo.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        projectInfo.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.eq(reqProj.getIsSuperOrg())) {
            projectInfo.setSuperOrgCode(user.getMhUnitIdStr());
            projectInfo.setSuperOrg(user.getMhUnitName());
            reqProj.setSuperOrgCode(user.getMhUnitIdStr());
            reqProj.setSuperOrg(user.getMhUnitName());
        }

        //判断申报金额 是否等于总的 判断年度支付金额 是否等于总金额
        declaredProjectHelper.checkAmount(reqProj);

        WflowModels model = processModelManage.getWflowModelsNoNull(ProjectProcessType.COMPLIANCE_REVIEW,
                user.getMhUnitId());

        //首先要判断 项目当前状态 是不是 单位内部拒绝
        VUtils.isTrue(!ProjectStatus.COMPLIANCE_REVIEW_FAILED.eq(projectInfo.getStatus()))
                .throwMessage("提交失败 该项目不是 单位内部拒绝审核状态或者未立项阶段");

        ProcessStartParamsVo instParam = new ProcessStartParamsVo();
        instParam.setUser(declaredProjectHelper.buildUser(user));
        instParam.setProcessUsers(Collections.emptyMap());

        //把条件值给放入工作流
        declaredProjectHelper.buildCondition(instParam, req);

        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Project project = new Project();
        BeanUtils.copyProperties(reqProj, project);
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instId = processService.startProcessLs(model, instParam, orgModelMap);
        log.info("重新申报项目成功：{} ==> {}", project.getProjectCode(), instId);

        //保存项目相关
        Project newProj = projectLibManage.saveProjectInDeclared(reqProj, instId,
                user.getUserId(), ProjectStatus.NOT_APPROVED, ProjectStatus.COMPLIANCE_REVIEW_FAILED,
                ProjectProcessType.COMPLIANCE_REVIEW);

        //发送给第一个审批人消息
        noticeManage.sendFirstUser(newProj, model.getFormName(), instId,
                WorkNoticeConst.PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);

        return instId;
    }

    public PageVo<ProjectDraftVO> pageDraft(DeclaredProjectListParamDTO req, Integer draftType) {
        Long userId = LoginUserUtil.getUserId();
        Page<ProjectDraft> page = req.page();
        LambdaQueryWrapper<ProjectDraft> wrapper = Wrappers.lambdaQuery(ProjectDraft.class)
                .eq(ProjectDraft::getUserId, userId)
                .eq(ProjectDraft::getDraftType, draftType)
                .ge(Objects.nonNull(req.getStartTime()), ProjectDraft::getCreateOn, req.getStartTime())
                .le(Objects.nonNull(req.getEndTime()), ProjectDraft::getCreateOn, req.getEndTime())
                .eq(Objects.nonNull(req.getProjectType()), ProjectDraft::getProjectType, req.getProjectType())
                .eq(Objects.nonNull(req.getProjectYear()), ProjectDraft::getProjectYear, req.getProjectYear())
                .eq(Objects.nonNull(req.getProjectStage()), ProjectDraft::getStage, req.getProjectStage())
                .eq(Objects.nonNull(req.getProjectStatus()), ProjectDraft::getStatus, req.getProjectStatus())
                .eq(req.getUnitStrip() != null, ProjectDraft::getUnitStrip, req.getUnitStrip())
                .like(StringUtils.isNotBlank(req.getProjectName()), ProjectDraft::getProjectName, req.getProjectName())
                .orderByDesc(ProjectDraft::getUpdateOn);
        projectDraftService.page(page, wrapper);
        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        List<ProjectDraftVO> res = page.getRecords().stream().map(draft -> {
            ProjectDraftVO vo = new ProjectDraftVO();
            BeanUtils.copyProperties(draft, vo);
            vo.setBizDomain(Objects.nonNull(draft.getBizDomain()) ? String.valueOf(draft.getBizDomain()) : StringUtils.EMPTY);
            return vo;
        }).collect(Collectors.toList());
        return PageVo.of(res, page.getTotal());
    }

    public ProjectDraftVO draftDetail(Long id) {
        ProjectDraft draft = projectDraftService.getById(id);
        ProjectDraftVO vo = new ProjectDraftVO();
        BeanUtils.copyProperties(draft, vo);
        vo.setBizDomain(Objects.nonNull(draft.getBizDomain()) ? String.valueOf(draft.getBizDomain()) : StringUtils.EMPTY);
        if (StrUtil.isNotBlank(draft.getProjectApplicationList())) {
            vo.setApplicationList(JSON.parseArray(draft.getProjectApplicationList(), ProjectApplication.class));
        }
        if (StrUtil.isNotBlank(draft.getPaymentPlans())) {
            vo.setPaymentPlanList(JSONUtil.toList(draft.getPaymentPlans(), ProjectPaymentPlanDTO.class));
        }
        if (StrUtil.isNotBlank(draft.getDynamicForm())) {
            vo.setDynamicForm(JSON.parseObject(draft.getDynamicForm(), new TypeReference<Map<String, Object>>() {
            }));
        }
        if (StrUtil.isNotBlank(draft.getGovSystemReplaceInfos())) {
            vo.setSystemReplaceInfos(JSONUtil.toList(draft.getGovSystemReplaceInfos(), GovSystemReplaceInfoDTO.class));
        }
        return vo;
    }

    /**
     * 保存至草稿箱
     *
     * @param dto \
     * @return \
     */
    public synchronized Long saveToDraft(ProjectDraftSaveDTO dto, ProjectDraftTypeEnum draftType) {
        UserInfoDetails user = LoginUserUtil.userDetail();

        ProjectDTO projectInfo = dto.getProjectInfo();
        ProjectDraft draft = new ProjectDraft();
        BeanUtils.copyProperties(projectInfo, draft);
        draft.setId(projectInfo.getDraftId());
        //写死 是否有主管单位
        draft.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        draft.setIsSuperOrg(CommonEnum.YES.getCode());
        draft.setUserId(user.getUserIdStr());
        draft.setBuildOrgCode(user.getMhUnitIdStr());
        draft.setBuildOrgName(user.getMhUnitName());
        UnitDTO unit = mhUnitCache.getById(user.getMhUnitId());
        Assert.notNull(unit, "申报单位不存在");
        MhUnitTypeEnum unitType = Objects.requireNonNull(unit.getType(), "申报单位类型错误");
        draft.setUnitStrip(unitType.getStrip().getCode());
        if (CollUtil.isNotEmpty(projectInfo.getDynamicForm())) {
            draft.setDynamicForm(JSON.toJSONString(projectInfo.getDynamicForm()));
        }
        if (CollUtil.isNotEmpty(projectInfo.getPaymentPlanList())) {
            draft.setPaymentPlans(JSON.toJSONString(projectInfo.getPaymentPlanList()));
        }
        if (CollUtil.isNotEmpty(projectInfo.getApplicationList())) {
            draft.setProjectApplicationList(JSON.toJSONString(projectInfo.getApplicationList()));
        }
        if (Objects.isNull(draft.getId())) {
            draft.setCreateOn(LocalDateTime.now());
        } else {
            ProjectDraft old = projectDraftService.getById(draft.getId());
            if (Objects.isNull(old)) {
                draft.setCreateOn(LocalDateTime.now());
            }
        }
        if (CollUtil.isNotEmpty(projectInfo.getSystemReplaceInfos())) {
            draft.setGovSystemReplaceInfos(JSONUtil.toJsonStr(projectInfo.getSystemReplaceInfos()));
        }
        draft.setDraftType(draftType.getCode());
        draft.setUpdateOn(LocalDateTime.now());
        draft.setUserId(user.getUserIdStr());
        projectDraftService.saveOrUpdate(draft);
        return draft.getId();
    }


    public DeclaredProgressVO declaredProgress(ProjectListReq req) {
        req.setStatus(null);
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        // 当非预审申报的时候 是自己单位 当是预审申报的时候 要主管单位
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return null;
        }
        query.ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE)
                .select(Project::getStatus, Project::getProjectCode, Project::getStage);
        List<Project> projects = projectService.list(query);
        Map<ProjectStatus, Long> countMap = CollUtils.groupCount(projects,
                w -> ProjectStatus.getNoNull(w.getStatus()));
        DeclaredProgressVO stat = new DeclaredProgressVO();
        stat.setTotalCount(projects.size());
        stat.setTodoCount(countMap.getOrDefault(ProjectStatus.TO_BE_DECLARED, 0L));
        stat.setAuditCount(countMap.getOrDefault(ProjectStatus.ON_COMPLIANCE_REVIEW, 0L));
        stat.setFailedCount(countMap.getOrDefault(ProjectStatus.COMPLIANCE_REVIEW_FAILED, 0L));
        if (!projects.isEmpty()) {
            List<String> projectCodes = CollUtils.fieldList(projects, Project::getProjectCode);
            stat.setPassedCount(projectStatusChangeService.count(projectCodes, COMPLIANCE_REVIEW_PASS));
        }
        return stat;
    }


    /**
     * 项目列表
     *
     * @param req
     * @return
     */
    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        Integer status = req.getStatus();
        req.setStatus(null);
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        // 当非预审申报的时候 是自己单位 当是预审申报的时候 要主管单位
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return PageVo.empty();
        }
        if (status != null) {
            if (ProjectStatus.COMPLIANCE_REVIEW_PASSED.eq(status)) {
                query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", COMPLIANCE_REVIEW_PASS);
            } else {
                query.eq(Project::getStatus, status);
            }
        }
        query.ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = BeanUtil.copyProperties(w, ProjectLibListItemVO.class);
            item.setDeclaredAmount(w.getDeclareAmount());
            if (!ProjectStatus.TO_BE_DECLARED.eq(w.getStatus())
                    && !ProjectStatus.COMPLIANCE_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.ON_COMPLIANCE_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.CHANGE.eq(w.getStage())) {
                item.setStatus(ProjectStatus.COMPLIANCE_REVIEW_PASSED.getCode());
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    private boolean projectQueryPermission(LambdaQueryWrapper<Project> query,
                                           UserFullInfoDTO user,
                                           Long declaredUnitId) {
        BiConsumer<LambdaQueryWrapper<Project>, List<Long>> addUnitIdsToQuery = (q, unitIds) -> {
            List<String> unitIdStrList = CollUtils.convert(unitIds, String::valueOf);
            q.and(q1 -> q1.in(Project::getBuildOrgCode, unitIdStrList)
                    .or(q2 -> q2.eq(Project::getStage, ProjectStatus.NOT_APPROVED.getCode())
                            .in(Project::getSuperOrgCode, unitIdStrList)));
        };
        boolean queryState = true;
        Optional<DataScopeDTO> currDs = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!currDs.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            switch (currDs.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER: {
                    if (declaredUnitId == null) {
                        if (user.getMhUnitId() != MhUnitConst.ROOT_UNIT_ID) {
                            List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                            addUnitIdsToQuery.accept(query, viewUnitIds);
                        }
                    } else {
                        boolean userUnitIsRoot = user.getMhUnitId() == MhUnitConst.ROOT_UNIT_ID;
                        boolean queryUnitIsRoot = declaredUnitId == MhUnitConst.ROOT_UNIT_ID;
                        boolean userUnitEqQueryUnit = user.getMhUnitId().equals(declaredUnitId);
                        if ((userUnitIsRoot && !queryUnitIsRoot)
                                || (!userUnitIsRoot && userUnitEqQueryUnit)) {
                            List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                            addUnitIdsToQuery.accept(query, viewUnitIds);
                        } else if (!userUnitIsRoot && queryUnitIsRoot) {
                            queryState = false;
                        } else if (!userUnitIsRoot) {
                            List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                            if (viewUnitIds.contains(declaredUnitId)) {
                                addUnitIdsToQuery.accept(query, mhUnitCache.getViewChildIdsRecursion(declaredUnitId));
                            } else {
                                queryState = false;
                            }
                        }
                    }
                }
                break;
                case SUPER_ADMIN: {
                    if (declaredUnitId != null && MhUnitConst.ROOT_UNIT_ID != declaredUnitId) {
                        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                        addUnitIdsToQuery.accept(query, viewUnitIds);
                    }
                }
                break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
        }
        return queryState;
    }

    public void exportList(HttpServletResponse response, ProjectListReq param) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        //当非预审申报的时候 是自己单位 当是预审申报的时候 要主管单位
        if (!projectQueryPermission(query, user, param.getDeclaredUnitId())) {
            throw BizException.wrap("暂无数据可以导出");
        }
        query.eq(Project::getNewest, Boolean.TRUE);
        List<Project> records = projectService.list(query);

        AtomicInteger serialNumber = new AtomicInteger(0);
        List<DeclaredProjectExportDTO> collect = records.stream().map(r -> {
            DeclaredProjectExportDTO item = new DeclaredProjectExportDTO();
            BeanUtils.copyProperties(r, item);
            item.setProjectTypeName(ProjectTypeNewEnum.getDesc(r.getProjectType()));
            item.setStatusName(ProjectStatus.getDesc(r.getStatus()));
            String createOnStr = DateUtil.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
            item.setCreateOn(createOnStr);
            item.setSerialNumber(serialNumber.incrementAndGet());
            return item;
        }).collect(Collectors.toList());
        String fileName = "项目申报列表";
        ExcelDownUtil.setFileName(fileName, response);
        //数据导出处理函数
        try {
            EasyExcelFactory.write(response.getOutputStream(), DeclaredProjectExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(collect);
        } catch (IOException e) {
            throw new BizException(e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void deleteProjectLogic(Long projectId) {
        Project project = projectService.getNewestNoNull(projectId);
        if (!ProjectStatus.COMPLIANCE_REVIEW_FAILED.eq(project.getStatus())) {
            throw ReturnException.wrap("当前项目不可删除");
        }
        UserInfoDetails user = LoginUserUtil.userDetailNotNull();
        if (!StrUtil.equals(project.getBuildOrgCode(), user.getMhUnitIdStr())
                && !user.getUserId().equals(project.getCreateBy())
                && !user.hasSuperAdmin()) {
            throw ReturnException.wrap("暂无权限删除此项目");
        }
        ProjectStatusChange change = new ProjectStatusChange();
        change.setProjectId(project.getId());
        change.setBeforeStatus(project.getStatus());
        change.setAfterStatus(ProjectStatus.DELETED_PASSED.getCode());
        change.setEvent(ProjectStateChangeEvent.DELETED_APPLY_PASSED.name());
        change.setProjectCode(project.getProjectCode());
        change.setCreateOn(LocalDateTime.now());
        projectStatusChangeService.save(change);
        // 修改项目状态为已删除
        project.setStage(ProjectStatus.DELETED.getCode());
        project.setStatus(ProjectStatus.DELETED_PASSED.getCode());
        projectService.updateById(project);
    }

}
