package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.model.constant.MhUnitConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.meeting.entity.domain.MeetingInnerProject;
import com.hz.pm.api.meeting.entity.dto.ProjectReviewResultDTO;
import com.hz.pm.api.meeting.service.IMeetingInnerProjectService;
import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectdeclared.model.entity.ProjectReview;
import com.hz.pm.api.projectdeclared.model.vo.ExpertReviewProgressVO;
import com.hz.pm.api.projectdeclared.service.IProjectReviewService;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.helper.ProjectSaveHelper;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.UploadConstructionPlanSealFileReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.scheduler.listener.event.MhTodoHandedEvent;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * ProjectReviewManage
 * </p>
 *
 * @author WendyYang
 * @since 10:54 2024/4/17
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectReviewManage {

    private final IProjectReviewService projectReviewService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final IProjectService projectService;
    private final ProcessInstanceService processInstanceService;
    private final IProjectInstService projectInstService;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final MhUnitCache mhUnitCache;
    private final UserInfoHelper userInfoHelper;
    private final IMeetingInnerProjectService meetingInnerProjectService;
    private final ProjectSaveHelper projectSaveHelper;
    private final ProjectLibManage projectLibManage;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void projectReviewApply(DefaultDeclaredDTO req) {
        ProjectDTO reqProj = req.getProjectInfo();
        Project project = projectService.getById(reqProj.getId());
        if (!ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(project.getStatus())
                && !ProjectStatus.PROJECT_REVIEW_FAILED.eq(project.getStatus())) {
            throw BizException.wrap("该项目不允许申请项目评审");
        }

        UserInfoDetails user = LoginUserUtil.userDetailNotNull();
        WflowModels model = processModelManage.getWflowModelsNoNull(ProjectProcessType.PROJECT_REVIEW,
                user.getMhUnitId());

        ProcessStartParamsVo processParam = new ProcessStartParamsVo();
        processParam.setUser(declaredProjectHelper.buildUser(user));
        processParam.setProcessUsers(Collections.emptyMap());
        processParam.setFormData(Collections.emptyMap());
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instanceId = processInstanceService.startProcessLs(model, processParam, orgMap);

        BeanUtils.copyProperties(reqProj, project);
        if (projectService.updateById(project)) {
            projectSaveHelper.saveApplication(reqProj, project, null);
            projectSaveHelper.savePaymentPlans(project, reqProj.getPaymentPlanList());
            projectSaveHelper.saveGovSystemReplaceInfos(project.getId(), project.getProjectCode(), reqProj.getSystemReplaceInfos());
        }

        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instanceId);
        projectInst.setInstType(ProjectProcessType.PROJECT_REVIEW);
        projectInst.setProjectId(reqProj.getId());
        projectInstService.save(projectInst);
        projectStateMachineUtil.pass(project);
        projectService.updateById(project);
        SpringUtil.publishEvent(MhTodoHandedEvent.of(MHTodoTypeEnum.PROJECT_REVIEW_APPLY, project.getId()));
        log.info("项目评审提交成功 【{}】", instanceId);
    }

    public ProjectReviewResultDTO projectReviewDetail(Long projectId) {
        ProjectReview projReview = projectReviewService.getOne(Wrappers
                .lambdaQuery(ProjectReview.class)
                .eq(ProjectReview::getProjectCode, ProjectIdCodeCacheUtil.get(projectId))
                .orderByDesc(ProjectReview::getId)
                .last(BizConst.LIMIT_1));
        return BeanUtil.copyProperties(projReview, ProjectReviewResultDTO.class);
    }

    /**
     * 项目列表
     *
     * @param req \
     * @return \
     */
    public ExpertReviewProgressVO reviewProjectProgress(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return null;
        }
        req.setStatus(null);
        ProjectManageUtil.projectQuery(query, req);
        query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.EXPERT_REVIEW_PASS)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE)
                .select(Project::getStatus, Project::getId);
        List<Project> projects = projectService.list(query);
        Map<ProjectStatus, Long> countMap = CollUtils.groupCount(projects, w -> {
            if (!ProjectStatus.PROJECT_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.EXPERT_REVIEW_PASSED.eq(w.getStatus())
                    && !ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_CHANGE_APPLY.eq(w.getStatus())) {
                return ProjectStatus.PROJECT_REVIEW_PASSED;
            }
            return ProjectStatus.getNoNull(w.getStatus());
        });
        return ExpertReviewProgressVO.builder()
                .totalCount(projects.size())
                .todoCount(countMap.getOrDefault(ProjectStatus.WITHOUT_PROJECT_REVIEW, 0L))
                .auditCount(countMap.getOrDefault(ProjectStatus.ON_PROJECT_REVIEW, 0L))
                .passedCount(countMap.getOrDefault(ProjectStatus.PROJECT_REVIEW_PASSED, 0L))
                .failedCount(countMap.getOrDefault(ProjectStatus.PROJECT_REVIEW_FAILED, 0L))
                .build();
    }

    /**
     * 项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> pageReviewProject(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return PageVo.empty();
        }
        Integer status = req.clearStatus();
        if (status != null) {
            req.setStatus(null);
            if (ProjectStatus.PROJECT_REVIEW_PASSED.eq(status)) {
                query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
            } else {
                query.eq(Project::getStatus, status);
            }
        } else {
            query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                    " and npsc.event = {0}", ProjectStateChangeEvent.EXPERT_REVIEW_PASS);
        }
        ProjectManageUtil.projectQuery(query, req);
        query.eq(Project::getNewest, Boolean.TRUE)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED);
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = BeanUtil.copyProperties(w, ProjectLibListItemVO.class);
            if (!ProjectStatus.PROJECT_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.EXPERT_REVIEW_PASSED.eq(w.getStatus())
                    && !ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_CHANGE_APPLY.eq(w.getStatus())) {
                item.setStatus(ProjectStatus.PROJECT_REVIEW_PASSED.getCode());
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    public ExpertReviewProgressVO expertReviewProjectProgress(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return null;
        }
        req.setStatus(null);
        ProjectManageUtil.projectQuery(query, req);
        query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE)
                .select(Project::getStatus, Project::getId, Project::getConstructionPlanSealFile);
        List<Project> projects = projectService.list(query);
        Map<ProjectStatus, Long> countMap = CollUtils.groupCount(projects, w -> {
            if (!ProjectStatus.EXPERT_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY.eq(w.getStatus())
                    && !ProjectStatus.COMPLIANCE_REVIEW_PASSED.eq(w.getStatus())
                    && !ProjectStatus.WITHOUT_EXPERT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_EXPERT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_CHANGE_APPLY.eq(w.getStatus())) {
                return ProjectStatus.EXPERT_REVIEW_PASSED;
            }
            return ProjectStatus.getNoNull(w.getStatus());
        });
        return ExpertReviewProgressVO.builder()
                .totalCount(projects.size())
                .withoutConstructionPlanSealFileCount(CollUtil.count(projects,
                        w -> ProjectStatus.WITHOUT_EXPERT_REVIEW.eq(w.getStatus())
                                && StrUtil.isBlank(w.getConstructionPlanSealFile())))
                .todoCount(countMap.getOrDefault(ProjectStatus.WITHOUT_EXPERT_REVIEW, 0L))
                .auditCount(countMap.getOrDefault(ProjectStatus.ON_EXPERT_REVIEW, 0L))
                .passedCount(countMap.getOrDefault(ProjectStatus.EXPERT_REVIEW_PASSED, 0L))
                .failedCount(countMap.getOrDefault(ProjectStatus.EXPERT_REVIEW_FAILED, 0L) +
                        countMap.getOrDefault(ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY, 0L))
                .toBeResubmitCount(countMap.getOrDefault(ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY, 0L))
                .build();
    }

    /**
     * 项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> pageExpertReviewProject(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return PageVo.empty();
        }
        Integer status = req.getStatus();
        if (status != null) {
            req.setStatus(null);
            if (ProjectStatus.EXPERT_REVIEW_PASSED.eq(status)) {
                query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.EXPERT_REVIEW_PASS);
            } else if (ProjectStatus.EXPERT_REVIEW_FAILED.eq(status)) {
                query.in(Project::getStatus, ProjectStatus.EXPERT_REVIEW_FAILED.getCode(),
                        ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY.getCode());
            } else {
                query.eq(Project::getStatus, status);
            }
        } else {
            query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                    " and npsc.event = {0}", ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS);
        }
        ProjectManageUtil.projectQuery(query, req);
        ProjectManageUtil.projectBaseQuery(query);
        query.eq(Project::getNewest, Boolean.TRUE)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED);
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<Long> projectIds = new ArrayList<>();
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            projectIds.add(w.getId());
            ProjectLibListItemVO item = BeanUtil.copyProperties(w, ProjectLibListItemVO.class);
            if (!ProjectStatus.EXPERT_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY.eq(w.getStatus())
                    && !ProjectStatus.COMPLIANCE_REVIEW_PASSED.eq(w.getStatus())
                    && !ProjectStatus.WITHOUT_EXPERT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_EXPERT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_CHANGE_APPLY.eq(w.getStatus())) {
                item.setStatus(ProjectStatus.EXPERT_REVIEW_PASSED.getCode());
            }
            return item;
        });
        Wrapper<MeetingInnerProject> wrapper = Wrappers.lambdaQuery(MeetingInnerProject.class)
                .in(MeetingInnerProject::getProjectId, projectIds)
                .notExists(ExistsSqlConst.MEETING_INNER_PROJECT_EXISTS_MEETING + " and status == 3");
        List<MeetingInnerProject> meetingInnerProjects = meetingInnerProjectService.list(wrapper);
        Map<Long, MeetingInnerProject> lastMeetingInnerProjects = BizUtils.groupFirstMap(meetingInnerProjects,
                MeetingInnerProject::getProjectId, Comparator.comparing(MeetingInnerProject::getId));
        for (ProjectLibListItemVO item : records) {
            MeetingInnerProject meetingInnerProject = lastMeetingInnerProjects.get(item.getId());
            if (meetingInnerProject != null) {
                item.setMeetingId(meetingInnerProject.getMeetingId());
                item.setMeetingProjectId(meetingInnerProject.getId());
            }
        }
        return PageVo.of(records, page.getTotal());
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void resubmitByModify(DefaultDeclaredDTO req) {
        ProjectDTO reqProj = req.getProjectInfo();
        Assert.notNull(reqProj.getId(), "项目ID不能为空");

        if (StrUtil.isBlank(reqProj.getBuildOrgCode())) {
            UserInfoDetails user = LoginUserUtil.userDetailNotNull();
            reqProj.setBuildOrgCode(user.getMhUnitIdStr());
            reqProj.setBuildOrgName(user.getMhUnitName());
        }

        Project oldProj = projectService.getNoNull(reqProj.getId());
        //首先要判断 项目当前状态 是不是 单位内部拒绝
        if (!ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY.eq(oldProj.getStatus())) {
            throw ReturnException.wrap("该项目不是修改方案后重新评审状态");
        }
        //项目名称去重
        if (StrUtil.isNotBlank(reqProj.getProjectName()) &&
                !reqProj.getProjectName().equals(oldProj.getProjectName())) {
            reqProj.setProjectCode(oldProj.getProjectCode());
            declaredProjectHelper.projectNameDuplicateCheck(reqProj);
        }

        //判断申报金额 是否等于总的 判断年度支付金额 是否等于总金额
        declaredProjectHelper.checkAmount(reqProj);
        // 保存项目相关
        Project newProj = projectLibManage.newProjectWithVersion(reqProj, oldProj, null);
        log.info("项目申报提交成功，项目名称：{} 项目ID：{}", newProj.getProjectName(), newProj.getId());

    }

    private boolean projectQueryPermission(LambdaQueryWrapper<Project> query,
                                           UserFullInfoDTO user,
                                           Long declaredUnitId) {
        boolean queryState = true;
        Optional<DataScopeDTO> currDs = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!currDs.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            switch (currDs.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER: {
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    if (declaredUnitId != null && childUnitIds.contains(declaredUnitId)
                            && !Objects.equals(declaredUnitId, user.getMhUnitId())) {
                        childUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                    }
                    List<String> viewUnitIdList = CollUtils.convert(childUnitIds, String::valueOf);
                    //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    query.in(Project::getBuildOrgCode, viewUnitIdList);
                }
                break;
                case SUPER_ADMIN: {
                    if (declaredUnitId != null && MhUnitConst.ROOT_UNIT_ID != declaredUnitId) {
                        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                        List<String> viewUnitIdList = CollUtils.convert(viewUnitIds, String::valueOf);
                        query.in(Project::getBuildOrgCode, viewUnitIdList);
                    }
                }
                break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
        }
        return queryState;
    }

    public void uploadConstructionPlanSealFile(UploadConstructionPlanSealFileReq req) {
        Project project = projectService.getNewProject(req.getProjectId());
        if (project == null || (!ProjectStatus.WITHOUT_EXPERT_REVIEW.eq(project.getStatus())
                && !ProjectStatus.EXPERT_REVIEW_FAILED.eq(project.getStatus()))) {
            throw ReturnException.wrap("项目不存在或项目状态异常");
        }
        Wrapper<Project> wrapper = Wrappers.lambdaUpdate(Project.class)
                .set(Project::getConstructionPlanSealFile, req.getConstructionPlanSealFile())
                .set(Project::getUpdateOn, LocalDateTime.now())
                .set(Project::getUpdateBy, LoginUserUtil.getUserId())
                .eq(Project::getId, req.getProjectId());
        projectService.update(wrapper);
        SpringUtil.publishEvent(MhTodoHandedEvent.of(MHTodoTypeEnum.WITHOUT_BUILD_SCHEME, project.getId()));
    }

}
