package com.hz.pm.api.projectdeclared.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;

import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * IPurchaseInstService
 * </p>
 *
 * @author WendyYang
 * @since 00:12 2024/2/25
 */
public interface IPurchaseInstService extends IService<PurchaseInst> {

    default PurchaseInst getByProjectIdAndInstCode(Long projectId, String instCode) {
        Wrapper<PurchaseInst> query = Wrappers.lambdaQuery(PurchaseInst.class)
                .eq(PurchaseInst::getProjectId, projectId)
                .eq(PurchaseInst::getInstCode, instCode);
        return getOne(query);
    }

    default List<PurchaseInst> listByInstCodes(Collection<String> instCodes) {
        Wrapper<PurchaseInst> query = Wrappers.lambdaQuery(PurchaseInst.class)
                .in(PurchaseInst::getInstCode, instCodes);
        return list(query);
    }

    default PurchaseInst getByInstCode(String instCode) {
        Wrapper<PurchaseInst> query = Wrappers.lambdaQuery(PurchaseInst.class)
                .eq(PurchaseInst::getInstCode, instCode);
        return getOne(query);
    }

    default List<PurchaseInst> listByProjectId(Long projectId) {
        Wrapper<PurchaseInst> query = Wrappers.lambdaQuery(PurchaseInst.class)
                .eq(PurchaseInst::getProjectId, projectId);
        return list(query);
    }

    default Map<Long, PurchaseInst> listNewest(Collection<Long> purchaseIds, Integer instType) {
        List<PurchaseInst> instances = list(Wrappers.lambdaQuery(PurchaseInst.class)
                .in(PurchaseInst::getBidId, purchaseIds)
                .eq(PurchaseInst::getInstType, instType));
        return BizUtils.groupFirstMap(instances, PurchaseInst::getBidId,
                Comparator.comparing(PurchaseInst::getCreateOn).reversed());
    }

}
