package com.hz.pm.api.projectdeclared.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.projectdeclared.mapper.PurchaseMapper;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.model.req.XcfhxListReq;
import com.hz.pm.api.projectdeclared.model.vo.XcfhxApplyListVO;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderXcfhxApplyStatus;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zpf
 * @since 2023-02-05
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class PurchaseServiceImpl extends ServiceImpl<PurchaseMapper, Purchase>
        implements IPurchaseService {

    private final UserInfoHelper userInfoHelper;

    private final IProjectService projectService;

    private final IContractService contractService;

    private final MhUnitCache mhUnitCache;

    @Override
    public List<XcfhxApplyListVO> listXcfhxApplyListVO(XcfhxListReq req) {
        LambdaQueryWrapper<Purchase> query = Wrappers.lambdaQuery(Purchase.class)
                .like(StrUtil.isNotBlank(req.getBidName()), Purchase::getBidName, req.getBidName())
                .le(req.getCreateOnMax() != null, Purchase::getCreateOn, req.getCreateOnMax())
                .ge(req.getCreateOnMin() != null, Purchase::getCreateOn, req.getCreateOnMin())
                .eq(req.getStatus() != null, Purchase::getXcfhxApplyStatus, req.getStatus())
                .eq(req.getSourceType() != null, Purchase::getSourceType, req.getSourceType())
                .eq(Purchase::getBidType, BidTypeEnum.BUILD_APP.getCode())
                .notExists(ExistsSqlConst.PURCHASE_EXISTS_PROJECT + " and np.stage in ({0},{1})",
                        ProjectStatus.STOPPED.getCode(), ProjectStatus.CHANGE.getCode())
                .orderByDesc(Purchase::getCreateOn);

        if (Boolean.TRUE.equals(req.getFinished())) {
            query.eq(Purchase::getXcfhxApplyStatus, TenderXcfhxApplyStatus.XCFHX_APPLY_PASSED.getCode());
        } else if (Boolean.FALSE.equals(req.getFinished())) {
            query.ne(Purchase::getXcfhxApplyStatus, TenderXcfhxApplyStatus.XCFHX_APPLY_PASSED.getCode());
        }
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        List<XcfhxApplyListVO> others = Lists.newArrayList();
        if (!buildPurchaseQueryPermission(query, user)) {
            return others;
        }
        List<Purchase> purchases = list(query);
        if (CollUtil.isNotEmpty(purchases)) {
            Set<Long> projectIds = CollUtils.fieldSet(purchases, Purchase::getProjectId);
            List<Project> projects = projectIds.isEmpty() ? Lists.newArrayList() : projectService.list(Wrappers.lambdaQuery(Project.class)
                    .in(Project::getId, projectIds)
                    .like(StringUtils.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName()));
            Map<Long, Project> projectMap = CollUtils.listToMap(projects, Project::getId);
            Map<Long, Contract> contractMap = projectIds.isEmpty() ? Maps.newHashMap()
                    : contractService.listByProjectIds(projectIds);
            others = purchases.stream().filter(w -> projectMap.containsKey(w.getProjectId()))
                    .map(w -> {
                        XcfhxApplyListVO item = XcfhxApplyListVO.builder()
                                .bidId(w.getId())
                                .bizName(w.getBidName())
                                .status(w.getXcfhxApplyStatus())
                                .projectId(w.getProjectId())
                                .createOn(w.getCreateOn())
                                .sourceType(w.getSourceType())
                                .build();
                        Project project = projectMap.get(w.getProjectId());
                        item.setProjectCode(project.getProjectCode());
                        item.setProjectName(project.getProjectName());
                        item.setBuildOrgCode(project.getBuildOrgCode());
                        item.setBuildOrgName(project.getBuildOrgName());

                        Contract contract = contractMap.get(w.getId());
                        if (contract != null) {
                            if (Objects.nonNull(req.getAmountStart()) && Objects.nonNull(contract.getTotalAmount())
                                    && req.getAmountStart().compareTo(contract.getTotalAmount()) > 0) {
                                return null;
                            }
                            if (Objects.nonNull(req.getAmountEnd()) && Objects.nonNull(contract.getTotalAmount())
                                    && req.getAmountEnd().compareTo(contract.getTotalAmount()) < 0) {
                                return null;
                            }
                            item.setContractAmount(contract.getTotalAmount());
                        } else {
                            if (Objects.nonNull(req.getAmountStart()) || Objects.nonNull(req.getAmountEnd())) {
                                return null;
                            }
                        }
                        return item;
                    }).collect(Collectors.toList());
        }
        return others;
    }

    @Override
    public boolean buildPurchaseQueryPermission(LambdaQueryWrapper<Purchase> query, UserFullInfoDTO user) {
        boolean queryState = true;
        Optional<DataScopeDTO> datascope = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!datascope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            final String existsSql = ExistsSqlConst.PURCHASE_EXISTS_PROJECT + " and np.build_org_code";
            switch (datascope.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    query.exists(String.format("%s = '%s'", existsSql, user.getMhUnitIdStr()));
                    break;
                case COMPANY_MANAGER:
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    List<String> viewUnitIdList = CollUtils.convert(childUnitIds, String::valueOf);
                    query.exists(String.format("%s in %s", existsSql, BizUtils.inSqlJoin(viewUnitIdList)));
                    break;
                case SUPER_ADMIN:
                    // 超级管理员 看到全市的 并且也要判断他 同时是不是单位管理员
                    break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
        }
        return queryState;
    }
}
