package com.hz.pm.api.projectlib.controller;

import com.hz.pm.api.projectlib.manage.QxProjectImportManage;
import com.hz.pm.api.projectlib.manage.QxProjectManage;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.req.*;
import com.hz.pm.api.projectlib.model.vo.QxProjectDetailVO;
import com.hz.pm.api.projectlib.model.vo.QxProjectVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author WendyYang
 * @since 2024-08-31
 */
@Api(tags = "区县项目库控制器")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/qx/projectLib")
public class QxProjectController {

    private final QxProjectManage qxProjectManage;
    private final QxProjectImportManage qxProjectImportManage;

    @PostMapping("/submitProjectRecord")
    @ApiOperation("项目备案")
    @WebLog("项目备案")
    public void submitProjectRecord(@Valid @RequestBody QxProjectRecordReq req) {
        qxProjectManage.submitProjectRecord(req);
    }

    @PostMapping("/submitPurchaseRecord")
    @ApiOperation("采购备案")
    @WebLog("采购备案")
    public void submitPurchaseRecord(@Valid @RequestBody QxPurchaseRecordReq req) {
        qxProjectManage.submitPurchaseRecord(req);
    }

    @PostMapping("/submitAcceptRecord")
    @ApiOperation("验收备案")
    @WebLog("验收备案")
    public void submitAcceptRecord(@Valid @RequestBody QxAcceptRecordReq req) {
        qxProjectManage.submitAcceptRecord(req);
    }

    @PostMapping("/submitUpgradeRecord")
    @ApiOperation("改造提升&综合集成备案")
    @WebLog("改造提升&综合集成备案")
    public void submitUpgradeRecord(@Valid @RequestBody QxUpgradeRecordReq req) {
        qxProjectManage.submitUpgradeRecord(req);
    }

    @GetMapping("/detail/{id}")
    @ApiOperation("项目详情")
    public QxProjectDetailVO detail(@PathVariable Long id) {
        return qxProjectManage.getProjectDetail(id);
    }

    @GetMapping("/page")
    @ApiOperation("项目列表")
    public PageVo<QxProjectVO> page(QxProjectLibReq req) {
        return qxProjectManage.page(req);
    }

    @GetMapping("/systemReplaceInfos/{projectId}")
    @ApiOperation("项目替换系统列表")
    public List<MhSystemReplaceInfoVO> listSystemReplaceInfos(@PathVariable Long projectId) {
        return qxProjectManage.listReplaceInfo(projectId);
    }

    @PostMapping("/importProject")
    @ApiOperation("导入项目")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    public void importProject(MultipartFile file) {
        qxProjectImportManage.importProject(file);
    }

}
