package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus.TO_BE_PURCHASED;

/**
 * 招标采购处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectPurchaseHandle extends AbstractProcessHandle {

    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;
    private final ProcessInstanceService processInstanceService;

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_PURCHASE;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        if (!StepStatusEnum.COMPLETED.equals(ctx.getPreStepStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
            return;
        }
        Project project = ctx.getProject();
        if (TO_BE_PURCHASED.eq(project.getStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
            ctx.setPurchases(purchases);
            if (purchases.isEmpty()) {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            } else {
                // 根据采购备案的时间获取
                ProjectStatusChange change = ctx.getProjectStateChange(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                if (change != null) {
                    detail.setStepStatus(StepStatusEnum.COMPLETED);
                    detail.setFinishTime(change.getCreateOn());
                } else {
                    detail.setStepStatus(StepStatusEnum.ON_GOING);
                }
                List<PurchaseInst> purchaseInstList = purchaseInstService.listByProjectId(project.getId());
                ctx.setPurchaseInstList(purchaseInstList);
            }
        }
        if (!StepStatusEnum.NOT_START.equals(detail.getStepStatus())) {
            Map<Long, Purchase> purchaseMap = CollUtils.listToMap(ctx.getPurchases(), Purchase::getId);
            Map<Long, PurchaseInst> instMap = purchaseInstService.listNewest(purchaseMap.keySet(), InstTypeEnum.PURCHASE_NOTICE.getCode());
            List<ProcessInfoVO> processes = new ArrayList<>();
            instMap.forEach((bidId, inst) -> {
                ProcessProgressVo instDetail = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
                ProcessInfoVO tenderProgress = new ProcessInfoVO();
                tenderProgress.setProcessName(purchaseMap.get(bidId).getBidName());
                tenderProgress.setProcessProgressVo(instDetail);
                processes.add(tenderProgress);
            });
            detail.setProcessInfoList(processes);
        }
        ctx.setPreStepStatus(detail.getStepStatus());
    }

}
