package com.hz.pm.api.projectlib.helper;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.performance.model.dto.ProjectCoreBusinessDTO;
import com.hz.pm.api.performance.model.entity.ProjectCoreBusinessIndicators;
import com.hz.pm.api.performance.service.IProjectCoreBusinessIndicatorsService;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoUpgradeContentDTO;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectPaymentPlanDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectAnnualPaymentPlan;
import com.hz.pm.api.projectlib.model.entity.ProjectApplication;
import com.hz.pm.api.projectlib.service.IProjectAnnualPaymentPlanService;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * ProjectSaveHelper
 * </p>
 *
 * @author WendyYang
 * @since 16:12 2024/9/26
 */
@Component
@RequiredArgsConstructor
public class ProjectSaveHelper {

    private final IProjectService projectService;
    private final IProjectApplicationService projectApplicationService;
    private final IProjectGovSystemReplaceInfosService projectGovSystemReplaceInfosService;
    private final IProjectCoreBusinessIndicatorsService projectCoreBusinessIndicatorsService;
    private final IProjectAnnualPaymentPlanService projectAnnualPaymentPlanService;

    public void savePaymentPlans(Project project, List<ProjectPaymentPlanDTO> paymentPlans) {
        if (CollUtil.isEmpty(paymentPlans)) {
            return;
        }
        Integer version = Objects.nonNull(project.getVersion()) ? project.getVersion() : 1;
        projectAnnualPaymentPlanService.remove(project.getProjectCode(), version);
        List<ProjectAnnualPaymentPlan> plans = paymentPlans.stream().map(w -> {
            ProjectAnnualPaymentPlan plan = BeanUtil.copyProperties(w, ProjectAnnualPaymentPlan.class);
            plan.setProjectCode(project.getProjectCode());
            plan.setProjectVersion(version);
            return plan;
        }).collect(Collectors.toList());
        projectAnnualPaymentPlanService.saveBatch(plans);
    }

    public void saveApplication(Project project, Project oldProject, Boolean isConstruct) {
        List<ProjectApplication> apps = projectApplicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(ProjectApplication::getProjectCode, oldProject.getProjectCode())
                .eq(ProjectApplication::getProjectVersion, oldProject.getVersion()));
        if (CollUtil.isNotEmpty(apps)) {
            apps.forEach(app -> {
                Long oldAppId = app.getId();
                app.setProjectVersion(project.getVersion());
                app.setProjectId(project.getId());
                app.setId(null);
                if (Objects.nonNull(isConstruct)) {
                    app.setIsConstruct(isConstruct);
                }
                projectApplicationService.save(app);

                //核心业务
                List<ProjectCoreBusinessIndicators> cores = projectCoreBusinessIndicatorsService.list(Wrappers.lambdaQuery(ProjectCoreBusinessIndicators.class)
                        .eq(ProjectCoreBusinessIndicators::getApplicationId, oldAppId));
                if (CollUtil.isNotEmpty(cores)) {
                    cores.forEach(c -> {
                        c.setId(null);
                        c.setApplicationId(app.getId());
                        projectCoreBusinessIndicatorsService.save(c);
                    });
                }
            });
        }
    }

    public void saveApplication(ProjectDTO projectDto, Project project, Boolean isConstruct) {
        UserInfoDetails user = LoginUserUtil.userDetail();
        //保存项目应用
        boolean isApp = Objects.nonNull(projectDto.getIncludeApplication()) && CommonEnum.YES.getCode().equals(projectDto.getIncludeApplication())
                ? Boolean.TRUE : Boolean.FALSE;
        //采取批量删除 批量添加的方式
        Integer version = Objects.nonNull(project.getVersion()) ? project.getVersion() : 1;
        //删除当前版本的app 一般情况是没有 保险起见
        projectApplicationService.remove(project.getProjectCode(), version);
        if (isApp && CollUtil.isNotEmpty(projectDto.getApplicationList())) {
            projectDto.getApplicationList().forEach(application -> {
                ProjectApplication projectApplication = new ProjectApplication();
                BeanUtils.copyProperties(application, projectApplication);
                projectApplication.setId(null);
                projectApplication.setProjectId(project.getId());
                projectApplication.setProjectCode(project.getProjectCode());
                projectApplication.setBuildOrgCode(project.getBuildOrgCode());
                projectApplication.setBuildOrgName(project.getBuildOrgName());
                projectApplication.setProjectVersion(version);
                projectApplication.setIsConstruct(isConstruct);
                if (StringUtils.isNotBlank(application.getRelatedExistsApplication())) {
                    projectApplication.setApplicationName(null);
                }
                boolean result = projectApplicationService.save(projectApplication);
                // 保存应用关联的核心业务
                List<ProjectCoreBusinessDTO> coreBusinessList = application.getCoreBusinessList();
                Boolean hasCoreBusiness = CollUtil.isNotEmpty(coreBusinessList) ? Boolean.TRUE : Boolean.FALSE;
                if (Boolean.TRUE.equals(result) && Boolean.TRUE.equals(hasCoreBusiness)) {
                    for (ProjectCoreBusinessDTO coreBusiness : coreBusinessList) {
                        ProjectCoreBusinessIndicators projectCoreBusinessIndicators = new ProjectCoreBusinessIndicators();
                        BeanUtils.copyProperties(coreBusiness, projectCoreBusinessIndicators);
                        projectCoreBusinessIndicators.setId(null);
                        projectCoreBusinessIndicators.setApplicationId(projectApplication.getId());
                        projectCoreBusinessIndicators.setCreateOn(LocalDateTime.now());
                        projectCoreBusinessIndicators.setCreateBy(user.getUsername());
                        projectCoreBusinessIndicatorsService.save(projectCoreBusinessIndicators);
                    }
                }
            });
        }
    }


    public void saveGovSystemReplaceInfos(Long projectId,
                                          String projectCode,
                                          List<GovSystemReplaceInfoDTO> systemReplaceInfos) {
        Wrapper<ProjectGovSystemReplaceInfos> delQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .eq(ProjectGovSystemReplaceInfos::getProjectCode, projectCode);
        projectGovSystemReplaceInfosService.remove(delQuery);
        LambdaUpdateWrapper<Project> projectUpdate = Wrappers.lambdaUpdate(Project.class)
                .eq(Project::getProjectCode, projectCode)
                .eq(Project::getNewest, Boolean.TRUE);
        if (CollUtil.isNotEmpty(systemReplaceInfos)) {
            List<ProjectGovSystemReplaceInfos> replaceInfos = new ArrayList<>();
            List<GovSystemReplaceInfoUpgradeContentDTO> upgradeContents = new ArrayList<>();
            for (int i = 0; i < systemReplaceInfos.size(); i++) {
                int batchNo = i + 1;
                GovSystemReplaceInfoDTO curr = systemReplaceInfos.get(i);
                for (int i1 = 0; i1 < curr.getSourceSystemNames().size(); i1++) {
                    ProjectGovSystemReplaceInfos replaceInfo = new ProjectGovSystemReplaceInfos();
                    replaceInfo.setProjectId(projectId);
                    replaceInfo.setProjectCode(projectCode);
                    replaceInfo.setReplaceType(curr.getReplaceType());
                    replaceInfo.setSourceSystem(curr.getSourceSystemNames().get(i1));
                    replaceInfo.setTargetSystem(curr.getTargetSystemName());
                    replaceInfo.setBatchNo(batchNo);
                    if (ArrayUtil.isNotEmpty(curr.getSourceSystemIds())) {
                        replaceInfo.setSourceSystemId(curr.getSourceSystemIds().get(i1));
                    }
                    replaceInfos.add(replaceInfo);
                }
                GovSystemReplaceInfoUpgradeContentDTO upgradeContent = new GovSystemReplaceInfoUpgradeContentDTO();
                upgradeContent.setBatchNo(batchNo);
                upgradeContent.setIterativeUpgradeContent(curr.getIterativeUpgradeContent());
                upgradeContent.setIsAddIterativeUpgradeContent(curr.getIsAddIterativeUpgradeContent());
                upgradeContents.add(upgradeContent);
            }
            projectUpdate.set(Project::getIterativeUpgradeContent, JSONUtil.toJsonStr(upgradeContents));
            projectGovSystemReplaceInfosService.saveBatch(replaceInfos);
        } else {
            projectUpdate.set(Project::getIterativeUpgradeContent, null);
        }
        projectService.update(projectUpdate);
    }

}
