package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.model.entity.ExcelExportWriter;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectApprovedReq;
import com.hz.pm.api.projectlib.model.req.ProjectIdReq;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.StartProjectDeclareReq;
import com.hz.pm.api.projectlib.model.vo.AnnualPlanListItemVO;
import com.hz.pm.api.projectlib.model.vo.AnnualPlanProgressVO;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.todocenter.constant.TodoCenterConst;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper.UnitQueryState;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus.ON_ANNUAL_PLAN;

/**
 * <p>
 * AnnualPlanLibManage
 * </p>
 *
 * @author WendyYang
 * @since 14:09 2023/2/11
 */
@Component
@AllArgsConstructor
@Slf4j
public class AnnualPlanLibManage {

    private final IProjectService projectService;
    private final IProjectApplicationService applicationService;
    private final ProjectStateMachineUtil stateMachine;
    private final MhUnitQueryAuthHelper mhUnitQueryAuthHelper;


    public AnnualPlanProgressVO annualPlanProgress(ProjectListReq req) {
        req.setIsStartDeclaredProject(null);
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(req.clearBuildOrgCode(), query)) {
            return null;
        }
        ProjectManageUtil.projectQuery(query, req);
        query.select(Project::getId, Project::getAnnualPlanOpened)
                .eq(Project::getNewest, Boolean.TRUE)
                .and(q1 -> q1.in(Project::getStatus, ON_ANNUAL_PLAN.getCode())
                        .or(q2 -> q2.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE + " and event = {0}",
                                ProjectStateChangeEvent.ANNUAL_PLAN_PASS)))
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode());
        List<Project> projects = projectService.list(query);
        AnnualPlanProgressVO stat = new AnnualPlanProgressVO();
        stat.setTotalCount(projects.size());
        stat.setOpenAnnualPlanCount(CollUtil.count(projects, w -> Boolean.TRUE.equals(w.getAnnualPlanOpened())));
        stat.setNotOpenAnnualPlanCount(stat.getTotalCount() - stat.getOpenAnnualPlanCount());
        return stat;
    }

    public PageVo<AnnualPlanListItemVO> annulPlanLibList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(req.clearBuildOrgCode(), query)) {
            return PageVo.empty();
        }
        ProjectManageUtil.projectQuery(query, req);
        query.eq(Project::getNewest, Boolean.TRUE);
        query.and(q1 -> q1.in(Project::getStatus, ON_ANNUAL_PLAN.getCode())
                        .or(q2 -> q2.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE + " and event = {0}",
                                ProjectStateChangeEvent.ANNUAL_PLAN_PASS)))
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode());
        Page<Project> page = projectService.page(req.page(), query);
        long total = page.getTotal();
        if (total == 0) {
            return PageVo.empty();
        }
        List<AnnualPlanListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            AnnualPlanListItemVO item = new AnnualPlanListItemVO();
            item.setProjectId(w.getId());
            item.setProjectId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectYear(w.getProjectYear());
            item.setProjectType(w.getProjectType());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setBuildOrg(w.getBuildOrgName());
            item.setCreateOn(w.getCreateOn());
            item.setApprovalAmount(w.getApprovalAmount());
            item.setIsStartDeclaredProject(w.getAnnualPlanOpened());
            return item;
        });
        return PageVo.of(records, total);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void startProjectDeclared(StartProjectDeclareReq req) {
        List<Project> projects = projectService.listByIds(req.getProjectIds());
        projects.forEach(w -> {
            if (!ON_ANNUAL_PLAN.eq(w.getStatus())) {
                throw ReturnException.wrap("开启方案申报失败");
            }
        });
        projects.forEach(w -> {
            stateMachine.pass(w);
            w.setAnnualPlanOpened(Boolean.TRUE);
        });
        projectService.updateBatchById(projects);
    }

    @Transactional(rollbackFor = Exception.class)
    public void projectApproved(ProjectApprovedReq req) {
        Project oldProject = projectService.getById(req.getProjectId());
        Project project = BeanUtil.copyProperties(oldProject, Project.class);
        // 被撤回重新申报的项目,项目ID要置空
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        stateMachine.pass(project);
        //计划出 计划验收时间
        LocalDateTime planAcceptanceTime = getPlanAcceptanceTime(req);
        //立项批复总投资
        project.setApprovalAmount(req.getApprovedAmount());
        //立项批复预算
        project.setApprovalBudget(req.getApprovalBudget());
        project.setApprovedFile(String.valueOf(req.getApprovedFileId()));
        project.setBuildCycle(String.valueOf(req.getBuildCycle()));
        project.setOriginBuildCycle(req.getBuildCycle());
        BizUtils.notNull(req.getBuildPlanFileId(), w -> project.setApprovedConstructionPlanFile(String.valueOf(w)));
        project.setApprovalDate(req.getApprovedDate());
        project.setPlanAcceptanceTime(planAcceptanceTime);
        project.setOriginPlanAcceptanceTime(planAcceptanceTime);
        // 项目编号不变，版本号加1
        project.setProjectCode(oldProject.getProjectCode());
        project.setVersion(oldProject.getVersion() + 1);
        project.setInstCode(TodoCenterConst.Declared.NULL_INST_CODE);
        project.setId(null);
        if (projectService.save(project)) {
            applicationService.saveApplication(project, oldProject, Boolean.TRUE);
            // 将旧的项目版本置为不是最新
            projectService.reverseNewest(project.getProjectCode(), project.getId());
        }
    }

    public LocalDateTime getPlanAcceptanceTime(ProjectApprovedReq req) {
        if (Objects.nonNull(req.getApprovedDate()) && Objects.nonNull(req.getBuildCycle())) {
            return req.getApprovedDate().plusMonths(req.getBuildCycle());
        }
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    public void suspendAnnualPlan(ProjectIdReq req) {
        Project project = projectService.getById(req.getProjectId());
        stateMachine.reject(project);
        projectService.updateById(project);
    }

    public void updateAnnualPlan(ProjectDTO req) {
        Project project = BeanUtil.copyProperties(req, Project.class);
        projectService.updateById(project);
    }

    public void exportList(ProjectListReq param, HttpServletResponse response) {
        param.setPageNumber(CommonConst.EXPORT_PAGE_NUMBER);
        param.setPageSize(CommonConst.EXPORT_PAGE_SIZE);
        Integer isTemporaryAugment = param.getIsTemporaryAugment();
        if (Objects.isNull(isTemporaryAugment)) {
            throw new BizException("请传入是否临时增补标志!");
        }
        param.setIsTemporaryAugment(isTemporaryAugment);
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByDesc(Project::getAnnualPlanAddTime);
        List<Project> projects = projectService.list(query);

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();
        String fileName = null;
        if (CommonEnum.NO.getCode().equals(isTemporaryAugment)) {
            fileName = "年度计划库列表";
        } else if (CommonEnum.YES.getCode().equals(isTemporaryAugment)) {
            fileName = "年度计划增补库列表";
        }
        excelExportWriter.setFileName(fileName);
        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getProjectExportDatas(param.getExportOptionList(), projects);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * build 项目的角色权限到req
     *
     * @param query \
     */
    private boolean projectQueryPermission(Long buildOrgCode, LambdaQueryWrapper<Project> query) {
        UnitQueryState state = mhUnitQueryAuthHelper.listCanViewUnitIds(buildOrgCode);
        if (!state.isState()) {
            return Boolean.FALSE;
        }
        BizUtils.notEmpty(state.getUnitIds(), w -> {
            List<String> unitIdStrList = CollUtils.convert(w, String::valueOf);
            query.in(Project::getBuildOrgCode, unitIdStrList);
        });
        return Boolean.TRUE;
    }

}
