package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.projectlib.entity.*;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.dto.MhSystemStoppedRecordDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.MhSystemReplaceType;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.MhSystemReplaceReq;
import com.hz.pm.api.projectlib.service.*;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper.UnitQueryState;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * MhSystemReplaceManage
 * </p>
 *
 * @author WendyYang
 * @since 15:11 2024/8/16
 */
@Component
@RequiredArgsConstructor
public class MhSystemReplaceManage {

    private final IMhSystemReplaceInfoService mhSystemReplaceInfoService;
    private final MhApiClient mhApiClient;
    private final IQxProjectService qxProjectService;
    private final IProjectGovSystemReplaceInfosService projectGovSystemReplaceInfosService;
    private final IProjectService projectService;
    private final IQxProjectSystemReplaceInfosRelationService qxProjectSystemReplaceInfosRelationService;
    private final MhUnitQueryAuthHelper mhUnitQueryAuthHelper;
    private final IMhSystemReplaceInfoStoppedRecordService mhSystemReplaceInfoStoppedRecordService;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void syncMhSystemReplaceInfo() {
        List<MhSystemReplaceInfoDTO> systemReplaceInfos = mhApiClient.querySystemReplaceInfos();
        mhSystemReplaceInfoService.remove(null);
        if (CollUtil.isEmpty(systemReplaceInfos)) {
            return;
        }
        List<String> ids = CollUtils.fieldList(systemReplaceInfos, MhSystemReplaceInfoDTO::getId);
        for (List<String> strings : ListUtil.partition(ids, 1000)) {
            Wrapper<MhSystemReplaceInfo> removeWrapper = Wrappers.lambdaQuery(MhSystemReplaceInfo.class)
                    .notIn(MhSystemReplaceInfo::getId, strings);
            mhSystemReplaceInfoService.remove(removeWrapper);
        }
        LocalDateTime now = LocalDateTime.now();
        List<MhSystemReplaceInfo> newData = systemReplaceInfos.stream().map(w -> {
            MhSystemReplaceInfo replaceInfo = new MhSystemReplaceInfo();
            replaceInfo.setActualFinishDate(w.getFinalTime());
            replaceInfo.setBuildOrgCode(w.getCreateUnit());
            replaceInfo.setBuildOrgName(w.getBuildUnit());
            replaceInfo.setExpectCost(w.getExpectCost());
            replaceInfo.setHost(w.getHost());
            replaceInfo.setId(w.getId());
            replaceInfo.setCreateOn(now);
            replaceInfo.setIsSecret(w.getIsSecret());
            replaceInfo.setPid(w.getPid());
            replaceInfo.setPlanFinishDate(w.getExpectDate());
            replaceInfo.setPlanStartDate(w.getExpectStart());
            replaceInfo.setReplaceType(w.getAlternativeType());
            replaceInfo.setSourceName(w.getSystemName());
            replaceInfo.setSystemNum(1);
            replaceInfo.setTargetName(w.getAlternativeName());
            return replaceInfo;
        }).collect(Collectors.toList());
        mhSystemReplaceInfoService.saveOrUpdateBatch(newData);
    }

    public PageVo<MhSystemReplaceInfoVO> page(MhSystemReplaceReq req) {
        LambdaQueryWrapper<MhSystemReplaceInfo> query = buildMhSystemReplaceInfoQuery(req);
        UnitQueryState state = mhUnitQueryAuthHelper.listCanViewUnitIds(req.getBuildOrgCode());
        if (!state.isState()) {
            return PageVo.empty();
        }
        BizUtils.notEmpty(state.getUnitIds(), w -> query.in(MhSystemReplaceInfo::getBuildOrgCode, w));
        Page<MhSystemReplaceInfo> page = mhSystemReplaceInfoService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MhSystemReplaceInfoVO> data = BeanUtil.copyToList(page.getRecords(), MhSystemReplaceInfoVO.class);
        List<String> stoppedIds = data.stream()
                .filter(w -> Boolean.TRUE.equals(w.getStopped()))
                .map(MhSystemReplaceInfoVO::getId)
                .collect(Collectors.toList());
        Map<String, MhSystemStoppedRecordDTO> stoppedRecordMap;
        if (stoppedIds.isEmpty()) {
            stoppedRecordMap = Collections.emptyMap();
        } else {
            Wrapper<MhSystemReplaceInfoStoppedRecord> stoppedQuery = Wrappers.lambdaQuery(MhSystemReplaceInfoStoppedRecord.class)
                    .in(MhSystemReplaceInfoStoppedRecord::getSystemReplaceInfosId, stoppedIds);
            List<MhSystemReplaceInfoStoppedRecord> stoppedRecords = mhSystemReplaceInfoStoppedRecordService.list(stoppedQuery);
            stoppedRecordMap = CollUtils.listToMap(stoppedRecords,
                    MhSystemReplaceInfoStoppedRecord::getSystemReplaceInfosId,
                    w -> BeanUtil.copyProperties(w, MhSystemStoppedRecordDTO.class));
        }
        List<String> sourceSystemIds = CollUtils.fieldList(data, MhSystemReplaceInfoVO::getId);
        Map<String, Project> projectMap = listProjectMapBySourceSystemIds(sourceSystemIds);
        Map<String, QxProject> qxProjectMap = listQxProjectBySourceSystemIds(sourceSystemIds);
        for (MhSystemReplaceInfoVO datum : data) {
            if (Boolean.TRUE.equals(datum.getStopped())) {
                datum.setStoppedRecord(stoppedRecordMap.get(datum.getId()));
            }
            String sourceSystemId = datum.getId();
            Project project = projectMap.get(sourceSystemId);
            if (project != null) {
                datum.setIsCountyProject(Boolean.FALSE);
                datum.setProjectId(project.getId());
                datum.setProjectName(project.getProjectName());
            } else {
                QxProject qxProject = qxProjectMap.get(sourceSystemId);
                if (qxProject != null) {
                    datum.setIsCountyProject(Boolean.TRUE);
                    datum.setProjectId(qxProject.getId());
                    datum.setProjectName(qxProject.getProjectName());
                }
            }
        }
        return PageVo.of(data, page.getTotal());
    }

    private static LambdaQueryWrapper<MhSystemReplaceInfo> buildMhSystemReplaceInfoQuery(MhSystemReplaceReq req) {
        LambdaQueryWrapper<MhSystemReplaceInfo> query = Wrappers.lambdaQuery(MhSystemReplaceInfo.class)
                .like(StrUtil.isNotBlank(req.getBuildOrgName()), MhSystemReplaceInfo::getBuildOrgName, req.getBuildOrgName())
                .and(StrUtil.isNotBlank(req.getSystemName()), q1 -> q1.like(MhSystemReplaceInfo::getSourceName, req.getSystemName())
                        .or(q2 -> q2.like(MhSystemReplaceInfo::getTargetName, req.getSystemName())))
                .eq(StrUtil.isNotBlank(req.getReplaceType()), MhSystemReplaceInfo::getReplaceType, req.getReplaceType())
                .ge(req.getPlanFinishDateMin() != null, MhSystemReplaceInfo::getPlanFinishDate, req.getPlanFinishDateMin())
                .le(req.getPlanFinishDateMax() != null, MhSystemReplaceInfo::getPlanFinishDate, req.getPlanFinishDateMax())
                .ge(req.getPlanStartDateMin() != null, MhSystemReplaceInfo::getPlanStartDate, req.getPlanStartDateMin())
                .le(req.getPlanStartDateMax() != null, MhSystemReplaceInfo::getPlanStartDate, req.getPlanStartDateMax());
        if (req.getFinish() != null) {
            if (Boolean.TRUE.equals(req.getFinish())) {
                query.orderByDesc(MhSystemReplaceInfo::getActualFinishDate);
                query.isNotNull(MhSystemReplaceInfo::getActualFinishDate);
            } else {
                query.orderByDesc(MhSystemReplaceInfo::getPlanFinishDate);
                query.isNull(MhSystemReplaceInfo::getActualFinishDate);
            }
        } else {
            query.orderByDesc(MhSystemReplaceInfo::getPlanFinishDate);
        }
        return query;
    }

    public List<MhSystemReplaceInfoVO> option(MhSystemReplaceReq req) {
        LambdaQueryWrapper<MhSystemReplaceInfo> query = buildMhSystemReplaceInfoQuery(req);
        query.eq(MhSystemReplaceInfo::getStopped, Boolean.FALSE);
        if (StrUtil.isBlank(req.getReplaceType())) {
            query.in(MhSystemReplaceInfo::getReplaceType, Arrays.asList(MhSystemReplaceType.IM, MhSystemReplaceType.IT));
        } else if (MhSystemReplaceType.NE.eq(req.getReplaceType())) {
            return Collections.emptyList();
        }
        UnitQueryState state = mhUnitQueryAuthHelper.listCanViewUnitIds(req.getBuildOrgCode());
        if (!state.isState()) {
            return Collections.emptyList();
        }
        BizUtils.notEmpty(state.getUnitIds(), w -> query.in(MhSystemReplaceInfo::getBuildOrgCode, w));
        query.notExists("select 1 from qx_project_system_replace_infos_relation qpr " +
                "where qpr.source_system_id = mh_system_replace_info.id");
        String includeProjectCodeSql;
        if (StrUtil.isNotBlank(req.getIncludeProjectCode())) {
            includeProjectCodeSql = " and t.project_code != '" + req.getIncludeProjectCode() + "'";
        } else {
            includeProjectCodeSql = "";
        }
        query.and(q1 -> q1.notExists("select 1 from (SELECT ROW_NUMBER() OVER (PARTITION BY SOURCE_SYSTEM_ID ORDER BY CREATE_ON DESC) row_no,"
                        + "PROJECT_CODE,REPLACE_TYPE FROM ND_PROJECT_GOV_SYSTEM_REPLACE_INFOS npr where npr.source_system_id = mh_system_replace_info.id) t"
                        + " where row_no = 1 and t.replace_type != 6" + includeProjectCodeSql)
                .or(q2 -> q2.exists("select 1 from (SELECT ROW_NUMBER() OVER (PARTITION BY SOURCE_SYSTEM_ID ORDER BY CREATE_ON DESC) row_no," +
                        "PROJECT_CODE FROM ND_PROJECT_GOV_SYSTEM_REPLACE_INFOS npr where npr.source_system_id = mh_system_replace_info.id) t inner join nd_project np" +
                        " on t.project_code = np.project_code where row_no = 1 and np.newest = 1" +
                        " and np.status in ({0})", ProjectStatus.STOPPED_PASSED.getCode())));
        // 终止和变更通过的项目关联的系统可以再次被选中
        List<MhSystemReplaceInfo> page = mhSystemReplaceInfoService.list(query);
        return BeanUtil.copyToList(page, MhSystemReplaceInfoVO.class);
    }

    private Map<String, Project> listProjectMapBySourceSystemIds(List<String> sourceSystemIds) {
        Wrapper<ProjectGovSystemReplaceInfos> pgsriQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .select(ProjectGovSystemReplaceInfos::getProjectCode, ProjectGovSystemReplaceInfos::getSourceSystemId)
                .in(ProjectGovSystemReplaceInfos::getSourceSystemId, sourceSystemIds);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = projectGovSystemReplaceInfosService.list(pgsriQuery);
        if (CollUtil.isEmpty(systemReplaceInfos)) {
            return Collections.emptyMap();
        }
        Map<String, String> replaceInfoMap = CollUtils.listToMap(systemReplaceInfos,
                ProjectGovSystemReplaceInfos::getSourceSystemId, ProjectGovSystemReplaceInfos::getProjectCode);
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .in(Project::getProjectCode, replaceInfoMap.values())
                .eq(Project::getNewest, Boolean.TRUE)
                .select(Project::getProjectCode, Project::getId, Project::getProjectName);
        List<Project> projects = projectService.list(query);
        if (CollUtil.isEmpty(projects)) {
            return Collections.emptyMap();
        }
        Map<String, Project> projectMap = CollUtils.listToMap(projects, Project::getProjectCode);
        return CollUtils.listToMap(replaceInfoMap.entrySet(), Map.Entry::getKey, w -> projectMap.get(w.getValue()));
    }

    private Map<String, QxProject> listQxProjectBySourceSystemIds(List<String> sourceSystemIds) {
        Wrapper<QxProjectSystemReplaceInfosRelation> relationQuery = Wrappers.lambdaQuery(QxProjectSystemReplaceInfosRelation.class)
                .select(QxProjectSystemReplaceInfosRelation::getProjectId, QxProjectSystemReplaceInfosRelation::getSourceSystemId)
                .in(QxProjectSystemReplaceInfosRelation::getSourceSystemId, sourceSystemIds);
        List<QxProjectSystemReplaceInfosRelation> relations = qxProjectSystemReplaceInfosRelationService.list(relationQuery);
        if (CollUtil.isEmpty(relations)) {
            return Collections.emptyMap();
        }
        Map<String, Long> relationMap = CollUtils.listToMap(relations, QxProjectSystemReplaceInfosRelation::getSourceSystemId,
                QxProjectSystemReplaceInfosRelation::getProjectId);
        Wrapper<QxProject> projectQuery = Wrappers.lambdaQuery(QxProject.class)
                .select(QxProject::getId, QxProject::getProjectName)
                .in(QxProject::getId, relationMap.values());
        List<QxProject> qxProjects = qxProjectService.list(projectQuery);
        if (CollUtil.isEmpty(qxProjects)) {
            return Collections.emptyMap();
        }
        Map<Long, QxProject> qxProjectMap = CollUtils.listToMap(qxProjects, QxProject::getId);
        return CollUtils.listToMap(relationMap.entrySet(), Map.Entry::getKey, w -> qxProjectMap.get(w.getValue()));
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitStoppedRecord(MhSystemStoppedRecordDTO req) {
        MhSystemReplaceInfo replaceInfo = mhSystemReplaceInfoService.getById(req.getSystemReplaceInfosId());
        if (replaceInfo == null || Boolean.TRUE.equals(replaceInfo.getStopped())) {
            return;
        }
        MhSystemReplaceInfoStoppedRecord stoppedRecord = new MhSystemReplaceInfoStoppedRecord();
        BeanUtil.copyProperties(req, stoppedRecord);
        mhSystemReplaceInfoStoppedRecordService.save(stoppedRecord);
        replaceInfo.setStopped(Boolean.TRUE);
        mhSystemReplaceInfoService.updateById(replaceInfo);
    }

}
