package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.model.vo.SystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.vo.UnitProjectGovSystemReplaceVO;
import com.hz.pm.api.projectlib.model.vo.UnitProjectGovSystemReplaceVO.ProjectGovSystemReplaceVO;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.hz.pm.api.workbench.manage.WorkbenchManage.IS_VALID_SYSTEM;

/**
 * <p>
 * ProjectGovSystemReplaceInfoManage
 * </p>
 *
 * @author WendyYang
 * @since 15:40 2024/3/20
 */
@Component
@RequiredArgsConstructor
public class ProjectGovSystemReplaceInfoManage {

    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IProjectService projectService;
    private final MhUnitCache mhUnitCache;

    public SystemReplaceInfoVO systemReplaceInfos(String targetSystem) {
        SystemReplaceInfoVO retDetail = new SystemReplaceInfoVO();
        LambdaQueryWrapper<ProjectGovSystemReplaceInfos> pgsriQuery;
        if (StrUtil.isNotBlank(targetSystem)) {
            pgsriQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                    .like(ProjectGovSystemReplaceInfos::getTargetSystem, targetSystem);
        } else {
            pgsriQuery = null;
        }
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(pgsriQuery);
        if (systemReplaceInfos.isEmpty()) {
            return retDetail;
        }
        Map<Long, List<ProjectGovSystemReplaceInfos>> projectSystemMap = CollUtils.group(systemReplaceInfos,
                ProjectGovSystemReplaceInfos::getProjectId);
        List<Project> projects = projectService.listByIds(projectSystemMap.keySet());
        Map<Long, List<Project>> unitProjectMap = projects.stream()
                .filter(w -> NumberUtil.isNumber(w.getBuildOrgCode()))
                .collect(Collectors.groupingBy(w -> Long.parseLong(w.getBuildOrgCode())));
        List<UnitProjectGovSystemReplaceVO> retList = new ArrayList<>();
        AtomicInteger before = new AtomicInteger(0);
        AtomicInteger after = new AtomicInteger(0);
        unitProjectMap.forEach((unitId, tmpProjects) -> {
            UnitDTO unit = mhUnitCache.getById(unitId);
            UnitProjectGovSystemReplaceVO item = new UnitProjectGovSystemReplaceVO();
            retList.add(item);
            item.setUnitName(unit == null ? "" : unit.getName());
            item.setProjectSystemReplaces(new ArrayList<>());
            for (Project proj : tmpProjects) {
                List<ProjectGovSystemReplaceInfos> replaceInfos = projectSystemMap.get(proj.getId());
                if (replaceInfos == null) {
                    continue;
                }
                ProjectGovSystemReplaceVO projectSystem = new ProjectGovSystemReplaceVO();
                projectSystem.setProjectId(proj.getId());
                projectSystem.setProjectCode(proj.getProjectCode());
                projectSystem.setProjectName(proj.getProjectName());
                item.getProjectSystemReplaces().add(projectSystem);
                Map<Integer, List<ProjectGovSystemReplaceInfos>> replaceInfoGroup = CollUtils.group(replaceInfos,
                        ProjectGovSystemReplaceInfos::getBatchNo);
                List<GovSystemReplaceInfoDTO> replaceInfoList = replaceInfoGroup.entrySet().stream()
                        .sorted(Comparator.comparingInt(Map.Entry::getKey))
                        .map(w -> {
                            List<ProjectGovSystemReplaceInfos> currBatchValues = w.getValue();
                            ProjectGovSystemReplaceInfos firstReplaceInfo = currBatchValues.get(0);
                            GovSystemReplaceInfoDTO replaceInfo = new GovSystemReplaceInfoDTO();
                            replaceInfo.setReplaceType(firstReplaceInfo.getReplaceType());
                            replaceInfo.setTargetSystemName(firstReplaceInfo.getTargetSystem());
                            List<String> sourceSystemNames = CollUtils.fieldList(currBatchValues,
                                    val -> {
                                        String sourceSystem = val.getSourceSystem();
                                        if (IS_VALID_SYSTEM.test(sourceSystem)) {
                                            before.incrementAndGet();
                                        }
                                        return sourceSystem;
                                    });
                            replaceInfo.setSourceSystemNames(sourceSystemNames);
                            if (IS_VALID_SYSTEM.test(replaceInfo.getTargetSystemName())) {
                                after.incrementAndGet();
                            }
                            return replaceInfo;
                        }).collect(Collectors.toList());
                projectSystem.setReplaceSystems(replaceInfoList);

            }
        });
        retDetail.setBeforeReplace(before.get());
        retDetail.setAfterReplace(after.get());
        retDetail.setReplaceDetail(retList);
        return retDetail;
    }

    public void downTmp(HttpServletResponse response) {
        List<ProjectGovSystemReplaceInfos> replaceInfos = systemReplaceInfosService.list();
        if (replaceInfos.isEmpty()) {
            throw ReturnException.wrap("暂无数据可供导出");
        }
        for (ProjectGovSystemReplaceInfos replaceInfo : replaceInfos) {
            if (StrUtil.isNotBlank(replaceInfo.getReplaceType())) {
                String replaceType = replaceInfo.getReplaceType();
                String[] split = replaceType.split(",");
                String replaceTypeStr = Arrays.stream(split)
                        .filter(NumberUtil::isNumber)
                        .map(w -> GovSystemReplaceTypeEnum.getVal(Integer.parseInt(w)))
                        .collect(Collectors.joining(","));
                replaceInfo.setReplaceType(replaceTypeStr);
            } else {
                replaceInfo.setReplaceType(StrUtil.EMPTY);
            }
        }
        try (ServletOutputStream sos = response.getOutputStream()) {
            response.setContentType("application/vnd.ms-excel");
            ExcelUtil.getWriter()
                    .write(replaceInfos)
                    .flush(sos)
                    .close();
        } catch (Exception e) {
            throw ReturnException.wrap("导出失败");
        }
    }

}
