package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.hz.pm.api.common.compare.CompareUtils;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.model.entity.ExcelExportWriter;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.expert.model.entity.ExpertReview;
import com.hz.pm.api.expert.model.enumeration.ReviewTemplateTypeEnum;
import com.hz.pm.api.expert.service.IExpertReviewService;
import com.hz.pm.api.filemanage.model.entity.ProjectApplyBorrow;
import com.hz.pm.api.filemanage.service.INdProjectApplyBorrowService;
import com.hz.pm.api.meeting.entity.domain.MeetingInnerProject;
import com.hz.pm.api.meeting.entity.dto.ProjectReviewResultDTO;
import com.hz.pm.api.meeting.service.IMeetingInnerProjectService;
import com.hz.pm.api.performance.model.entity.ProjectCoreBusinessIndicators;
import com.hz.pm.api.performance.service.IProjectCoreBusinessIndicatorsService;
import com.hz.pm.api.portrait.model.entity.ProjectTag;
import com.hz.pm.api.portrait.model.entity.Tag;
import com.hz.pm.api.portrait.model.vo.TagVO;
import com.hz.pm.api.portrait.service.IProjectTagService;
import com.hz.pm.api.portrait.service.ITagService;
import com.hz.pm.api.projectdeclared.manage.ConstructionManage;
import com.hz.pm.api.projectdeclared.model.dto.DelayedApplyDTO;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.PreInsAcceptancePerson;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.model.vo.*;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectdeclared.service.IPreInsAcceptancePersonService;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.ProjectCodeGenUtil;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.handle.ProcessProgressHelper;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.helper.ProjectSaveHelper;
import com.hz.pm.api.projectlib.model.constant.ProjectConstant;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoUpgradeContentDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectPaymentPlanDTO;
import com.hz.pm.api.projectlib.model.entity.*;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.ProjectRenewalApprovalStatusEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import com.hz.pm.api.projectlib.model.po.ProjectPO;
import com.hz.pm.api.projectlib.model.req.BindSystemToBidReq;
import com.hz.pm.api.projectlib.model.req.ProjectApplicationListReq;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.*;
import com.hz.pm.api.projectlib.service.*;
import com.hz.pm.api.todocenter.model.req.ProcessDetailReq;
import com.hz.pm.api.todocenter.service.ITodoService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.service.FileService;
import com.wflow.exception.BusinessException;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.flowable.task.api.TaskInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * ProjectLibManage
 * </p>
 *
 * @author WendyYang
 * @since 14:19 2023/2/1
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class ProjectLibManage {

    private final ProjectSaveHelper projectSaveHelper;
    private final IProjectService projectService;
    private final IProjectApplicationService applicationService;
    private final IProjectRenewalFundDeclarationService renewalFundDeclarationService;
    private final ProcessProgressHelper processExecuteHandle;
    private final FileService fileService;
    private final IProjectApplicationService projectApplicationService;
    private final ProjectCodeGenUtil projectCodeGenUtil;
    private final IProjectInstService projectInstService;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final INdProjectDelayApplyService projectDelayApplyService;
    private final INdProjectApplyBorrowService projectApplyBorrowService;
    private final IPurchaseService purchaseService;
    private final IPreInsAcceptancePersonService acceptancePersonService;
    private final ITodoService todoService;
    private final IProjectCoreBusinessIndicatorsService projectCoreBusinessIndicatorsService;
    private final IProjectAnnualPaymentPlanService projectAnnualPaymentPlanService;
    private final IOperationService operationService;
    private final IProjectTagService projectTagService;
    private final ITagService tagService;
    private final TaskService taskService;
    private final UserInfoHelper userInfoHelper;
    private final IExpertReviewService expertReviewService;
    private final ConstructionManage constructionManage;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final MhUnitCache mhUnitCache;
    private final IMhProjectSchemaTargetDataService projectSchemaTargetDataService;
    private final IMhProjectService mhProjectService;
    private final IMeetingInnerProjectService meetingInnerProjectService;
    private final AmountApprovalManage amountApprovalManage;
    private final IPurchaseInstService purchaseInstService;

    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        query.eq(Project::getNewest, Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        Set<String> projectCodes = new HashSet<>();
        List<Long> projectIds = new ArrayList<>();
        for (Project project : page.getRecords()) {
            projectIds.add(project.getId());
            projectCodes.add(project.getProjectCode());
        }
        Map<Long, List<ProjectRenewalFundDeclaration>> renewalMap = listRenewDeclarations(projectCodes);
        Map<Long, List<ProjectAnnualPaymentPlan>> paymentPlansMap = projectAnnualPaymentPlanService.listByProjectIds(projectIds);

        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            List<ProjectAnnualPaymentPlan> paymentPlans = paymentPlansMap.getOrDefault(w.getId(), Collections.emptyList());
            BigDecimal totalAnnualPaymentAmount = paymentPlans.stream()
                    .map(ProjectAnnualPaymentPlan::getAnnualPlanAmount)
                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);

            List<ProjectRenewalFundDeclaration> fundDeclarations = renewalMap.get(w.getId());
            if (fundDeclarations != null) {
                item.setAnnualAccumulateAmount(computeAmount(totalAnnualPaymentAmount, fundDeclarations));
                item.setAnnualAccumulateAmountList(convertAccumulate(w, totalAnnualPaymentAmount, fundDeclarations));
            } else {
                item.setAnnualAccumulateAmount(totalAnnualPaymentAmount);
                item.setAnnualAccumulateAmountList(convertAccumulate(w, totalAnnualPaymentAmount, Collections.emptyList()));
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    public PageVo<ProjectLibListItemVO> projectLibListWithPermission(ProjectListReq req, UserFullInfoDTO user) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        if (!buildProjectLibPermission(query, user)) {
            return PageVo.empty();
        }
        //项目查最新
        query.eq(Project::getNewest, Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        Set<String> projectCodes = new HashSet<>();
        List<Long> projectIds = new ArrayList<>();
        for (Project project : page.getRecords()) {
            projectIds.add(project.getId());
            projectCodes.add(project.getProjectCode());
        }
        Map<String, List<TagVO>> projectTagMap = getProjectTags(projectCodes);
        Map<Long, List<ProjectAnnualPaymentPlan>> paymentPlansMap = projectAnnualPaymentPlanService.listByProjectIds(projectIds);

        Map<Long, List<ProjectRenewalFundDeclaration>> renewalMap = listRenewDeclarations(projectCodes);
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            if (user.isOrgAdmin() && StringUtils.isNotBlank(w.getSuperOrgCode())
                    && w.getSuperOrgCode().equals(user.getMhUnitIdStr())) {
                item.setCanPreDeclared(Boolean.TRUE);
            }
            item.setApprovedAmount(w.getApprovalAmount());
            List<ProjectAnnualPaymentPlan> paymentPlans = paymentPlansMap.getOrDefault(w.getId(), Collections.emptyList());
            BigDecimal totalAnnualPaymentAmount = paymentPlans.stream()
                    .map(ProjectAnnualPaymentPlan::getAnnualPlanAmount)
                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
            item.setAnnualPlanAmount(totalAnnualPaymentAmount);
            item.setPrePlanProjectId(w.getPrePlanProjectId());
            item.setTags(projectTagMap.get(w.getProjectCode()));
            List<ProjectRenewalFundDeclaration> fundDeclarations = renewalMap.get(w.getId());
            if (fundDeclarations != null) {
                item.setAnnualAccumulateAmount(computeAmount(totalAnnualPaymentAmount, fundDeclarations));
                item.setAnnualAccumulateAmountList(convertAccumulate(w, totalAnnualPaymentAmount, fundDeclarations));
            } else {
                item.setAnnualAccumulateAmount(totalAnnualPaymentAmount);
                item.setAnnualAccumulateAmountList(convertAccumulate(w, totalAnnualPaymentAmount, Collections.emptyList()));
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    /**
     * 项目列表带当前审批人(工作流的)
     *
     * @param req  \
     * @param user \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> projectLibListApprove(ProjectListReq req, UserFullInfoDTO user) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        if (!buildProjectLibPermission(query, user)) {
            return PageVo.empty();
        }
        //项目查最新
        query.eq(Project::getNewest, Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);
        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        Set<String> instCodes = page.getRecords().stream()
                .map(Project::getInstCode)
                .filter(StringUtils::isNotBlank).collect(Collectors.toSet());
        List<Task> tasks = taskService.createTaskQuery()
                .processInstanceIdIn(instCodes)
                .orderByTaskCreateTime()
                .asc()
                .list();
        Map<String, List<Task>> map = Maps.newHashMap();
        Map<Long, UserFullInfoDTO> userMap = Maps.newHashMap();
        if (CollUtil.isNotEmpty(tasks)) {
            map = CollUtils.group(tasks, Task::getProcessInstanceId);
            userMap = searchUser(tasks);
        }

        List<ProjectLibListItemVO> records = Lists.newArrayList();
        for (Project w : page.getRecords()) {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setApproveUsers(buildApproveUsers(w.getInstCode(), map, userMap));
            if (user.isOrgAdmin() && StringUtils.isNotBlank(w.getSuperOrgCode())
                    && w.getSuperOrgCode().equals(user.getMhUnitIdStr())) {
                item.setCanPreDeclared(Boolean.TRUE);
            }
            item.setApprovedAmount(w.getApprovalAmount());
            item.setPrePlanProjectId(w.getPrePlanProjectId());
            records.add(item);
        }
        return PageVo.of(records, page.getTotal());
    }

    private List<UserFullInfoDTO> buildApproveUsers(String instCode, Map<String, List<Task>> map,
                                                    Map<Long, UserFullInfoDTO> userMap) {
        if (map.containsKey(instCode)) {
            List<Task> tasks = map.get(instCode);
            return tasks.stream().map(TaskInfo::getAssignee)
                    .filter(NumberUtil::isNumber)
                    .map(w -> userMap.get(Long.parseLong(w)))
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    private Map<Long, UserFullInfoDTO> searchUser(List<Task> tasks) {
        if (CollUtil.isEmpty(tasks)) {
            return Collections.emptyMap();
        }
        Set<Long> userIds = tasks.stream()
                .filter(w -> NumberUtil.isNumber(w.getAssignee()))
                .map(w -> Long.parseLong(w.getAssignee()))
                .collect(Collectors.toSet());
        if (userIds.isEmpty()) {
            return Collections.emptyMap();
        }
        List<UserFullInfoDTO> users = userInfoHelper.listUserFullInfoByUserIds(userIds);
        return CollUtils.listToMap(users, UserFullInfoDTO::getUserId);
    }

    private Map<String, List<TagVO>> getProjectTags(Collection<String> projectCodes) {
        Map<String, List<ProjectTag>> projectTagMap = projectTagService.listByProjectCodes(projectCodes);
        if (CollUtil.isNotEmpty(projectTagMap)) {
            Set<Long> tagIds = projectTagMap.values().stream()
                    .flatMap(Collection::stream)
                    .map(ProjectTag::getTagId)
                    .collect(Collectors.toSet());
            List<Tag> tags = tagService.listByIds(tagIds);
            Map<Long, Tag> tagMap = CollUtils.listToMap(tags, Tag::getId);
            return projectTagMap.entrySet().stream()
                    .collect(Collectors.toMap(Map.Entry::getKey, w -> w.getValue().stream().map(tg -> {
                        Tag tag = tagMap.get(tg.getTagId());
                        if (tag == null) {
                            return null;
                        }
                        TagVO tagVO = new TagVO();
                        tagVO.setId(tag.getId());
                        tagVO.setName(tag.getName());
                        return tagVO;
                    }).filter(Objects::nonNull).collect(Collectors.toList())));
        }
        return Collections.emptyMap();
    }

    /**
     * 项目申报和项目归集的并集
     *
     * @param req  \
     * @param user \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> projectAllListWithPermission(ProjectListReq req, UserFullInfoDTO user) {
        if (!buildProjectLibPermission(req, user)) {
            return PageVo.empty();
        }
        Page<ProjectPO> page = projectService.pageAllWithPermission(req.page(), req);
        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        Set<String> projectCodes = CollUtils.fieldSet(page.getRecords(), ProjectPO::getProjectCode);
        Map<String, List<TagVO>> projectTagsMap = getProjectTags(projectCodes);
        Map<Long, List<ProjectRenewalFundDeclaration>> renewalMap = listRenewDeclarations(projectCodes);

        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setFromType(w.getFromType());
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setReviewAmount(w.getReviewAmount());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            if (user.isOrgAdmin() && StringUtils.isNotBlank(w.getSuperOrgCode())
                    && w.getSuperOrgCode().equals(user.getMhUnitIdStr())) {
                item.setCanPreDeclared(Boolean.TRUE);
            }
            item.setApprovedAmount(w.getApprovalAmount());
            item.setAnnualPlanAmount(w.getAnnualPlanAmount());
            item.setPrePlanProjectId(w.getPrePlanProjectId());
            item.setTags(projectTagsMap.get(w.getProjectCode()));
            List<ProjectRenewalFundDeclaration> currRenewDeclarations = renewalMap.get(item.getId());
            if (currRenewDeclarations != null) {
                item.setAnnualAccumulateAmount(computeAmount(w.getAnnualPlanAmount(), currRenewDeclarations));
                item.setAnnualAccumulateAmountList(convertAccumulate(w, currRenewDeclarations));
            } else {
                item.setAnnualAccumulateAmount(w.getAnnualPlanAmount());
                item.setAnnualAccumulateAmountList(convertAccumulate(w, Collections.emptyList()));
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    public PageVo<ProjectLibListItemVO> libListWithPermission(ProjectListReq req, UserFullInfoDTO user) {
        if (!buildProjectLibPermission(req, user)) {
            return PageVo.empty();
        }
        if (req.getDeclaredUnitId() != null) {
            List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getDeclaredUnitId());
            req.setDeclaredUnitIds(viewUnitIds);
        }
        req.setNotInStatusList(ProjectStatusConstant.CHANGE_DELETE_PASSED);
        Page<ProjectPO> page = projectService.pageLibWithPermission(req.page(), req);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        Set<String> projectCodes = page.getRecords().stream()
                .map(ProjectPO::getProjectCode).collect(Collectors.toSet());
        Map<String, List<TagVO>> projectTagsMap = getProjectTags(projectCodes);
        Map<Long, List<ProjectRenewalFundDeclaration>> renewalMap = listRenewDeclarations(projectCodes);

        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setFromType(w.getFromType());
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setReviewAmount(w.getReviewAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            if (user.isOrgAdmin() && StringUtils.isNotBlank(w.getSuperOrgCode())
                    && w.getSuperOrgCode().equals(user.getMhUnitIdStr())) {
                item.setCanPreDeclared(Boolean.TRUE);
            }
            item.setApprovedAmount(w.getApprovalAmount());
            item.setAnnualPlanAmount(w.getAnnualPlanAmount());
            item.setPrePlanProjectId(w.getPrePlanProjectId());
            item.setTags(projectTagsMap.get(w.getProjectCode()));
            List<ProjectRenewalFundDeclaration> currRenewDeclarations = renewalMap.get(item.getId());
            if (currRenewDeclarations != null) {
                item.setAnnualAccumulateAmount(computeAmount(w.getAnnualPlanAmount(), currRenewDeclarations));
                item.setAnnualAccumulateAmountList(convertAccumulate(w, currRenewDeclarations));
            } else {
                item.setAnnualAccumulateAmount(w.getAnnualPlanAmount());
                item.setAnnualAccumulateAmountList(convertAccumulate(w, Collections.emptyList()));
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    private Map<Long, List<ProjectRenewalFundDeclaration>> listRenewDeclarations(Collection<String> projectCodes) {
        Map<Long, List<ProjectRenewalFundDeclaration>> renewalMap;
        if (CollUtil.isNotEmpty(projectCodes)) {
            List<ProjectRenewalFundDeclaration> renewalDeclarations = renewalFundDeclarationService.list(Wrappers.lambdaQuery(ProjectRenewalFundDeclaration.class)
                    .in(ProjectRenewalFundDeclaration::getProjectCode, projectCodes)
                    .eq(ProjectRenewalFundDeclaration::getApprovalStatus, ProjectRenewalApprovalStatusEnum.PASS.name())
                    .eq(ProjectRenewalFundDeclaration::getDeleted, Boolean.FALSE)
                    .orderByAsc(ProjectRenewalFundDeclaration::getProjectYear));
            renewalMap = CollUtils.group(renewalDeclarations, ProjectRenewalFundDeclaration::getProjectId);
        } else {
            renewalMap = Collections.emptyMap();
        }
        return renewalMap;
    }

    public PageVo<ProjectLibListItemVO> listWithPermissionWorkBench(ProjectListReq req,
                                                                    UserFullInfoDTO user) {
        if (!buildProjectLibPermission(req, user)) {
            return PageVo.empty();
        }
        Page<ProjectPO> page = projectService.pageLibWithPermission(req.page(), req);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }

        Set<String> instCodes = page.getRecords().stream()
                .map(ProjectPO::getInstCode)
                .filter(StringUtils::isNotBlank)
                .collect(Collectors.toSet());
        List<Task> tasks;
        if (!instCodes.isEmpty()) {
            tasks = taskService.createTaskQuery()
                    .processInstanceIdIn(instCodes)
                    .orderByTaskCreateTime()
                    .asc()
                    .list();
        } else {
            tasks = Collections.emptyList();
        }

        Map<String, List<Task>> map;
        Map<Long, UserFullInfoDTO> userMap;
        if (CollUtil.isNotEmpty(tasks)) {
            map = CollUtils.group(tasks, Task::getProcessInstanceId);
            userMap = searchUser(tasks);
        } else {
            map = Collections.emptyMap();
            userMap = Collections.emptyMap();
        }

        List<ProjectLibListItemVO> records = Lists.newArrayList();
        for (ProjectPO w : page.getRecords()) {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setApproveUsers(buildApproveUsers(w.getInstCode(), map, userMap));
            if (user.isOrgAdmin() && StringUtils.isNotBlank(w.getSuperOrgCode())
                    && w.getSuperOrgCode().equals(user.getMhUnitIdStr())) {
                item.setCanPreDeclared(Boolean.TRUE);
            }
            item.setApprovedAmount(w.getApprovalAmount());
            item.setAnnualPlanAmount(w.getAnnualPlanAmount());
            item.setPrePlanProjectId(w.getPrePlanProjectId());
            records.add(item);
        }
        return PageVo.of(records, page.getTotal());
    }

    public static BigDecimal computeAmount(BigDecimal currentAmount, List<ProjectRenewalFundDeclaration> prfs) {
        BigDecimal res = Objects.isNull(currentAmount) ? BigDecimal.ZERO : currentAmount;
        for (ProjectRenewalFundDeclaration prf : prfs) {
            res = res.add(prf.getAnnualPaymentAmount());
        }
        return res;
    }

    public static List<AnnualAmountVO> convertAccumulate(ProjectPO project, List<ProjectRenewalFundDeclaration> prfs) {
        List<AnnualAmountVO> annualAmounts = Lists.newArrayList();
        annualAmounts.add(AnnualAmountVO.builder()
                .projectId(project.getId())
                .projectYear(project.getProjectYear())
                .annualAmount(project.getAnnualPlanAmount())
                .build());
        for (ProjectRenewalFundDeclaration prf : prfs) {
            annualAmounts.add(AnnualAmountVO.builder()
                    .projectId(prf.getProjectId())
                    .projectYear(prf.getProjectYear())
                    .annualAmount(prf.getAnnualPaymentAmount())
                    .build());
        }
        return annualAmounts;
    }

    public static List<AnnualAmountVO> convertAccumulate(Project project, BigDecimal totalAnnualPlanAmount, List<ProjectRenewalFundDeclaration> prfs) {
        List<AnnualAmountVO> annualAmounts = Lists.newArrayList();
        annualAmounts.add(AnnualAmountVO.builder()
                .projectId(project.getId())
                .projectYear(project.getProjectYear())
                .annualAmount(totalAnnualPlanAmount)
                .build());
        for (ProjectRenewalFundDeclaration prf : prfs) {
            annualAmounts.add(AnnualAmountVO.builder()
                    .projectId(prf.getProjectId())
                    .projectYear(prf.getProjectYear())
                    .annualAmount(prf.getAnnualPaymentAmount())
                    .build());
        }
        return annualAmounts;
    }

    public List<Project> projectList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        return projectService.list(query);
    }

    /**
     * @param projectId 项目详情
     * @return com.hz.pm.api.projectlib.model.entity.vo.ProjectDetailVO
     * @author ZPF
     * @since 2023/02/11 11:15
     */
    public ProjectDetailVO getProjectDetail(Long projectId) {
        // 查询最新的项目申报信息
        Project project = projectService.getNewProject(projectId);
        if (Objects.isNull(project)) {
            return null;
        }
        return getProjectDetailLogic(project);
    }

    public ProjectDetailVO getProjectDetail(Project project) {
        return getProjectDetailLogic(project);
    }

    public ProjectDetailVO detailProjectCode(String projectCode) {
        // 查询最新的项目申报信息
        Long projectId = ProjectIdCodeCacheUtil.newest(projectCode);
        Project project = projectService.getById(projectId);
        return project == null ? null : getProjectDetailLogic(project);
    }

    /**
     * @param projectId 项目详情 不查询最新版本
     * @return com.hz.pm.api.projectlib.model.entity.vo.ProjectDetailVO
     * @author ZPF
     * @since 2023/02/11 11:15
     */
    public ProjectDetailVO getProjectDetailThisVersion(Long projectId) {
        // 查询最新的项目申报信息
        Project project = projectService.getById(projectId);
        return project == null ? null : getProjectDetailLogic(project);
    }

    /**
     * 获取政务信息系统改造情况
     *
     * @param projectCode \
     * @return \
     */
    private List<GovSystemReplaceInfoDTO> getGovSystemReplaceInfos(String projectCode, String upgradeContent) {
        Wrapper<ProjectGovSystemReplaceInfos> query = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .eq(ProjectGovSystemReplaceInfos::getProjectCode, projectCode)
                .orderByAsc(ProjectGovSystemReplaceInfos::getId)
                .orderByAsc(ProjectGovSystemReplaceInfos::getBatchNo);
        List<ProjectGovSystemReplaceInfos> govSystemReplaceInfos = systemReplaceInfosService.list(query);
        if (govSystemReplaceInfos.isEmpty()) {
            return Collections.emptyList();
        }
        Map<Integer, GovSystemReplaceInfoUpgradeContentDTO> upgradeContentMap = new HashMap<>();
        if (StrUtil.isNotBlank(upgradeContent)) {
            List<GovSystemReplaceInfoUpgradeContentDTO> upgradeContents = JSONUtil.toList(upgradeContent, GovSystemReplaceInfoUpgradeContentDTO.class);
            upgradeContentMap.putAll(CollUtils.listToMap(upgradeContents, GovSystemReplaceInfoUpgradeContentDTO::getBatchNo));
        }
        Map<Integer, List<ProjectGovSystemReplaceInfos>> replaceInfoGroup = CollUtils.group(govSystemReplaceInfos, ProjectGovSystemReplaceInfos::getBatchNo);
        return replaceInfoGroup.entrySet().stream()
                .sorted(Comparator.comparingInt(Map.Entry::getKey))
                .map(w -> {
                    List<ProjectGovSystemReplaceInfos> currBatchValues = w.getValue();
                    ProjectGovSystemReplaceInfos firstReplaceInfo = currBatchValues.get(0);
                    GovSystemReplaceInfoDTO replaceInfo = new GovSystemReplaceInfoDTO();
                    replaceInfo.setReplaceType(firstReplaceInfo.getReplaceType());
                    replaceInfo.setTargetSystemName(firstReplaceInfo.getTargetSystem());
                    List<String> sourceSystemNames = new ArrayList<>();
                    List<String> sourceSystemIds = new ArrayList<>();
                    for (ProjectGovSystemReplaceInfos replaceInfos : currBatchValues) {
                        sourceSystemIds.add(StrUtil.nullToEmpty(replaceInfos.getSourceSystemId()));
                        sourceSystemNames.add(replaceInfos.getSourceSystem());
                    }
                    replaceInfo.setSourceSystemNames(sourceSystemNames);
                    replaceInfo.setSourceSystemIds(sourceSystemIds);
                    Optional.ofNullable(upgradeContentMap.get(w.getKey())).ifPresent(uc -> {
                        replaceInfo.setIterativeUpgradeContent(uc.getIterativeUpgradeContent());
                        replaceInfo.setIsAddIterativeUpgradeContent(uc.getIsAddIterativeUpgradeContent());
                    });
                    return replaceInfo;
                }).collect(Collectors.toList());
    }

    private ProjectDetailVO getProjectDetailLogic(Project project) {
        ProjectDetailVO retProjectDetail = new ProjectDetailVO();
        BeanUtils.copyProperties(project, retProjectDetail);
        retProjectDetail.buildDynamicForm(project.getDynamicForm());
        String projectCode = project.getProjectCode();
        if (ProjectStatus.CHANGE_APPLY_PASSED.eq(project.getStatus())) {
            Wrapper<Project> newProjectQuery = Wrappers.lambdaQuery(Project.class)
                    .select(Project::getId, Project::getProjectCode, Project::getProjectName)
                    .eq(Project::getHisProjectCode, project.getProjectCode())
                    .eq(Project::getNewest, Boolean.TRUE)
                    .last(BizConst.LIMIT_1);
            Project newProject = projectService.getOne(newProjectQuery);
            retProjectDetail.setNewProjectId(newProject.getId());
            retProjectDetail.setNewProjectName(newProject.getProjectName());
            retProjectDetail.setNewProjectCode(newProject.getProjectCode());
        }
        if (StrUtil.isNotBlank(project.getHisProjectCode())) {
            retProjectDetail.setHisProjectName(projectService.getProjectName(project.getHisProjectCode()));
            retProjectDetail.setHisProjectId(ProjectIdCodeCacheUtil.newest(project.getHisProjectCode()));
        }
        if (DecimalUtil.noNullAndGtZero(project.getApprovalAmount())) {
            retProjectDetail.setApprovalAmounts(amountApprovalManage.listApprovalAmounts(projectCode));
        }
        // 查询应用
        List<ProjectApplication> applications = applicationService.list(projectCode, project.getVersion(), project.getIsConstruct());

        List<Long> applicationIds = CollUtils.fieldList(applications, ProjectApplication::getId);

        //核心业务
        if (CollUtil.isNotEmpty(applicationIds)) {
            List<ProjectCoreBusinessIndicators> coreBizs = projectCoreBusinessIndicatorsService.listByApplicationIds(applicationIds);
            Map<Long, List<ProjectCoreBusinessIndicators>> coreBizMap = CollUtils.group(coreBizs, ProjectCoreBusinessIndicators::getApplicationId);
            retProjectDetail.setProjectApplications(CollUtils.convert(applications, app -> ProjectManageUtil.to(app, coreBizMap)));
        }
        retProjectDetail.setSystemReplaceInfos(getGovSystemReplaceInfos(project.getProjectCode(), project.getIterativeUpgradeContent()));
        //审批信息
        if (StrUtils.isNotBlank(project.getInstCode())) {
            ProcessDetailReq req = new ProcessDetailReq();
            req.setInstanceId(project.getInstCode());
            req.setProjectId(project.getId());
            retProjectDetail.setProcess(todoService.getProcessDetail(req));
        }

        List<Long> allVersionProjectIds = projectService.allVersionProjectIds(projectCode);
        // 查询采购备案
        retProjectDetail.setPurchaseDetails(new ArrayList<>());
        List<Purchase> purchases = purchaseService.listByProjectIds(allVersionProjectIds);
        if (!purchases.isEmpty()) {
            Map<Long, PurchaseVO> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId, w -> {
                PurchaseVO purchase = BeanUtil.copyProperties(w, PurchaseVO.class);
                purchase.setStatusName(TenderMainStatus.getDesc(w.getStatus()));
                return purchase;
            });
            Map<Long, PurchaseInst> purchaseInstMap = purchaseInstService.listNewest(purchaseMap.keySet(), InstTypeEnum.PURCHASE_NOTICE.getCode());
            //查询合同备案
            List<ContractVO> contracts = constructionManage.detailContractByBidIds(purchaseMap.keySet());
            Map<Long, ContractVO> contractMap = CollUtils.listToMap(contracts, ContractVO::getBidId);

            //查询实施信息
            List<Operation> operations = operationService.list(Wrappers.lambdaQuery(Operation.class)
                    .in(Operation::getBidId, purchaseMap.keySet()));
            List<OperationVO> tmpOperations = BeanUtil.copyToList(operations, OperationVO.class);
            Map<Long, OperationVO> operationMap = CollUtils.listToMap(tmpOperations, OperationVO::getBidId);

            //查询初验信息
            Map<Long, List<PreInsAcceptancePerson>> acceptPersonsMap = acceptancePersonService.listByBidIds(purchaseMap.keySet());
            List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.listByBidIds(purchaseMap.keySet());
            Map<Long, List<ProjectGovSystemReplaceInfos>> replaceInfosMap = CollUtils.group(systemReplaceInfos, ProjectGovSystemReplaceInfos::getBidId);
            purchaseMap.forEach((k, v) -> {
                // 赋值最新的采购公告流程实例编码
                BizUtils.notNull(purchaseInstMap.get(v.getId()), w -> v.setPurchaseNoticeInstCode(w.getInstCode()));
                PurchaseFullInfoVO purchaseFullInfo = new PurchaseFullInfoVO();
                purchaseFullInfo.setPurchase(v);
                purchaseFullInfo.setOperation(operationMap.get(k));
                purchaseFullInfo.setConstruction(contractMap.get(k));
                List<PreInsAcceptancePerson> tmpPersons = acceptPersonsMap.get(k);
                if (tmpPersons != null) {
                    purchaseFullInfo.setFirstInspectedPersons(convertPersons(tmpPersons));
                }
                purchaseFullInfo.setSystemReplaceInfos(replaceInfosMap.get(k));
                retProjectDetail.getPurchaseDetails().add(purchaseFullInfo);
            });
            // 建设方案
            mhProjectService.getIdByProjectCode(projectCode).ifPresent(w -> {
                MhProjectSchemaTargetData schemaTargetData = projectSchemaTargetDataService.getByMhProjectId(w);
                ConstructionDetailVO construction = new ConstructionDetailVO();
                BeanUtil.copyProperties(schemaTargetData, construction);
                retProjectDetail.setConstruction(construction);
            });
        }

        List<ProjectAnnualPaymentPlan> annualPaymentPlans = projectAnnualPaymentPlanService.listByProjectId(project.getId());
        retProjectDetail.setAnnualPaymentPlans(BeanUtil.copyToList(annualPaymentPlans, ProjectPaymentPlanDTO.class));

        // 处理文件名
        List<Long> fileIdList = new ArrayList<>();
        BizUtils.notBlank(retProjectDetail.getApprovedFile(), w -> fileIdList.add(Long.parseLong(w)));
        BizUtils.notBlank(retProjectDetail.getApprovedConstructionPlanFile(), w -> fileIdList.add(Long.parseLong(w)));
        if (!fileIdList.isEmpty()) {
            List<File> files = fileService.listByIds(fileIdList);
            Map<String, String> fileMap = CollUtils.listToMap(files, w -> w.getId().toString(), File::getOriginalFileName);
            retProjectDetail.setApprovedFileName(fileMap.get(retProjectDetail.getApprovedFile()));
            retProjectDetail.setApprovedConstructionPlanFileName(fileMap.get(retProjectDetail.getApprovedConstructionPlanFile()));
        }
        //专家评审 其它建议 -> 建设方案的会议
        ExpertReview finalReview = expertReviewService.getFinalReview(projectCode,
                ReviewTemplateTypeEnum.CONSTRUCTION_SCHEME_REVIEW.getCode());

        if (Objects.nonNull(finalReview)) {
            retProjectDetail.setProposeAttach(finalReview.getOtherAdvice());
            retProjectDetail.setBaseReviewResults(finalReview.getReviewResult());
        }

        // 评审详情
        Wrapper<MeetingInnerProject> mipQuery = Wrappers.lambdaQuery(MeetingInnerProject.class)
                .eq(MeetingInnerProject::getProjectCode, project.getProjectCode())
                .exists(ExistsSqlConst.MEETING_INNER_PROJECT_EXISTS_MEETING + "and m.status != 3")
                .isNotNull(MeetingInnerProject::getReviewResult)
                .orderByDesc(MeetingInnerProject::getId);
        List<MeetingInnerProject> mip = meetingInnerProjectService.list(mipQuery);
        retProjectDetail.setReviewDetail(BeanUtil.copyToList(mip, ProjectReviewResultDTO.class));

        return retProjectDetail;
    }


    /**
     * 申报新项目时 保存项目信息和其它相关联的信息
     *
     * @param proj     \
     * @param instCode \
     * @return \
     */
    public Project saveProjectInDeclared(ProjectDTO proj, String instCode, Long userId) {
        Project project = saveProjectNewVersion(proj, instCode, userId, Boolean.FALSE);
        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instCode);
        projectInst.setCreateOn(LocalDateTime.now());
        projectInst.setUpdateOn(LocalDateTime.now());
        projectInst.setInstType(ProjectProcessType.COMPLIANCE_REVIEW.getCode());
        projectInstService.save(projectInst);
        return project;
    }

    /**
     * 申报新项目时 保存项目信息和其它相关联的信息
     *
     * @param proj     \
     * @param instCode \
     * @return \
     */
    public Project saveProjectInDeclared(ProjectDTO proj, String instCode, Long userId,
                                         ProjectStatus stage, ProjectStatus status,
                                         ProjectProcessType instType) {
        Project project = saveProjectNewVersion(proj, instCode, userId, Boolean.FALSE, stage, status);
        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instCode);
        projectInst.setInstType(instType.getCode());
        projectInstService.save(projectInst);
        return project;
    }

    /**
     * 申报新项目时 保存项目信息和其它相关联的信息
     *
     * @param projectDto \
     * @param instanceId \
     * @return \
     */
    private Project saveProjectNewVersion(ProjectDTO projectDto, String instanceId,
                                          Long userId, Boolean isContruct) {
        //流程启动之后 入库项目 重要业务信息 用于列表查询 展示
        return saveProjectNewVersion(projectDto, instanceId, userId, isContruct, ProjectStatus.NOT_APPROVED, ProjectStatus.TO_BE_DECLARED);
    }

    public Project saveProjectNewVersion(ProjectDTO projectDto, String instanceId,
                                         Long userId, Boolean isConstruct,
                                         ProjectStatus stage,
                                         ProjectStatus status) {
        //流程启动之后 入库项目 重要业务信息 用于列表查询 展示
        try {
            Project project;
            //为空 代表是新申报的
            if (Objects.isNull(projectDto.getId())) {
                project = new Project();
                BeanUtils.copyProperties(projectDto, project);
                project.setCreateOn(LocalDateTime.now());
                project.setUpdateOn(LocalDateTime.now());
                project.setStage(stage.getCode());
                project.setStatus(status.getCode());
                project.setInstCode(instanceId);
                project.setSponsor(String.valueOf(userId));
                String projectCode = projectCodeGenUtil.generateProjectCode(projectDto);
                project.setProjectCode(projectCode);
                project.setNewest(Boolean.TRUE);
                projectStateMachineUtil.pass(project);
                if (projectService.save(project)) {
                    projectSaveHelper.saveApplication(projectDto, project, null);
                    projectSaveHelper.savePaymentPlans(project, projectDto.getPaymentPlanList());
                    projectSaveHelper.saveGovSystemReplaceInfos(project.getId(), project.getProjectCode(), projectDto.getSystemReplaceInfos());
                }
            } else {
                //否则是重新提交的 新生成一个新版本的项目
                project = newProjectWithVersion(projectDto, isConstruct);
                if (Objects.nonNull(project)) {
                    project.setInstCode(instanceId);
                    project.setSponsor(String.valueOf(userId));
                    projectService.updateById(project);
                }
            }
            return project;
        } catch (Exception e) {
            log.error("项目信息入库错误", e);
            throw BizException.wrap("项目信息入库错误");
        }
    }

    public Project saveProjectNewVersion(ProjectDTO reqProj,
                                         Long userId,
                                         ProjectStatus status) {
        try {
            Project newProj = new Project();
            BeanUtils.copyProperties(reqProj, newProj);
            newProj.setCreateOn(LocalDateTime.now());
            newProj.setUpdateOn(LocalDateTime.now());
            newProj.setIsBackReject(Boolean.FALSE);
            newProj.setStage(status.getStage());
            newProj.setStatus(status.getCode());
            newProj.setSponsor(String.valueOf(userId));
            if (Objects.isNull(reqProj.getId())) {
                newProj.setProjectCode(projectCodeGenUtil.generateProjectCode(reqProj));
            } else {
                Project oldProj = projectService.getById(reqProj.getId());
                if (ProjectStatus.TO_BE_DECLARED.eq(status)
                        && !ProjectStatus.TO_BE_DECLARED.eq(oldProj.getStatus())) {
                    throw ReturnException.wrap("项目状态异常，保存失败");
                }
                newProj.setVersion(oldProj.getVersion() + 1);
            }
            newProj.setId(null);
            newProj.setNewest(Boolean.TRUE);
            if (projectService.save(newProj)) {
                if (Objects.nonNull(reqProj.getId())) {
                    projectService.reverseNewest(reqProj.getProjectCode(), newProj.getId());
                }
                projectSaveHelper.saveApplication(reqProj, newProj, null);
                projectSaveHelper.savePaymentPlans(newProj, reqProj.getPaymentPlanList());
                projectSaveHelper.saveGovSystemReplaceInfos(newProj.getId(), newProj.getProjectCode(), reqProj.getSystemReplaceInfos());
            }
            return newProj;
        } catch (Exception e) {
            log.error("项目信息入库错误", e);
            throw BizException.wrap("项目信息入库错误");
        }
    }


    /**
     * 在其它项目阶段 保存项目信息和其它相关联的信息
     *
     * @param projectDto \
     * @return \
     */
    public Project reSaveProjectNewVersion(ProjectDTO projectDto, Boolean isConstruct) {
        //流程启动之后 入库项目 重要业务信息 用于列表查询 展示
        try {
            return newProjectWithVersion(projectDto, isConstruct);
        } catch (Exception e) {
            log.error("项目信息入库错误 ", e);
            throw new BusinessException("项目信息入库错误");
        }
    }

    /**
     * 重新提交工作流时 舍弃在原有项目修改
     * 新增一个新的项目 新的版本号
     */
    public Project newProjectWithVersion(ProjectDTO reqProj, Boolean isConstruct) {
        Project oldProj = projectService.getNoNull(reqProj.getId());
        return saveProjectWithNewVersion(reqProj, oldProj, isConstruct);
    }

    public Project newProjectWithVersion(ProjectDTO reqProj, Project oldProj, Boolean isConstruct) {
        return saveProjectWithNewVersion(reqProj, oldProj, isConstruct);
    }

    private Project saveProjectWithNewVersion(ProjectDTO reqProj, Project oldProj, Boolean isConstruct) {
        Project newProj = new Project();
        BeanUtil.copyProperties(oldProj, newProj, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        BeanUtil.copyProperties(reqProj, newProj, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        newProj.setVersion(oldProj.getVersion() + 1);
        newProj.setId(null);
        LocalDateTime now = LocalDateTime.now();
        Long userId = LoginUserUtil.getUserId();
        newProj.setCreateOn(now);
        newProj.setUpdateOn(now);
        newProj.setCreateBy(userId);
        newProj.setUpdateBy(userId);
        newProj.setIsMajorProject(Boolean.FALSE);
        newProj.setIsBackReject(Boolean.FALSE);
        projectStateMachineUtil.pass(newProj);
        projectService.save(newProj);

        oldProj.setIsBackReject(Boolean.TRUE);
        projectService.updateById(oldProj);
        projectService.reverseNewest(newProj.getProjectCode(), newProj.getId());
        projectSaveHelper.saveApplication(reqProj, newProj, isConstruct);
        projectSaveHelper.savePaymentPlans(newProj, reqProj.getPaymentPlanList());
        projectSaveHelper.saveGovSystemReplaceInfos(newProj.getId(), newProj.getProjectCode(), reqProj.getSystemReplaceInfos());
        return newProj;
    }

    public Project saveProjectWithVersionAndStatus(ProjectDTO projectDto, Integer stageCode, Integer statusCode, Boolean isConstruct) {
        Project oldProject = projectService.getNoNull(projectDto.getId());
        Project project = new Project();
        BeanUtil.copyProperties(oldProject, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        BeanUtil.copyProperties(projectDto, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        //都是在驳回 重新提交时用的
        project.setIsBackReject(Boolean.FALSE);
        if (Objects.nonNull(stageCode)) {
            project.setStage(stageCode);
        }
        if (Objects.nonNull(statusCode)) {
            project.setStatus(statusCode);
        }

        oldProject.setIsBackReject(Boolean.TRUE);
        projectService.updateById(oldProject);

        if (projectService.save(project)) {
            projectService.reverseNewest(project.getProjectCode(), project.getId());
            projectSaveHelper.saveApplication(projectDto, project, isConstruct);
        }

        return project;
    }

    @Transactional(rollbackFor = Exception.class)
    public Project saveProjectWithNewVersion(ProjectDTO projectDto, String instanceId, Integer instType, Boolean isConstruct) {
        Project oldProject = projectService.getNoNull(projectDto.getId());
        Project project = new Project();
        BeanUtil.copyProperties(oldProject, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        BeanUtil.copyProperties(projectDto, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());

        if (projectService.save(project)) {
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);

            projectService.reverseNewest(project.getProjectCode(), project.getId());

            //app
            projectSaveHelper.saveApplication(projectDto, project, isConstruct);
        }

        return project;
    }

    public Project saveProjectWithNewVersion(Project oldProject, String instanceId, Integer instType) {
        Project project = new Project();
        Assert.notNull(oldProject, "项目不存在");
        BeanUtil.copyProperties(oldProject, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());

        if (projectService.save(project)) {
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);
            projectService.reverseNewest(project.getProjectCode(), project.getId());
            projectSaveHelper.saveApplication(project, oldProject, null);
        }

        return project;
    }

    public Project saveProjectWithVersionAndPass(Project oldProject, String instanceId, ProjectDTO dto, Integer instType, Boolean isBackReject) {
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        // 标识保存的项目信息是否为建设方案申报项目
        project.setIsConstruct(Boolean.TRUE);
        project.setIsBackReject(isBackReject);
        //终验材料
        project.setFinalAcceptanceMaterials(dto.getFinalAcceptanceMaterials());
        projectStateMachineUtil.pass(project);

        oldProject.setIsBackReject(isBackReject);
        projectService.updateById(oldProject);

        if (projectService.save(project)) {
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);

            projectService.reverseNewest(project.getProjectCode(), project.getId());

            //APP
            projectSaveHelper.saveApplication(project, oldProject, Boolean.TRUE);
        }

        return project;
    }

    public Project saveConstructProjectWithVersionAndPass(Project oldProject, String instanceId, Integer instType) {
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject, project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        // 标识保存的项目信息是否为建设方案申报项目
        project.setIsConstruct(Boolean.FALSE);
        projectStateMachineUtil.pass(project);

        oldProject.setIsBackReject(Boolean.TRUE);
        projectService.updateById(oldProject);

        if (projectService.save(project)) {
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);
            projectService.reverseNewest(project.getProjectCode(), project.getId());
            projectSaveHelper.saveApplication(project, oldProject, Boolean.TRUE);
        }
        return project;
    }

    /**
     * 获取项目初步方案详情
     *
     * @param projectId \
     * @return com.hz.pm.api.projectlib.model.vo.ProjectDetailVO
     * @author CMM
     * @since 2023/07/12 11:43
     */
    public ProjectDetailVO getPrePlanProjectDetail(Long projectId) {
        // 查询建设方案申报前最新的项目申报信息
        Project projectInfo = projectService.getById(projectId);
        if (Objects.isNull(projectInfo)) {
            return null;
        }
        ProjectDetailVO vo = new ProjectDetailVO();
        BeanUtils.copyProperties(projectInfo, vo);
        vo.buildDynamicForm(projectInfo.getDynamicForm());
        // 查询应用
        List<ProjectApplication> applications = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(ProjectApplication::getProjectCode, vo.getProjectCode())
                .eq(ProjectApplication::getIsConstruct, Boolean.FALSE)
                .eq(ProjectApplication::getProjectVersion, projectInfo.getVersion()));

        Optional.ofNullable(applications).ifPresent(apps ->
                vo.setProjectApplications(CollUtils.convert(apps,
                        ProjectManageUtil::to)
                ));

        return vo;
    }

    public List<ProjectGovSystemReplaceInfos> getSystemReplaceInfosOption(String projectCode) {
        return systemReplaceInfosService.listByProjectCode(projectCode);
    }

    public List<ProcessDetailVO> processScheduleDetail(Long projectId) {
        Project project = projectService.getById(projectId);
        return processExecuteHandle.handle(project);
    }

    public List<ProcessDetailVO> processScheduleDetail(Project project) {
        return processExecuteHandle.handle(project);
    }

    public void exportList(ProjectListReq param, HttpServletResponse response) {
        param.setPageNumber(CommonConst.EXPORT_PAGE_NUMBER);
        param.setPageSize(CommonConst.EXPORT_PAGE_SIZE);
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        query.eq(Project::getNewest, Boolean.TRUE);
        List<Project> projects = projectService.list(query);

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = "项目库列表";
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDataList = ExcelDownUtil.getProjectExportDatas(param.getExportOptionList(), projects);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDataList);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDataList);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * build 项目的角色权限到req
     *
     * @param query 、
     */
    public boolean buildProjectLibPermission(LambdaQueryWrapper<Project> query, UserFullInfoDTO user) {
        boolean queryState = true;
        Optional<DataScopeDTO> currentUserDataScope = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!currentUserDataScope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            switch (currentUserDataScope.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER:
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    List<String> viewUnitIdList = CollUtils.convert(childUnitIds, String::valueOf);
                    //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    query.in(Project::getBuildOrgCode, viewUnitIdList);
                    break;
                case SUPER_ADMIN:
                    //超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                    break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
        }
        return queryState;
    }

    public boolean buildProjectLibPermission(ProjectListReq req, UserFullInfoDTO user) {
        boolean queryState = true;
        Optional<DataScopeDTO> dataScope = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!dataScope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = Boolean.FALSE;
        } else {
            switch (dataScope.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    req.setUserType("normal");
                    req.setUserValue(user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER:
                    Long buildOrgCode = null;
                    if (StrUtil.isNotBlank(req.getBuildOrgCode())) {
                        buildOrgCode = Long.parseLong(req.getBuildOrgCode());
                    }
                    //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    if (buildOrgCode != null && childUnitIds.contains(buildOrgCode)
                            && !buildOrgCode.equals(user.getMhUnitId())) {
                        childUnitIds = mhUnitCache.getViewChildIdsRecursion(buildOrgCode);
                    }
                    String viewUnitIdStr = CollUtils.joinByComma(childUnitIds, w -> "'" + w + "'");
                    req.setUserType("org");
                    req.setUserValue(viewUnitIdStr);
                    break;
                case SUPER_ADMIN:
                    //超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                    break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = Boolean.FALSE;
                    break;
            }
        }
        return queryState;
    }

    public void saveProjectByApplyDelay(DelayedApplyDTO dto, Project project, String instanceId) {

        // 更新项目流程实例ID
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        projectService.updateById(project);

        // 保存延期申请信息和申请延期前项目建设周期和验收时间
        ProjectDelayApply projectDelayApply = new ProjectDelayApply();
        projectDelayApply.setDelayTime(dto.getDelayedMonth());
        projectDelayApply.setDelayApplyFile(dto.getSupportingMaterials());
        projectDelayApply.setDelayApplyReason(dto.getDelayedReason());
        projectDelayApply.setProjectId(project.getId());
        projectDelayApply.setInstanceId(instanceId);
        projectDelayApply.setCreateBy(LoginUserUtil.getUserId());
        projectDelayApply.setCreateOn(LocalDateTime.now());
        projectDelayApply.setUpdateBy(LoginUserUtil.getUserId());
        projectDelayApply.setUpdateOn(LocalDateTime.now());
        projectDelayApplyService.save(projectDelayApply);

        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instanceId);
        projectInstService.save(projectInst);
    }

    public void saveProjectByApplyBorrow(Project project, UserInfoDetails user, String instanceId) {
        // 更新项目流程实例ID
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        projectService.updateById(project);

        // 保存项目借阅信息
        ProjectApplyBorrow projectApplyBorrow = new ProjectApplyBorrow();
        projectApplyBorrow.setProjectId(project.getId());
        projectApplyBorrow.setApplyBorrowUserId(user.getUserIdStr());
        projectApplyBorrow.setApplyBorrowOrgCode(user.getMhUnitIdStr());
        projectApplyBorrow.setInstanceId(instanceId);
        projectApplyBorrow.setCreateOn(LocalDateTime.now());
        projectApplyBorrow.setCreateBy(LoginUserUtil.getUserId());
        projectApplyBorrow.setUpdateOn(LocalDateTime.now());
        projectApplyBorrow.setUpdateBy(LoginUserUtil.getUserId());
        projectApplyBorrow.setIsSuccess(Boolean.FALSE);
        projectApplyBorrowService.save(projectApplyBorrow);

        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instanceId);
        projectInstService.save(projectInst);
    }

    /**
     * 应用列表
     *
     * @param req
     * @return
     */
    public PageVo<ProjectApplicationVO> applicationList(ProjectApplicationListReq req) {
        Page<ProjectApplication> page = req.page();
        LambdaQueryWrapper<ProjectApplication> wrapper = Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(StringUtils.isNotBlank(req.getBuildOrgName()), ProjectApplication::getBuildOrgName, req.getBuildOrgName());

        if (StringUtils.isNotBlank(req.getApplicationName())) {
            wrapper.and(q1 -> q1.like(ProjectApplication::getApplicationName, req.getApplicationName())
                    .or(q2 -> q2.like(ProjectApplication::getRelatedExistsApplication, req.getApplicationName())));
        }
        wrapper.orderByDesc(ProjectApplication::getUpdateOn);
        projectApplicationService.page(page, wrapper);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }

        List<ProjectApplicationVO> res = page.getRecords().stream()
                .map(app -> BeanUtil.copyProperties(app, ProjectApplicationVO.class))
                .collect(Collectors.toList());

        return PageVo.of(res, page.getTotal());
    }

    private List<PreInsAcceptancePersonVO> convertPersons(List<PreInsAcceptancePerson> acceptancePersons) {
        if (CollUtil.isEmpty(acceptancePersons)) {
            return Collections.emptyList();
        }
        return acceptancePersons.stream()
                .map(u -> BeanUtil.copyProperties(u, PreInsAcceptancePersonVO.class))
                .collect(Collectors.toList());
    }

    /**
     * 历史版本 项目列表
     *
     * @param id
     * @return
     */
    public JSONObject historyProjects(Long id) {
        JSONObject res = new JSONObject();
        Project project = projectService.getById(id);

        if (Objects.isNull(project)) {
            return res;
        }

        //只查 当前版本 和历史被驳回|退回的版本
        List<Project> historyProjects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectCode, project.getProjectCode())
                .and(q1 -> q1.eq(Project::getNewest, Boolean.TRUE)
                        .or(q2 -> q2.eq(Project::getIsBackReject, Boolean.TRUE)))
                .orderByDesc(Project::getCreateOn));

        if (CollUtil.isEmpty(historyProjects)) {
            return res;
        }

        List<ProjectDetailVO> basicProjects = historyProjects.stream()
                .map(p -> this.getProjectDetailThisVersion(p.getId()))
                .filter(p -> Objects.isNull(p.getIsConstruct()) || !p.getIsConstruct())
                .collect(Collectors.toList());
        List<ProjectDetailVO> constructProjects = historyProjects.stream()
                .map(p -> this.getProjectDetailThisVersion(p.getId()))
                .filter(p -> Objects.nonNull(p.getIsConstruct()) && p.getIsConstruct())
                .collect(Collectors.toList());
        res.put(ProjectConstant.ProjectLib.PROJECT_BASIC, basicProjects);
        res.put(ProjectConstant.ProjectLib.PROJECT_CONSTRUCT, constructProjects);
        return res;
    }

    /**
     * 查看 项目的 变更记录(驳回 重新发起的 和上个版本的变更字段)
     *
     * @param projectId
     * @return
     */
    public JSONObject changeRecord(Long projectId) {
        JSONObject res = new JSONObject();

        //1.先判断下 项目存不存在
        Project project = projectService.getNoNull(projectId);

        //2.要判断 项目在当前状态 有没有被驳回和退回过
        //当前项目状态流程的 实例 看看是不是有2个以上 有2个说明 有退回 驳回
        boolean isChangeRecord = todoService.isChangeRecord(projectId);
        if (!isChangeRecord) {
            return res;
        }

        //3. 去对比 当前版本和上个版本 的字段对比
        String projectCode = project.getProjectCode();
        List<Project> twoVersions = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectCode, projectCode)
                .orderByDesc(Project::getVersion)
                .last("limit 2"));
        if (CollUtil.isEmpty(twoVersions) || twoVersions.size() < 2) {
            return res;
        }

        Project lastProject = twoVersions.get(1);
        Project thisProject = twoVersions.get(0);
        res.put(ProjectConstant.ProjectLib.CHANGES_KEY,
                new CompareUtils<Project>().compareToJson(thisProject, lastProject, null));
        ProcessDetailReq req = new ProcessDetailReq();
        req.setProjectId(lastProject.getId());
        req.setInstanceId(lastProject.getInstCode());
        res.put(ProjectConstant.ProjectLib.PROCESS_KEY, todoService.getProcessDetail(req));
        return res;
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void bindSystemToBid(BindSystemToBidReq req) {
        Wrapper<ProjectGovSystemReplaceInfos> query = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(CollUtil.isNotEmpty(req.getSystemReplaceInfoIds()),
                        ProjectGovSystemReplaceInfos::getId, req.getSystemReplaceInfoIds())
                .or(q -> q.eq(ProjectGovSystemReplaceInfos::getBidId, req.getBidId()));
        List<ProjectGovSystemReplaceInfos> replaceInfos = systemReplaceInfosService.list(query);
        if (CollUtil.isNotEmpty(req.getSystemReplaceInfoIds()) && CollUtil.isEmpty(replaceInfos)) {
            throw ReturnException.wrap("未找到相关数据");
        }
        if (CollUtil.contains(replaceInfos, w -> w.getBidId() != null && !w.getBidId().equals(req.getBidId()))) {
            throw ReturnException.wrap("存在已关联其他标项的系统");
        }
        replaceInfos.removeIf(w -> req.getBidId().equals(w.getBidId())
                && CollUtil.contains(req.getSystemReplaceInfoIds(), w.getId()));
        if (CollUtil.isEmpty(replaceInfos)) {
            return;
        }
        // 解绑的系统
        Map<Boolean, List<Long>> relationMap = replaceInfos.stream()
                .collect(Collectors.groupingBy(w -> CollUtil.contains(req.getSystemReplaceInfoIds(), w.getId()),
                        Collectors.mapping(ProjectGovSystemReplaceInfos::getId, Collectors.toList())));
        List<Long> canceledIds = relationMap.get(Boolean.FALSE);
        if (CollUtil.isNotEmpty(canceledIds)) {
            Wrapper<ProjectGovSystemReplaceInfos> update = Wrappers.lambdaUpdate(ProjectGovSystemReplaceInfos.class)
                    .set(ProjectGovSystemReplaceInfos::getBidId, null)
                    .in(ProjectGovSystemReplaceInfos::getId, canceledIds);
            systemReplaceInfosService.update(update);
        }
        // 关联系统
        List<Long> bindIds = relationMap.get(Boolean.TRUE);
        if (CollUtil.isNotEmpty(bindIds)) {
            Wrapper<ProjectGovSystemReplaceInfos> update = Wrappers.lambdaUpdate(ProjectGovSystemReplaceInfos.class)
                    .set(ProjectGovSystemReplaceInfos::getBidId, req.getBidId())
                    .in(ProjectGovSystemReplaceInfos::getId, bindIds);
            systemReplaceInfosService.update(update);
        }
    }

}
