package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * 信创符合性申请状态
 * </p>
 *
 * @author WendyYang
 * @since 10:59 2024/4/8
 */
@Getter
@AllArgsConstructor
public enum TenderXcfhxApplyStatus implements ITenderStatus {

    TO_BE_SUBMIT_XCFHX_APPLY(null, 301, "待信创符合性申请"),
    ON_XCFHX_APPLY(null, 302, "信创符合性申请中"),
    XCFHX_APPLY_FAILED(null, 303, "信创符合性申请不通过"),
    XCFHX_APPLY_PASSED(null, 304, "信创符合性申请通过");

    private final ProjectStatus projectStatus;
    private final Integer code;
    private final String desc;

    private static Optional<TenderXcfhxApplyStatus> get(Integer code) {
        return Arrays.stream(values()).filter(w -> w.eq(code)).findFirst();
    }

    public static TenderXcfhxApplyStatus getNoNull(Integer code) {
        return get(code).orElseThrow(() -> BizException.wrap("无效的信创符合性审查状态：%s", code));
    }

    public static String getDesc(Integer code) {
        return get(code).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }


}
