package com.hz.pm.api.projectlib.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.ningdatech.basic.util.CollUtils;

import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author WendyYang
 * @since 2024-04-01
 */
public interface IPurchaseStatusChangeService extends IService<PurchaseStatusChange> {
    default boolean exists(String projectCode, TenderStateChangeEvent event) {
        Wrapper<PurchaseStatusChange> pscQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                .eq(PurchaseStatusChange::getProjectCode, projectCode)
                .eq(PurchaseStatusChange::getEvent, event);
        return count(pscQuery) > 0;
    }

    default PurchaseStatusChange getLastOne(String projectCode, Collection<TenderStateChangeEvent> events) {
        LambdaQueryWrapper<PurchaseStatusChange> pscQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                .eq(PurchaseStatusChange::getProjectCode, projectCode)
                .in(PurchaseStatusChange::getEvent, events)
                .orderByDesc(PurchaseStatusChange::getCreateOn)
                .last(BizConst.LIMIT_1);
        return getOne(pscQuery);
    }

    default Map<Long, PurchaseStatusChange> listLastOne(Collection<Long> bidIds, TenderStateChangeEvent event) {
        LambdaQueryWrapper<PurchaseStatusChange> pscQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                .in(PurchaseStatusChange::getBidId, bidIds)
                .eq(PurchaseStatusChange::getEvent, event)
                .orderByDesc(PurchaseStatusChange::getCreateOn);
        return BizUtils.groupFirstMap(list(pscQuery), PurchaseStatusChange::getBidId,
                Comparator.comparing(PurchaseStatusChange::getCreateOn).reversed());
    }

    default List<PurchaseStatusChange> listByProjectCode(String projectCode, Collection<Long> bidIds) {
        LambdaQueryWrapper<PurchaseStatusChange> pscQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                .eq(PurchaseStatusChange::getProjectCode, projectCode)
                .in(PurchaseStatusChange::getBidId, bidIds)
                .orderByDesc(PurchaseStatusChange::getCreateOn);
        return list(pscQuery);
    }

}
