package com.hz.pm.api.scheduler.listener;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.scheduler.listener.event.MhTodoHandedEvent;
import com.hz.pm.api.sys.model.entity.MhTodoRecord;
import com.hz.pm.api.sys.model.entity.MhTodoRecord.TodoStatusEnum;
import com.hz.pm.api.sys.service.IMhTodoRecordService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

/**
 * <p>
 * MhTodoHandedListener
 * </p>
 *
 * @author WendyYang
 * @since 14:11 2024/10/19
 */
@Component
@RequiredArgsConstructor
public class MhTodoHandedListener {

    private final IMhTodoRecordService mhTodoRecordService;

    @EventListener(MhTodoHandedEvent.class)
    public void handleEvent(MhTodoHandedEvent event) {
        LambdaUpdateWrapper<MhTodoRecord> wrapper = Wrappers.lambdaUpdate(MhTodoRecord.class)
                .set(MhTodoRecord::getStatus, TodoStatusEnum.HANDING.getCode())
                .eq(MhTodoRecord::getStatus, TodoStatusEnum.PENDING.getCode())
                .eq(MhTodoRecord::getType, event.getMhTodoType());
        if (CollUtil.isNotEmpty(event.getBizIds())) {
            wrapper.in(MhTodoRecord::getBizId, CollUtils.convert(event.getBizIds(), String::valueOf));
        } else {
            wrapper.eq(MhTodoRecord::getBizId, event.getBizId());
        }
        mhTodoRecordService.update(wrapper);
    }

}
