package com.hz.pm.api.scheduler.listener;

import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.model.dto.MhZwddWorkNoticeDTO;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.external.todo.dto.MhTodoExtraParamDTO;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.Xinchuang;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.scheduler.listener.event.MhTodoSendEvent;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import lombok.RequiredArgsConstructor;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

/**
 * <p>
 * MhTodoSendListener
 * </p>
 *
 * @author WendyYang
 * @since 14:47 2024/10/19
 */
@Component
@RequiredArgsConstructor
public class MhTodoSendListener {

    private final MhApiClient mhApiClient;
    private final MHTodoClient mhTodoClient;
    private final UserInfoHelper userInfoHelper;

    @Async
    @EventListener(classes = MhTodoSendEvent.class)
    public void handleEvent(MhTodoSendEvent event) {
        MHTodoTypeEnum todoType = event.getMhTodoType();
        MHTodoTypeEnum nextStep = ObjUtil.defaultIfNull(todoType.getNextStep(), todoType);
        Purchase purchase = event.getPurchase();
        Project project = event.getProject();
        Xinchuang xinchuang = event.getXinchuang();
        boolean isBidTodo = purchase != null;
        String tmpName;
        Long createBy;
        Long bizId;
        if (isBidTodo) {
            tmpName = purchase.getBidName();
            createBy = Long.valueOf(purchase.getCreateBy());
            bizId = purchase.getId();
        } else if (project != null) {
            tmpName = project.getProjectName();
            createBy = project.getCreateBy();
            bizId = project.getId();
        } else if (xinchuang != null) {
            tmpName = xinchuang.getProjectName();
            createBy = Long.parseLong(xinchuang.getCreateBy());
            bizId = xinchuang.getId();
        } else {
            return;
        }
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(createBy);
        if (user == null || user.getMhUserId() == null) {
            return;
        }
        if (StrUtil.isNotBlank(nextStep.getZwddMsg())) {
            String zwddMsg = String.format(nextStep.getZwddMsg(), tmpName);
            MhZwddWorkNoticeDTO notice = MhZwddWorkNoticeDTO.builder()
                    .content(zwddMsg)
                    .title(nextStep.getVal())
                    .targetUserName(user.getRealName())
                    .targetUser(user.getMhUserId())
                    .build();
            mhApiClient.sendZwddWorkNotice(notice);
        }
        if (StrUtil.isNotBlank(nextStep.getTodoMsg())) {
            String todoMsg = String.format(nextStep.getTodoMsg(), tmpName);
            MhTodoExtraParamDTO paramObj = MhTodoExtraParamDTO.builder()
                    .path(nextStep.getPagePath())
                    .userName(user.getRealName())
                    .userId(createBy)
                    .activeName(ObjUtil.defaultIfNull(event.getActiveName(), todoType.getActiveName()))
                    .build();
            mhTodoClient.addTodo(paramObj, user, nextStep, bizId.toString(), todoMsg);
        }
    }

}
