package com.hz.pm.api.scheduler.task;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.sys.model.entity.MhTodoRecord;
import com.hz.pm.api.sys.service.IMhTodoRecordService;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;

import static com.hz.pm.api.sys.model.entity.MhTodoRecord.TodoStatusEnum.HANDED;
import static com.hz.pm.api.sys.model.entity.MhTodoRecord.TodoStatusEnum.HANDING;

/**
 * <p>
 * MhTodoCloseTask
 * </p>
 *
 * @author WendyYang
 * @since 13:49 2024/10/19
 */
@Component
@Profile("prod")
@RequiredArgsConstructor
public class MhTodoCloseTask {

    private final IMhTodoRecordService mhTodoRecordService;
    private final MHTodoClient mhTodoClient;

    @Scheduled(fixedRate = 2, timeUnit = TimeUnit.MINUTES)
    public void closeMhTodo() {
        Wrapper<MhTodoRecord> query = Wrappers.lambdaQuery(MhTodoRecord.class)
                .eq(MhTodoRecord::getStatus, HANDING.getCode());
        List<MhTodoRecord> mhTodoList = mhTodoRecordService.list(query);
        if (mhTodoList.isEmpty()) {
            return;
        }
        for (MhTodoRecord mhTodo : mhTodoList) {
            if (mhTodoClient.complete(mhTodo.getMhTodoId())) {
                Wrapper<MhTodoRecord> update = Wrappers.lambdaUpdate(MhTodoRecord.class)
                        .set(MhTodoRecord::getStatus, HANDED.getCode())
                        .eq(MhTodoRecord::getId, mhTodo.getId());
                mhTodoRecordService.update(update);
            }
        }
    }

}
