package com.hz.pm.api.sys.manage;

import cn.hutool.core.bean.BeanUtil;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.entity.MenuTreeEntity;
import com.hz.pm.api.common.util.TreeUtil;
import com.hz.pm.api.sys.model.entity.Menu;
import com.hz.pm.api.sys.model.vo.MenuRoleVO;
import com.hz.pm.api.sys.service.IMenuService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * MenuManage - 菜单管理业务层
 * </p>
 *
 * @author WendyYang
 * @since 14:23 2023/1/28
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class MenuManage {

    private final IMenuService menuService;

    public List<MenuRoleVO> buildUserMenu(List<Menu> list) {
        Set<Long> menuIds = new HashSet<>();
        List<MenuRoleVO> menuRoles = CollUtils.convert(list, w -> {
            menuIds.add(w.getId());
            return BeanUtil.copyProperties(w, MenuRoleVO.class);
        });
        List<Long> pidList = menuRoles.stream()
                .map(MenuTreeEntity::getPid)
                .filter(pid -> !menuIds.contains(pid) && pid != BizConst.PARENT_ID)
                .collect(Collectors.toList());
        parentMenuCheck(pidList, menuRoles, menuIds);
        return TreeUtil.buildUserTree(menuRoles);
    }

    public void parentMenuCheck(List<Long> pidList, List<MenuRoleVO> menuRoles, Set<Long> menuIds) {
        if (pidList.isEmpty()) {
            return;
        }
        List<Menu> menus = menuService.listByIds(pidList);
        if (menus.isEmpty()) {
            return;
        }
        Set<Long> tmpSet = new HashSet<>();
        menus.forEach(w -> {
            menuIds.add(w.getId());
            tmpSet.add(w.getPid());
            menuRoles.add(BeanUtil.copyProperties(w, MenuRoleVO.class));
        });
        List<Long> tmpPidList = tmpSet.stream()
                .filter(pid -> !menuIds.contains(pid) && pid != BizConst.PARENT_ID)
                .collect(Collectors.toList());
        parentMenuCheck(tmpPidList, menuRoles, menuIds);
    }

}
