package com.hz.pm.api.user.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.model.constant.MhUnitConst;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.external.model.enumeration.MhUnitTypeEnum;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.MhUnitTreeDTO;
import com.hz.pm.api.user.model.dto.MhUnitUserTreeDTO;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.model.entity.MhUnit;
import com.hz.pm.api.user.model.entity.UserInfo;
import com.hz.pm.api.user.model.enumeration.UnitUserNodeType;
import com.hz.pm.api.user.model.po.MhUnitListReq;
import com.hz.pm.api.user.model.po.UserTreeReq;
import com.hz.pm.api.user.model.vo.MhUnitListVO;
import com.hz.pm.api.user.service.IMhUnitService;
import com.hz.pm.api.user.service.IUserInfoService;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.TreeUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.hz.pm.api.common.model.constant.MhUnitConst.*;

/**
 * <p>
 * MhUnitManage
 * </p>
 *
 * @author WendyYang
 * @since 15:33 2024/1/10
 */
@Component
@RequiredArgsConstructor
public class MhUnitManage {


    @Value("${mh.unit.root-parent-id:0}")
    private Long unitRootParentId;

    private final IMhUnitService mhUnitService;
    private final MhUnitCache mhUnitCache;
    private final IUserInfoService userInfoService;

    public List<MhUnitTreeDTO> getMhUnitTree(String name, Long unitId) {
        List<UnitDTO> retUnits;
        List<UnitDTO> allUnits = mhUnitCache.all();
        if (StrUtil.isNotBlank(name) || unitId != null) {
            if (StrUtil.isNotBlank(name)) {
                allUnits.removeIf(w -> !w.getName().contains(name));
            }
            retUnits = allUnits.stream()
                    .map(w -> {
                        List<Long> unitIdPaths = mhUnitCache.getUnitIdPaths(w.getId());
                        if (unitId != null) {
                            if (!unitIdPaths.contains(unitId)) {
                                return null;
                            }
                            return unitIdPaths.subList(unitIdPaths.indexOf(unitId), unitIdPaths.size());
                        }
                        return unitIdPaths;
                    })
                    .filter(Objects::nonNull)
                    .flatMap(Collection::stream)
                    .distinct()
                    .map(mhUnitCache::getById)
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList());
        } else {
            retUnits = allUnits;
        }
        if (allUnits.isEmpty()) {
            return Collections.emptyList();
        }
        List<MhUnitTreeDTO> nodes = retUnits.stream().map(w -> {
            MhUnitTreeDTO node = BeanUtil.copyProperties(w, MhUnitTreeDTO.class);
            node.setStrip(node.getType().getStrip());
            return node;
        }).collect(Collectors.toList());
        Long tmpRootParentId;
        if (unitId != null) {
            tmpRootParentId = mhUnitCache.getById(unitId).getParentId();
        } else {
            tmpRootParentId = unitRootParentId;
        }
        return TreeUtil.convertToTree(nodes, tmpRootParentId);
    }

    public List<MhUnitTreeDTO> getMhUnitStripTree() {
        List<UnitDTO> allUnits = mhUnitCache.all();
        List<MhUnitTreeDTO> nodes = allUnits.stream()
                .filter(w -> MhUnitTypeEnum.SASAC.equals(w.getType()) ||
                        ((MhUnitTypeEnum.REGION.equals(w.getType()) || MhUnitTypeEnum.NODE.equals(w.getType()))
                                && (w.getName().endsWith(MhUnitConst.UNIT_XCB) || UNIT_HZ.equals(w.getName()))))
                .map(w -> BeanUtil.copyProperties(w, MhUnitTreeDTO.class))
                .collect(Collectors.toList());
        mhUnitOptionSort(nodes);
        return nodes;
    }

    private static void mhUnitOptionSort(List<MhUnitTreeDTO> nodes) {
        Function<MhUnitTreeDTO, Integer> firstSortFunc = w -> CollUtil.indexOf(UNIT_SORT, x -> w.getName().startsWith(x));
        Function<MhUnitTreeDTO, Integer> secondSortFunc = w -> {
            if (w.getName().contains(MhUnitConst.UNIT_XCB)) {
                return 1;
            } else if (w.getName().contains(UNIT_SOE)) {
                return 3;
            } else {
                return 2;
            }
        };
        nodes.sort(Comparator.comparing(firstSortFunc, Comparator.naturalOrder())
                .thenComparing(secondSortFunc, Comparator.naturalOrder()));
    }

    public List<MhUnitTreeDTO> getMhUnitDashboardOption() {
        List<UnitDTO> allUnits = mhUnitCache.all();
        List<MhUnitTreeDTO> nodes = allUnits.stream()
                .filter(w -> MhUnitTypeEnum.SASAC.equals(w.getType()) ||
                        ((MhUnitTypeEnum.REGION.equals(w.getType()) || MhUnitTypeEnum.NODE.equals(w.getType()))
                                && (w.getName().endsWith(UNIT_XCB)
                                || CITY_SELF.equals(w.getName())
                                || CITY_SELF_ALIAS.equals(w.getName())
                                || UNIT_HZ.equals(w.getName())
                                || COUNTY_SELF.equals(w.getName()))))
                .map(w -> {
                    MhUnitTreeDTO unit = BeanUtil.copyProperties(w, MhUnitTreeDTO.class);
                    if (COUNTY_SELF.equals(w.getName())) {
                        UnitDTO parent = mhUnitCache.getById(unit.getParentId());
                        if (parent != null) {
                            String parentName = StrUtil.blankToDefault(parent.getShortName(), parent.getName());
                            String unitName = StrUtil.blankToDefault(unit.getShortName(), unit.getName());
                            unit.setShortName(parentName + unitName);
                            unit.setName(unit.getShortName());
                        }
                    }
                    return unit;
                }).collect(Collectors.toList());
        mhUnitOptionSort(nodes);
        return nodes;
    }

    public PageVo<MhUnitListVO> page(MhUnitListReq req) {
        LambdaQueryWrapper<MhUnit> query = Wrappers.lambdaQuery(MhUnit.class);
        if (StrUtils.isNotBlank(req.getName())) {
            query.and(q1 -> q1.like(MhUnit::getName, req.getName())
                    .or(q2 -> q2.like(MhUnit::getShortName, req.getName())));
        }

        Page<MhUnit> page = mhUnitService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MhUnitListVO> records = page.getRecords().stream().map(w -> {
            MhUnitListVO unit = BeanUtil.copyProperties(w, MhUnitListVO.class, "type");
            MhUnitTypeEnum.getByCode(w.getType()).ifPresent(type -> {
                unit.setType(type);
                unit.setStrip(type.getStrip());
                unit.setTypeName(type.getVal());
                unit.setStripName(type.getStrip().getVal());
            });
            return unit;
        }).collect(Collectors.toList());
        return PageVo.of(records, page.getTotal());
    }

    public List<MhUnitUserTreeDTO> getUnitUserTree(UserTreeReq req) {
        List<MhUnitUserTreeDTO> selectedUnits = new ArrayList<>();
        Set<Long> viewUnitIds = new HashSet<>();
        boolean hasId = req.getId() != null;
        if (hasId) {
            List<Long> childIds = mhUnitCache.getViewChildIdsRecursion(req.getId());
            viewUnitIds.addAll(childIds);
        }
        boolean hasName = CharSequenceUtil.isNotBlank(req.getName());
        LambdaQueryWrapper<UserInfo> userQuery = Wrappers.lambdaQuery(UserInfo.class)
                .select(UserInfo::getMhUnitId, UserInfo::getId, UserInfo::getRealName)
                .in(!viewUnitIds.isEmpty(), UserInfo::getMhUnitId, viewUnitIds)
                .exists(ExistsSqlConst.USER_EXISTS_ROLE)
                .gt(UserInfo::getMhUnitId, 0);
        if (hasName) {
            // 查询满足名字的
            userQuery.like(UserInfo::getRealName, req.getName())
                    .last("limit 50");
        } else if (hasId) {
            List<UnitDTO> units = mhUnitCache.listByIds(viewUnitIds);
            selectedUnits.addAll(CollUtils.convert(units, this::getMhUnitUserTree));
        } else {
            List<UnitDTO> all = mhUnitCache.all();
            selectedUnits.addAll(CollUtils.convert(all, this::getMhUnitUserTree));
        }
        List<UserInfo> userInfos = userInfoService.list(userQuery);
        if (userInfos.isEmpty()) {
            return Collections.emptyList();
        }
        if (hasName) {
            Set<Long> tmpUnitIds = CollUtils.fieldSet(userInfos, UserInfo::getMhUnitId);
            Set<Long> allUnitIds = new HashSet<>();
            for (Long unitId : tmpUnitIds) {
                allUnitIds.addAll(mhUnitCache.getUnitIdPaths(unitId));
            }
            List<UnitDTO> allUnits = mhUnitCache.listByIds(allUnitIds);
            selectedUnits.addAll(CollUtils.convert(allUnits, this::getMhUnitUserTree));
        }
        selectedUnits.addAll(CollUtils.convert(userInfos, this::getMhUnitUserTree));
        String rootId = UnitUserNodeType.UNIT.name() + "#" + (hasId ? req.getId() : unitRootParentId);
        return TreeUtil.convertToTree(selectedUnits, rootId);
    }

    private MhUnitUserTreeDTO getMhUnitUserTree(UnitDTO unit) {
        MhUnitUserTreeDTO node = new MhUnitUserTreeDTO();
        node.setSort(unit.getSort());
        node.setName(unit.getName());
        node.setShortName(unit.getShortName());
        node.setBizId(unit.getId());
        node.setParentId(UnitUserNodeType.UNIT.name() + "#" + unit.getParentId());
        node.setId(UnitUserNodeType.UNIT.name() + "#" + unit.getId());
        node.setType(UnitUserNodeType.UNIT);
        return node;
    }

    private MhUnitUserTreeDTO getMhUnitUserTree(UserInfo w) {
        MhUnitUserTreeDTO node = new MhUnitUserTreeDTO();
        node.setSort((short) 1);
        node.setName(w.getRealName());
        node.setBizId(w.getId());
        node.setParentId(UnitUserNodeType.UNIT.name() + "#" + w.getMhUnitId());
        node.setId(UnitUserNodeType.USER.name() + "#" + w.getId());
        node.setType(UnitUserNodeType.USER);
        return node;
    }

}
