package com.hz.pm.api.user.security.auth.credential;

import cn.hutool.core.util.StrUtil;
import com.hz.pm.api.common.util.EnvironmentUtil;
import com.hz.pm.api.sms.constant.VerificationCodeType;
import com.hz.pm.api.sms.helper.VerifyCodeCheckHelper;
import com.hz.pm.api.user.model.enumeration.LoginTypeEnum;
import com.hz.pm.api.user.security.auth.constants.SessionTimeConst;
import com.hz.pm.api.user.security.auth.constants.UserDetailsServiceConstant;
import com.hz.pm.api.user.security.exception.BizLoginException;
import com.ningdatech.basic.exception.BizException;
import lombok.Setter;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;

import static com.hz.pm.api.user.security.auth.constants.SessionTimeConst.SUPER_VERIFY_CODE;
import static com.hz.pm.api.user.security.auth.constants.SessionTimeConst.SUPER_VERIFY_CODE_DEV;

/**
 * <p>
 * CredentialAuthProvider
 * </p>
 *
 * @author WendyYang
 * @since 09:26 2024/9/12
 */
@Setter
public class CredentialAuthProvider implements AuthenticationProvider {

    private UserDetailsService userDetailsService;
    private Boolean phoneVerifyCodeSkip;
    private EnvironmentUtil environmentUtil;
    private VerifyCodeCheckHelper verifyCodeCheckHelper;


    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        if (!(authentication instanceof CredentialAuthToken)) {
            throw new BizException("CustomAuthProvider 只支持 CustomAuthToken");
        }
        CredentialAuthToken authenticationToken = (CredentialAuthToken) authentication;
        String principal = (String) authenticationToken.getPrincipal();

        UserDetails user;
        LoginTypeEnum loginType = authenticationToken.getLoginTypeEnum();
        String credentials = (String) authenticationToken.getCredentials();
        switch (loginType) {
            case PHONE_VERIFICATION_CODE_LOGIN: {
                if (Boolean.FALSE.equals(phoneVerifyCodeSkip)
                        && !StrUtil.equals(credentials, environmentUtil.isProdEnv() ? SUPER_VERIFY_CODE : SUPER_VERIFY_CODE_DEV)
                        && !verifyCodeCheckHelper.verification(VerificationCodeType.LOGIN, principal, credentials)) {
                    throw new BizLoginException("验证码错误");
                }
                user = userDetailsService.loadUserByUsername(principal + UserDetailsServiceConstant.SEPARATOR + loginType.name());
            }
            break;
            case USERNAME_PASSWORD_LOGIN: {
                user = userDetailsService.loadUserByUsername(principal + UserDetailsServiceConstant.SEPARATOR + loginType.name());
            }
            break;
            default:
                throw new BizException("不支持的登录方式");
        }
        // 将用户定义的user放入token中，这样可以在session中查询到所有自定义的用户信息
        return new CredentialAuthToken(user, user.getPassword(), user.getAuthorities());
    }

    @Override
    public boolean supports(Class<?> authentication) {
        return CredentialAuthToken.class.isAssignableFrom(authentication);
    }

}
