package com.hz.pm.api.user.task;

import com.hz.pm.api.user.manage.SyncMhUserOrgManage;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.concurrent.TimeUnit;


/**
 * <p>
 * SyncMhUserTask
 * </p>
 *
 * @author WendyYang
 * @since 11:02 2023/12/25
 */
@Slf4j
@Component
@RequiredArgsConstructor
@ConditionalOnProperty(prefix = SyncMhUserProperties.PREFIX, value = "open", matchIfMissing = true)
public class SyncMhUserTask {

    private final SyncMhUserProperties syncMhUserProperties;
    private final SyncMhUserOrgManage syncMhUserOrgManage;

    private volatile Integer fullSyncDate;

    @Scheduled(fixedRateString = "#{syncMhUserProperties.getFixedRate()}", timeUnit = TimeUnit.MINUTES)
    public void execute() {
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime currSyncDateTime;
        if (now.getHour() == 0 && (fullSyncDate == null || fullSyncDate < now.getDayOfYear())) {
            currSyncDateTime = null;
            fullSyncDate = now.getDayOfYear();
        } else {
            Integer fixedRate = syncMhUserProperties.getFixedRate();
            currSyncDateTime = now.minusMinutes(fixedRate + 3L);
        }
        syncMhUserOrgManage.syncUsers(currSyncDateTime);
    }

}
