package com.hz.pm.api.user.util;

import cn.hutool.core.collection.CollUtil;
import com.ningdatech.basic.auth.AbstractLoginUserUtil;
import com.hz.pm.api.sys.model.entity.Role;
import com.hz.pm.api.user.model.enumeration.RoleEnum;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import org.springframework.util.Assert;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * LoginUserUtil
 * </p>
 *
 * @author WendyYang
 * @since 15:42 2024/1/12
 */
public class LoginUserUtil extends AbstractLoginUserUtil {

    public static UserInfoDetails userDetail() {
        return isLogin() ? getLoginUserPrincipal() : null;
    }

    public static UserInfoDetails userDetailNotNull() {
        UserInfoDetails userInfoDetails = userDetail();
        Assert.notNull(userInfoDetails, "用户未登录");
        return userInfoDetails;
    }

    public static List<Long> getRoleIdList() {
        UserInfoDetails userInfoDetails = userDetailNotNull();
        List<Role> userRoleList = userInfoDetails.getUserRoleList();
        if (userRoleList == null) {
            return Collections.emptyList();
        }
        return userRoleList.stream().map(Role::getId).collect(Collectors.toList());
    }

    public static boolean isExpert() {
        UserInfoDetails detail = userDetailNotNull();
        return CollUtil.anyMatch(detail.getUserRoleList(), w -> RoleEnum.EXPERT.eq(w.getCode()));
    }

    public static Long getMhUnitId() {
        return userDetailNotNull().getMhUnitId();
    }

}
