package com.hz.pm.api.workbench.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Sets;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.statemachine.event.AbstractStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.XcfhxStateChangeEvent;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.SystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.vo.UnitProjectGovSystemReplaceVO;
import com.hz.pm.api.projectlib.model.vo.UnitProjectGovSystemReplaceVO.ProjectGovSystemReplaceVO;
import com.hz.pm.api.projectlib.model.vo.WorkbenchProjectLibSystemReplaceInfoVO;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.enumeration.ProjectEarlyWarningStatusEnum;
import com.hz.pm.api.sys.manage.EarlyWarningManage;
import com.hz.pm.api.sys.model.entity.ProjectEarlyWarning;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.model.vo.ProjectEarlyWarningVO;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import com.hz.pm.api.sys.service.IProjectEarlyWarningService;
import com.hz.pm.api.todocenter.manage.TodoCenterManage;
import com.hz.pm.api.todocenter.model.req.ToBeProcessedReq;
import com.hz.pm.api.todocenter.model.vo.ResToBeProcessedVO;
import com.hz.pm.api.todocenter.model.vo.TodoCenterStatisticsVO;
import com.hz.pm.api.todocenter.model.vo.TodoVO;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper.UnitQueryState;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.hz.pm.api.workbench.converter.WorkbenchConverter;
import com.hz.pm.api.workbench.model.WorkbenchProcessNode;
import com.hz.pm.api.workbench.model.WorkbenchProjectLibReq;
import com.hz.pm.api.workbench.model.WorkbenchReq;
import com.hz.pm.api.workbench.model.vo.*;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.service.IEarlyWarningService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.math3.util.Pair;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import static com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent.ADAPT_INFO_PASSED;
import static com.hz.pm.api.common.statemachine.event.SelfTestStateChangeEvent.SELF_TEST_PASSED;
import static com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent.*;
import static com.hz.pm.api.common.statemachine.event.TestValidStateChangeEvent.TEST_VALID_INFO_PASSED;
import static com.hz.pm.api.projectlib.helper.ProjectManageUtil.projectQueryExistsProjectStatusChange;
import static com.hz.pm.api.projectlib.helper.ProjectManageUtil.projectQueryNotExistsProjectStatusChange;
import static com.hz.pm.api.workbench.model.vo.ProjectProcessStatVO.TenderAdaptFinishStatVO;

/**
 * <p>
 * WorkbenchManage
 * </p>
 *
 * @author WendyYang
 * @since 18:58 2024/3/23
 */
@Slf4j
@Component
@AllArgsConstructor
public class WorkbenchManage {

    private final TodoCenterManage todoCenterManage;
    private final IEarlyWarningRecordsService earlyWarningRecordsService;
    private final IEarlyWarningService earlyWarningService;
    private final UserInfoHelper userInfoHelper;
    private final ProjectLibManage projectLibManage;
    private final EarlyWarningManage earlyWarningManage;
    private final IProjectService projectService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseService purchaseService;
    private final IContractService contractService;
    private final MhUnitCache mhUnitCache;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final IProjectEarlyWarningService projectEarlyWarningService;
    private final MhUnitQueryAuthHelper mhUnitQueryAuthHelper;

    public WorkbenchVO getWorkbenchData(Integer year) {
        WorkbenchVO res = new WorkbenchVO();
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);
        // 待办中心数据
        TodoCenterStatisticsVO todoStat = todoCenterManage.todoCenterStatistics(userId);
        ToBeProcessedReq todoReq = new ToBeProcessedReq();
        todoReq.page(1, 50);
        todoReq.setLoginUserId(userId);
        todoReq.setUserId(userId);
        PageVo<ResToBeProcessedVO> todoPage = todoCenterManage.todoProjectList(todoReq);
        List<TodoVO> todoList = CollUtils.fieldList(todoPage.getRecords(), WorkbenchConverter::convert);
        todoStat.setTodoList(todoList);
        res.setTodoCerter(todoStat);
        // 项目列表
        ProjectListReq projReq = new ProjectListReq();
        projReq.page(1, 20);
        projReq.setProjectYear(year);
        PageVo<ProjectLibListItemVO> projPage = projectLibManage.listWithPermissionWorkBench(projReq, user);
        res.setProjects(new ArrayList<>(projPage.getRecords()));
        return res;
    }

    /**
     * 催办
     *
     * @param recordId \
     * @return \
     */
    public synchronized String urging(Long recordId) {
        WflowEarlyWarningRecords warningRecords = earlyWarningRecordsService.getById(recordId);
        Assert.notNull(warningRecords, "催办失败,id不存在!");
        String projectCode = warningRecords.getProjectCode();
        Project project = projectService.getNoNull(projectCode);
        String batchEmployees = warningRecords.getBatchUserIds();
        Integer ruleType = warningRecords.getRuleType();
        String areaCode = warningRecords.getAreaCode();
        //如果是临期 可以取到 超期的小时数
        String path = warningRecords.getPath();
        LocalDateTime instStart = warningRecords.getInstStart();
        Integer biz = warningRecords.getBiz();
        if (StringUtils.isBlank(batchEmployees)) {
            //如果批次员工号不存在 就取当前
            batchEmployees = warningRecords.getWarningUserId();
        }
        Assert.notEmpty(batchEmployees, "催办失败,员工为空!");
        String[] employees = batchEmployees.split(StrPool.COMMA);
        String noticeContent = warningRecords.getNoticeContent();
        //查出配置的模板
        WflowEarlyWarning config = earlyWarningService.getOne(Wrappers.lambdaQuery(WflowEarlyWarning.class)
                .eq(WflowEarlyWarning::getAreaCode, areaCode)
                .eq(WflowEarlyWarning::getRuleType, ruleType)
                .last(BizConst.LIMIT_1));
        //如果配置能取到 就取模板的
        if (Objects.nonNull(config)) {
            noticeContent = config.getNoticeContent();
        }

        //算出 当前和他的 小时差 临期
        long adventBetween = ChronoUnit.HOURS.between(instStart, LocalDateTime.now());
        String noticeMethod = warningRecords.getNoticeMethod();
        Integer noticeType = warningRecords.getNoticeType();
        for (String employee : employees) {
            earlyWarningManage.urging(noticeMethod, noticeContent,
                    (int) adventBetween, (int) adventBetween,
                    biz, instStart, Long.valueOf(employee),
                    project, ruleType, noticeType, path,
                    batchEmployees, warningRecords.getNodeId());
        }
        return "催办成功了" + employees.length + "个人";
    }

    private void initWorkbenchDefaultParam(WorkbenchReq req) {
        if (req.getUnitId() == null) {
            req.setUnitId(LoginUserUtil.userDetailNotNull().getMhUnitId());
        }
        if (req.getProjectYear() == null) {
            req.setProjectYear(LocalDate.now().getYear());
        }
    }

    public ProjectTotalViewVO projectTotalView(WorkbenchReq req) {
        ProjectTotalViewVO retDetail = new ProjectTotalViewVO();
        initWorkbenchDefaultParam(req);
        LambdaQueryWrapper<Project> projectQuery = Wrappers.lambdaQuery(Project.class)
                .select(Project::getReviewAmount, Project::getApprovalAmount, Project::getId, Project::getStage,
                        Project::getApprovalGovOwnFinanceAmount, Project::getApprovalGovSuperiorFinanceAmount,
                        Project::getDeclareAmount, Project::getStatus)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT)
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE);
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        projectQuery.in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(projectQuery);
        if (projects.isEmpty()) {
            return retDetail;
        }
        retDetail.setTotalProjectCount(projects.size());
        retDetail.setStoppedProjectCount(CollUtil.count(projects, w -> ProjectStatus.STOPPED_PASSED.eq(w.getStatus())));
        retDetail.setOngoingProjectCount(retDetail.getTotalProjectCount() - retDetail.getStoppedProjectCount());
        List<Project> validProjects = CollUtils.filter(projects, w -> !ProjectStatus.STOPPED.eq(w.getStage())
                && !ProjectStatus.COMPLIANCE_REVIEW_FAILED.eq(w.getStatus()) && !ProjectStatus.TO_BE_DECLARED.eq(w.getStatus())
                && !ProjectStatus.ON_COMPLIANCE_REVIEW.eq(w.getStatus()));
        retDetail.setTotalApprovalAmount(DecimalUtil.sum(validProjects, Project::getApprovalAmount));
        retDetail.setTotalDeclareAmount(DecimalUtil.sum(validProjects, Project::getDeclareAmount));
        retDetail.setTotalReviewAmount(DecimalUtil.sum(validProjects, Project::getReviewAmount));
        retDetail.setCurrOrgApprovalAmount(DecimalUtil.sum(validProjects, Project::getApprovalGovOwnFinanceAmount));
        retDetail.setUpperOrgApprovalAmount(DecimalUtil.sum(validProjects, Project::getApprovalGovSuperiorFinanceAmount));
        List<Long> validProjectIds = CollUtils.fieldList(validProjects, Project::getId);
        // 合同金额
        List<Contract> contracts = contractService.list(Wrappers.lambdaQuery(Contract.class)
                .select(Contract::getTotalAmount, Contract::getId)
                .in(Contract::getProjectId, validProjectIds));
        retDetail.setTotalContractAmount(DecimalUtil.sum(contracts, Contract::getTotalAmount));
        // 系统替代情况统计
        List<Long> projectIds = CollUtils.fieldList(projects, Project::getId);
        Wrapper<ProjectGovSystemReplaceInfos> pgsriQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(ProjectGovSystemReplaceInfos::getProjectId, projectIds);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(pgsriQuery);
        Map<String, List<ProjectGovSystemReplaceInfos>> replaceInfosMap = new HashMap<>();
        for (ProjectGovSystemReplaceInfos systemReplaceInfo : systemReplaceInfos) {
            List<String> replaceTypes = StrUtil.split(systemReplaceInfo.getReplaceType(), ",");
            for (String replaceType : replaceTypes) {
                if (!NumberUtil.isNumber(replaceType)) {
                    replaceType = "其他";
                } else {
                    replaceType = GovSystemReplaceTypeEnum.get(Integer.parseInt(replaceType))
                            .flatMap(t -> Optional.of(t.getVal()))
                            .orElse("其他");
                }
                replaceInfosMap.computeIfAbsent(replaceType, k -> new ArrayList<>()).add(systemReplaceInfo);
            }
        }
        replaceInfosMap.remove("其他");

        List<SystemReplaceStatVO> systemReplaceStat = replaceInfosMap.entrySet().stream().map(w -> {
            String replaceType = w.getKey();
            List<ProjectGovSystemReplaceInfos> replaceInfosList = w.getValue();
            Map<String, List<ProjectGovSystemReplaceInfos>> replaceGroupProject = CollUtils.group(replaceInfosList,
                    t -> t.getProjectCode() + "#" + t.getBatchNo());
            int sourceCount = 0;
            int targetCount = 0;
            for (List<ProjectGovSystemReplaceInfos> currSystems : replaceGroupProject.values()) {
                sourceCount += CollUtil.count(CollUtils.fieldSet(currSystems, ProjectGovSystemReplaceInfos::getSourceSystem),
                        IS_VALID_SYSTEM::test);
                targetCount += CollUtil.count(CollUtils.fieldSet(currSystems, ProjectGovSystemReplaceInfos::getTargetSystem),
                        IS_VALID_SYSTEM::test);
            }
            return SystemReplaceStatVO.builder()
                    .sourceCount(sourceCount)
                    .targetCount(targetCount)
                    .replaceType(replaceType)
                    .build();
        }).collect(Collectors.toList());
        Pair<Integer, Integer> totalCount = countSystemReplaceInfos(systemReplaceInfos);
        retDetail.setTotalSourceSystemCount(totalCount.getKey());
        retDetail.setTotalTargetSystemCount(totalCount.getValue());
        retDetail.setSystemReplaceInfos(systemReplaceStat);
        return retDetail;
    }

    public static final Predicate<String> IS_VALID_SYSTEM = x -> StrUtil.isNotBlank(x) && !"/".equals(x);

    private Pair<Integer, Integer> countReplaceSystemByProjectIds(Collection<Long> projectIds) {
        List<String> projectCodes = CollUtils.convert(projectIds, ProjectIdCodeCacheUtil::get);
        return countReplaceSystemByProjectCodes(projectCodes);
    }

    private Pair<Integer, Integer> countReplaceSystemByProjectCodes(List<String> projectCodes) {
        if (CollUtil.isEmpty(projectCodes)) {
            return Pair.create(0, 0);
        }
        Wrapper<ProjectGovSystemReplaceInfos> query = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .select(ProjectGovSystemReplaceInfos::getSourceSystem, ProjectGovSystemReplaceInfos::getTargetSystem,
                        ProjectGovSystemReplaceInfos::getProjectCode, ProjectGovSystemReplaceInfos::getBatchNo)
                .in(ProjectGovSystemReplaceInfos::getProjectCode, projectCodes);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(query);
        return countSystemReplaceInfos(systemReplaceInfos);
    }

    private static Pair<Integer, Integer> countSystemReplaceInfos(List<ProjectGovSystemReplaceInfos> systemReplaceInfos) {
        Collection<Pair<Integer, Integer>> countByProject = systemReplaceInfos.stream()
                .collect(Collectors.groupingBy(w -> w.getProjectCode() + "#" + w.getBatchNo(),
                        Collectors.collectingAndThen(Collectors.toList(),
                                w -> {
                                    Set<String> sourceNames = CollUtils.fieldSet(w, ProjectGovSystemReplaceInfos::getSourceSystem);
                                    int sourceCount = CollUtil.count(sourceNames, IS_VALID_SYSTEM::test);
                                    Set<String> targetNames = CollUtils.fieldSet(w, ProjectGovSystemReplaceInfos::getTargetSystem);
                                    int targetCount = CollUtil.count(targetNames, IS_VALID_SYSTEM::test);
                                    return Pair.create(sourceCount, targetCount);
                                }))).values();
        int totalTargetCount = 0;
        int totalSourceCount = 0;
        for (Pair<Integer, Integer> pair : countByProject) {
            totalSourceCount += pair.getKey();
            totalTargetCount += pair.getValue();
        }
        return Pair.create(totalSourceCount, totalTargetCount);
    }


    public List<String> getProjectCodes(Map<String, List<ProjectStatusChange>> projectStatusChanges,
                                        ProjectStateChangeEvent event) {
        return projectStatusChanges.entrySet().stream()
                .filter(w -> {
                    List<ProjectStatusChange> changes = w.getValue();
                    /*if (CollUtil.anyMatch(changes, w1 -> ProjectStateChangeEvent.STOPPED_APPLY_PASS.eq(w1.getEvent())
                            || ProjectStateChangeEvent.STOPPED_APPLY_AUTO_PASS.eq(w1.getEvent()))) {
                        return false;
                    }*/
                    return CollUtil.anyMatch(w.getValue(), w1 -> event.eq(w1.getEvent()));
                })
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
    }

    public Integer processNodeNotFinishedCount(Map<String, List<ProjectStatusChange>> projectStatusChanges,
                                               ProjectStateChangeEvent startEvent,
                                               ProjectStateChangeEvent finishEvent,
                                               Map<Long, List<Purchase>> projectPurchaseMap,
                                               Map<String, Project> projectMap) {
        return CollUtil.count(projectStatusChanges.entrySet(), w -> {
            List<ProjectStatusChange> changes = w.getValue();
            if (CollUtil.anyMatch(changes, w1 -> ProjectStateChangeEvent.STOPPED_APPLY_PASS.eq(w1.getEvent())
                    || ProjectStateChangeEvent.STOPPED_APPLY_AUTO_PASS.eq(w1.getEvent()))) {
                return false;
            }
            if (startEvent.equals(ProjectStateChangeEvent.ANNUAL_PLAN_PASS)) {
                Project project = projectMap.get(w.getKey());
                if (project.getApprovalAmount() == null || !NumberUtil.isGreater(project.getApprovalAmount(), BigDecimal.ZERO)) {
                    return false;
                }
            }
            if (startEvent.equals(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD)) {
                Long projectId = ProjectIdCodeCacheUtil.newest(w.getKey());
                List<Purchase> purchases = projectPurchaseMap.getOrDefault(projectId, Collections.emptyList());
                if (!CollUtil.anyMatch(purchases, x -> BidTypeEnum.BUILD_APP.eq(x.getBidType()))) {
                    return false;
                }
            }
            return CollUtil.anyMatch(changes, w1 -> startEvent.eq(w1.getEvent()))
                    && CollUtil.allMatch(changes, w1 -> !finishEvent.eq(w1.getEvent()));
        });
    }


    public List<ProjectProcessStatVO> projectProcessStageStatistics(WorkbenchReq req) {
        initWorkbenchDefaultParam(req);
        List<ProjectProcessStatVO> retData = new ArrayList<>();
        List<Long> allProjectIds = new ArrayList<>();
        List<String> allProjectCodes = new ArrayList<>();
        Map<String, Map<Long, Long>> purchaseEventMap = new HashMap<>();
        Map<Long, List<Purchase>> projectPurchaseCountMap = new HashMap<>();
        // 查询完成立项备案的项目
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getApprovalAmount, Project::getProjectCode, Project::getStatus, Project::getAnnualPlanOpened)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .eq(Project::getNewest, Boolean.TRUE)
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE + " and npsc.event = {0}",
                        ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT)
                .in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(query);
        Map<String, Project> projectMap = CollUtils.listToMap(projects, Project::getProjectCode);
        Map<String, List<ProjectStatusChange>> projectStatusChangeMap = new HashMap<>();
        for (WorkbenchProcessNode node : WorkbenchProcessNode.ALL) {
            if (Arrays.asList(WorkbenchProcessNode.FINISHED,
                    WorkbenchProcessNode.SUBMIT_START_FILE,
                    WorkbenchProcessNode.ORG_CONFIRM,
                    WorkbenchProcessNode.SELF_TEST,
                    WorkbenchProcessNode.TEST_VALID).contains(node)) {
                continue;
            }
            if (!projects.isEmpty()) {
                ProjectProcessStatVO currStat = ProjectProcessStatVO.builder()
                        .stageName(node.getName())
                        .stage(node)
                        .build();
                switch (node) {
                    case PROJECT_DECLARED: {
                        // 所有项目的状态变更记录
                        Wrapper<ProjectStatusChange> pscQuery = Wrappers.lambdaQuery(ProjectStatusChange.class)
                                .in(ProjectStatusChange::getProjectCode, CollUtils.fieldList(projects, Project::getProjectCode));
                        List<ProjectStatusChange> projectStatusChanges = projectStatusChangeService.list(pscQuery);
                        projectStatusChangeMap.putAll(CollUtils.group(projectStatusChanges, ProjectStatusChange::getProjectCode));
                        // 移除已停止项目
                        //projects.removeIf(w -> ProjectStatus.STOPPED_PASSED.eq(w.getStatus()));
                        currStat.setProjectCount(projects.size());
                        for (Project project : projects) {
                            allProjectCodes.add(project.getProjectCode());
                            allProjectIds.addAll(ProjectIdCodeCacheUtil.get(project.getProjectCode()));
                        }
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT, ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS));
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectCodes(allProjectCodes);
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                    }
                    break;
                    case PROJECT_REVIEW: {
                        List<String> projectCodes = getProjectCodes(projectStatusChangeMap, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                        currStat.setProjectCount(projectCodes.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS, ProjectStateChangeEvent.PROJECT_REVIEW_PASS));
                        int reviewFailed = lastMatchProjectCount(projectStatusChangeMap, ProjectStateChangeEvent.PROJECT_REVIEW_FAILED);
                        int reviewing = lastMatchProjectCount(projectStatusChangeMap, ProjectStateChangeEvent.PROJECT_REVIEW_SUBMIT,
                                ProjectStateChangeEvent.PROJECT_REVIEW_RESUBMIT);
                        currStat.setReviewingCount(reviewing);
                        currStat.setReviewFailedCount(reviewFailed);
                        currStat.setPendingCount(processNodeNotFinishedCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT, ProjectStateChangeEvent.PROJECT_REVIEW_PASS,
                                null, null));
                    }
                    break;
                    case APPROVAL_AMOUNT: {
                        List<String> projectCodes = projects.stream()
                                .filter(w -> w.getApprovalAmount() != null
                                        && NumberUtil.isGreater(w.getApprovalAmount(), BigDecimal.ZERO)
                                        && Boolean.TRUE.equals(w.getAnnualPlanOpened()))
                                .map(Project::getProjectCode)
                                .collect(Collectors.toList());
                        currStat.setProjectCount(projectCodes.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.PROJECT_REVIEW_PASS, ProjectStateChangeEvent.ANNUAL_PLAN_PASS));
                        currStat.setPendingCount(CollUtil.count(projects, w -> Boolean.TRUE.equals(w.getAnnualPlanOpened())
                                && (w.getApprovalAmount() == null || NumberUtil.isLessOrEqual(w.getApprovalAmount(), BigDecimal.ZERO))));
                    }
                    break;
                    case PROJECT_APPROVAL: {
                        List<String> projectCodes = getProjectCodes(projectStatusChangeMap, ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                        currStat.setProjectCount(projectCodes.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.ANNUAL_PLAN_PASS, ProjectStateChangeEvent.DECLARED_RECORD_PASS));
                        currStat.setPendingCount(processNodeNotFinishedCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.ANNUAL_PLAN_PASS, ProjectStateChangeEvent.DECLARED_RECORD_PASS,
                                null, projectMap));
                    }
                    break;
                    case PROJECT_PURCHASE: {
                        List<String> projectCodes = getProjectCodes(projectStatusChangeMap, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                        currStat.setProjectCount(projectCodes.size());
                        if (!projectCodes.isEmpty()) {
                            Wrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                                    .select(Purchase::getId, Purchase::getProjectId, Purchase::getBidType, Purchase::getStatus)
                                    .in(Purchase::getProjectId, allProjectIds);
                            List<Purchase> purchases = purchaseService.list(purchaseQuery);
                            projectPurchaseCountMap.putAll(CollUtils.group(purchases, Purchase::getProjectId));
                            Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                            currStat.setSourceCount(systemReplaceCount.getKey());
                            currStat.setTargetCount(systemReplaceCount.getValue());
                            currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                    ProjectStateChangeEvent.DECLARED_RECORD_PASS, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD));
                            Wrapper<PurchaseStatusChange> pQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                                    .select(PurchaseStatusChange::getEvent, PurchaseStatusChange::getProjectId)
                                    .in(PurchaseStatusChange::getBidId, CollUtils.fieldList(purchases, Purchase::getId))
                                    .in(PurchaseStatusChange::getEvent,
                                            ADAPT_INFO_PASSED,
                                            SELF_TEST_PASSED,
                                            TEST_VALID_INFO_PASSED,
                                            TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES,
                                            XcfhxStateChangeEvent.XCFHX_APPLY_PASSED,
                                            TenderStateChangeEvent.FINALLY_INSPECTED_PASSED,
                                            SUBMIT_PURCHASE_ORG_CONFIRM,
                                            TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO);
                            List<PurchaseStatusChange> purchaseChanges = purchaseStatusChangeService.list(pQuery);
                            Map<String, Map<Long, Long>> purchaseEventMapTmp = purchaseChanges.stream()
                                    .collect(Collectors.groupingBy(PurchaseStatusChange::getEvent,
                                            Collectors.collectingAndThen(Collectors.toList(),
                                                    w -> CollUtils.groupCount(w, PurchaseStatusChange::getProjectId))));
                            purchaseEventMap.putAll(purchaseEventMapTmp);
                        }
                        currStat.setPendingCount(processNodeNotFinishedCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.DECLARED_RECORD_PASS, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD,
                                null, null));
                    }
                    break;
                    case PROJECT_ADAPTION: {
                        List<String> projectIds = getProjectCodes(projectStatusChangeMap, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                        currStat.setProjectCount(projectIds.size());
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectCodes(projectIds);
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                        Map<Long, Long> submitPurcahseRecordMap = purchaseEventMap.get(TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO.name());

                        if (submitPurcahseRecordMap != null) {
                            List<Long> tmpProjectIds = submitPurcahseRecordMap.entrySet().stream()
                                    .filter(w -> w.getValue() > 0)
                                    .map(Map.Entry::getKey)
                                    .collect(Collectors.toList());
                            if (!tmpProjectIds.isEmpty()) {
                                currStat.setAdaptFinishStat(tenderAdaptStatistics(tmpProjectIds, projectPurchaseCountMap));
                            }
                        }
                        long finishPurchaseByWithoutApp = CollUtil.count(projectPurchaseCountMap.entrySet(), w -> {
                            List<Purchase> purchases = w.getValue();
                            return CollUtil.allMatch(purchases, x -> !BidTypeEnum.BUILD_APP.eq(x.getBidType()))
                                    && CollUtil.allMatch(purchases, x -> TenderMainStatus.FINALLY_INSPECTED_PASSED.eq(x.getStatus()));
                        });
                        TenderAdaptFinishStatVO adaptFinishStat = currStat.getAdaptFinishStat();
                        if (adaptFinishStat == null) {
                            currStat.setAdaptFinishStat(new TenderAdaptFinishStatVO());
                            adaptFinishStat = currStat.getAdaptFinishStat();
                        }
                        adaptFinishStat.setFinishPurchaseCountWithoutApp(finishPurchaseByWithoutApp);
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM));
                        currStat.setPendingCount(processNodeNotFinishedCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM,
                                projectPurchaseCountMap, null));
                    }
                    break;
                    case FIRST_INSPECTED: {
                        List<String> projectCodes = getProjectCodes(projectStatusChangeMap, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setProjectCount(projectCodes.size());
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES));
                        currStat.setPendingCount(processNodeNotFinishedCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES,
                                null, null));
                    }
                    break;
                    case FINAL_INSPECTED: {
                        List<String> projectCodes = getProjectCodes(projectStatusChangeMap, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS);
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setProjectCount(projectCodes.size());
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS));
                        currStat.setPendingCount(processNodeNotFinishedCount(projectStatusChangeMap,
                                ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS,
                                null, null));
                    }
                    break;
                    default:
                        break;
                }
                retData.add(currStat);
            }
        }
        return retData;
    }

    private TenderAdaptFinishStatVO tenderAdaptStatistics(List<Long> projectIds, Map<Long, List<Purchase>> projectPurchaseCountMap) {
        Wrapper<PurchaseStatusChange> purchaseQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                .select(PurchaseStatusChange::getEvent, PurchaseStatusChange::getBidId, PurchaseStatusChange::getProjectId)
                .in(PurchaseStatusChange::getProjectId, projectIds)
                .in(PurchaseStatusChange::getEvent, ADAPT_INFO_PASSED,
                        SELF_TEST_PASSED,
                        TEST_VALID_INFO_PASSED,
                        SUBMIT_PURCHASE_ORG_CONFIRM);
        List<PurchaseStatusChange> statusChanges = purchaseStatusChangeService.list(purchaseQuery);

        Map<Long, List<PurchaseStatusChange>> projectPurchaseStatusChangeMap = CollUtils.group(statusChanges, PurchaseStatusChange::getProjectId);

        TenderAdaptFinishStatVO stat = new TenderAdaptFinishStatVO();
        stat.setFinishTestValidCount(countAdaptBranchStatus(TEST_VALID_INFO_PASSED, projectIds, projectPurchaseCountMap, projectPurchaseStatusChangeMap));
        stat.setFinishAdaptionInfo(countAdaptBranchStatus(ADAPT_INFO_PASSED, projectIds, projectPurchaseCountMap, projectPurchaseStatusChangeMap));
        stat.setFinishOrgConfirmCount(countAdaptBranchStatus(SUBMIT_PURCHASE_ORG_CONFIRM, projectIds, projectPurchaseCountMap, projectPurchaseStatusChangeMap));
        stat.setFinishSelfTestCount(countAdaptBranchStatus(SELF_TEST_PASSED, projectIds, projectPurchaseCountMap, projectPurchaseStatusChangeMap));
        stat.setPendingAdaptionInfo(projectPurchaseProcessNodeNotFinishCount(projectPurchaseCountMap,
                projectPurchaseStatusChangeMap, SUBMIT_PURCHASE_CONSTRUCTION_INFO, ADAPT_INFO_PASSED));
        stat.setPendingSelfTestCount(projectPurchaseProcessNodeNotFinishCount(projectPurchaseCountMap,
                projectPurchaseStatusChangeMap, ADAPT_INFO_PASSED, SELF_TEST_PASSED));
        stat.setPendingTestValidCount(projectPurchaseProcessNodeNotFinishCount(projectPurchaseCountMap,
                projectPurchaseStatusChangeMap, SELF_TEST_PASSED, TEST_VALID_INFO_PASSED));
        stat.setPendingOrgConfirmCount(projectPurchaseProcessNodeNotFinishCount(projectPurchaseCountMap,
                projectPurchaseStatusChangeMap, TEST_VALID_INFO_PASSED, SUBMIT_PURCHASE_ORG_CONFIRM));
        return stat;
    }

    private static <T extends AbstractStateChangeEvent> int countAdaptBranchStatus(T event,
                                                                                   List<Long> projectIds,
                                                                                   Map<Long, List<Purchase>> projectPurchaseCountMap,
                                                                                   Map<Long, List<PurchaseStatusChange>> projectPurchaseChangeMap) {
        return (int) projectIds.stream().filter(w -> {
            int bidCount = CollUtil.count(projectPurchaseCountMap.getOrDefault(w, Collections.emptyList()),
                    w1 -> BidTypeEnum.BUILD_APP.eq(w1.getBidType()));
            List<PurchaseStatusChange> purchaseStatusChanges = projectPurchaseChangeMap.getOrDefault(w, Collections.emptyList());
            int finishedBidCount = purchaseStatusChanges.stream()
                    .filter(w1 -> w1.getProjectId().equals(w) && event.eq(w1.getEvent()))
                    .map(PurchaseStatusChange::getBidId)
                    .collect(Collectors.toSet()).size();
            return bidCount > 0 && finishedBidCount == bidCount;
        }).count();
    }

    private static <T extends AbstractStateChangeEvent> int projectPurchaseProcessNodeNotFinishCount(Map<Long, List<Purchase>> projectPurchaseCountMap,
                                                                                                     Map<Long, List<PurchaseStatusChange>> projectPurchaseChangeMap,
                                                                                                     T startEvent,
                                                                                                     T finishEvent) {
        return (int) projectPurchaseCountMap.entrySet().stream().filter(w -> {
            Long projectId = w.getKey();
            int bidCount = CollUtil.count(projectPurchaseCountMap.getOrDefault(projectId, Collections.emptyList()),
                    w1 -> BidTypeEnum.BUILD_APP.eq(w1.getBidType()));
            List<PurchaseStatusChange> purchaseStatusChanges = projectPurchaseChangeMap.getOrDefault(projectId, Collections.emptyList());
            int startFinishBidCount = purchaseStatusChanges.stream()
                    .filter(w1 -> w1.getProjectId().equals(projectId) && startEvent.eq(w1.getEvent()))
                    .map(PurchaseStatusChange::getBidId)
                    .collect(Collectors.toSet()).size();
            int finishBidCount = purchaseStatusChanges.stream()
                    .filter(w1 -> w1.getProjectId().equals(projectId) && finishEvent.eq(w1.getEvent()))
                    .map(PurchaseStatusChange::getBidId)
                    .collect(Collectors.toSet()).size();
            return bidCount != 0 && startFinishBidCount == bidCount && finishBidCount != bidCount;
        }).count();
    }

    private static int stoppedProjectCount(Map<String, List<ProjectStatusChange>> projectStatusChangeMap,
                                           AbstractStateChangeEvent startEvent,
                                           AbstractStateChangeEvent nextStartEvent) {
        return CollUtil.count(projectStatusChangeMap.values(), w ->
                CollUtil.anyMatch(w, x -> startEvent.eq(x.getEvent())) &&
                        CollUtil.allMatch(w, x -> !nextStartEvent.eq(x.getEvent())) &&
                        CollUtil.anyMatch(w, x -> ProjectStateChangeEvent.STOPPED_APPLY_PASS.eq(x.getEvent())));
    }

    private static int lastMatchProjectCount(Map<String, List<ProjectStatusChange>> projectStatusChangeMap,
                                             ProjectStateChangeEvent... events) {
        List<ProjectStateChangeEvent> changeEvents = Arrays.asList(events);
        return CollUtil.count(projectStatusChangeMap.entrySet(),
                w -> {
                    w.getValue().sort(Comparator.comparing(ProjectStatusChange::getId).reversed());
                    return changeEvents.stream().anyMatch(x -> x.eq(w.getValue().get(0).getEvent()));
                });
    }

    public PageVo<ProjectLibListItemVO> pageProjectLib(WorkbenchProjectLibReq req) {
        LambdaQueryWrapper<Project> query = workbenchProjectLibQuery(req);
        if (query == null) {
            return PageVo.empty();
        }
        ProjectManageUtil.projectBaseQuery(query);
        query.notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED);
        projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT);
        switch (req.getProcessNode()) {
            case PROJECT_DECLARED:
                break;
            case PROJECT_REVIEW:
                projectProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                break;
            case APPROVAL_AMOUNT:
                query.eq(Project::getAnnualPlanOpened, Boolean.TRUE);
                if (Boolean.FALSE.equals(req.getProcessNodeFinished())) {
                    query.ne(Project::getStatus, ProjectStatus.STOPPED_PASSED.getCode());
                    query.and(q1 -> q1.isNull(Project::getApprovalAmount)
                            .or(q2 -> q2.le(Project::getApprovalAmount, BigDecimal.ZERO)));
                } else {
                    query.isNotNull(Project::getApprovalAmount).gt(Project::getApprovalAmount, BigDecimal.ZERO);
                }
                break;
            case PROJECT_APPROVAL:
                query.isNotNull(Project::getApprovalAmount).gt(Project::getApprovalAmount, BigDecimal.ZERO);
                projectProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ProjectStateChangeEvent.ANNUAL_PLAN_PASS, ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                break;
            case PROJECT_PURCHASE:
                projectProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ProjectStateChangeEvent.DECLARED_RECORD_PASS, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                break;
            case FIRST_INSPECTED:
                projectProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                break;
            case ORG_CONFIRM:
                purchaseProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        TEST_VALID_INFO_PASSED, SUBMIT_PURCHASE_ORG_CONFIRM);
                break;
            case PROJECT_ADAPTION:
                query.exists(ExistsSqlConst.PROJECT_EXISTS_PURCHASE
                        + "and np.bid_type = {0}", BidTypeEnum.BUILD_APP.getCode());
                projectProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                break;
            case FINAL_INSPECTED:
                projectProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS);
                break;
            case FINISHED:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                query.notExists("select 1 from nd_purchase np where np.project_id = nd_project.id and bid_type = {0}",
                        BidTypeEnum.BUILD_APP.getCode());
                break;
            case TEST_VALID:
                purchaseProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        SELF_TEST_PASSED, TEST_VALID_INFO_PASSED);
                break;
            case SELF_TEST:
                purchaseProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        ADAPT_INFO_PASSED, SELF_TEST_PASSED);
                break;
            case SUBMIT_START_FILE:
                purchaseProcessNodeFinishedQuery(query, req.getProcessNodeFinished(),
                        SUBMIT_OPERATION_PLAN, ADAPT_INFO_PASSED);
                break;
            default:
                return PageVo.empty();
        }
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<String> projectCodes = CollUtils.fieldList(page.getRecords(), Project::getProjectCode);
        Map<Long, Integer> bidStatusMap = new HashMap<>();
        Map<String, List<ProjectGovSystemReplaceInfos>> systemReplaceInfoMap = new HashMap<>();
        fillSystemReplaceInfosAndReferBidStatus(projectCodes, systemReplaceInfoMap, bidStatusMap);
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(),
                w -> {
                    ProjectLibListItemVO item = ProjectLibListItemVO.builder()
                            .id(w.getId())
                            .fromType("1")
                            .projectName(w.getProjectName())
                            .projectCode(w.getProjectCode())
                            .projectType(w.getProjectType())
                            .projectYear(w.getProjectYear())
                            .buildOrg(w.getBuildOrgName())
                            .buildOrgCode(w.getBuildOrgCode())
                            .approvedAmount(w.getApprovalAmount())
                            .reviewAmount(w.getReviewAmount())
                            .declaredAmount(w.getDeclareAmount())
                            .createOn(w.getCreateOn())
                            .stage(w.getStage())
                            .status(w.getStatus())
                            .build();
                    buildReferSystemReplaceInfos(systemReplaceInfoMap, bidStatusMap, item);
                    return item;
                });
        return PageVo.of(records, page.getTotal());
    }

    private static void projectProcessNodeFinishedQuery(LambdaQueryWrapper<Project> query,
                                                        Boolean processNodeFinished,
                                                        AbstractStateChangeEvent startEvent,
                                                        AbstractStateChangeEvent finishEvent) {
        if (Boolean.FALSE.equals(processNodeFinished)) {
            query.ne(Project::getStatus, ProjectStatus.STOPPED_PASSED.getCode());
            projectQueryExistsProjectStatusChange(query, startEvent);
            projectQueryNotExistsProjectStatusChange(query, finishEvent);
        } else {
            projectQueryExistsProjectStatusChange(query, finishEvent);
        }
    }

    private static void purchaseProcessNodeFinishedQuery(LambdaQueryWrapper<Project> query,
                                                         Boolean processNodeFinished,
                                                         AbstractStateChangeEvent startEvent,
                                                         AbstractStateChangeEvent finishEvent) {
        if (Boolean.FALSE.equals(processNodeFinished)) {
            query.ne(Project::getStatus, ProjectStatus.STOPPED_PASSED.getCode());
            allPurchaseFinished(query, startEvent);
            anyPurchaseNotFinished(query, finishEvent);
        } else {
            allPurchaseFinished(query, finishEvent);
        }
    }

    private static void allPurchaseFinished(LambdaQueryWrapper<Project> query, AbstractStateChangeEvent event) {
        query.apply("(select t.bidCnt from (select count(1) bidCnt from nd_purchase np where np.project_id = nd_project.id and bid_type = {0}) t where bidCnt > 0) " +
                "= (select count(distinct bid_id) from nd_purchase_status_change npsc where npsc.project_code = nd_project.project_code " +
                "and event = {1})", BidTypeEnum.BUILD_APP.getCode(), event);
    }

    private static void anyPurchaseNotFinished(LambdaQueryWrapper<Project> query, AbstractStateChangeEvent event) {
        query.apply("(select t.bidCnt from (select count(1) bidCnt from nd_purchase np where np.project_id = nd_project.id and bid_type = {0}) t where bidCnt > 0) " +
                "!= (select count(distinct bid_id) from nd_purchase_status_change npsc where npsc.project_code = nd_project.project_code " +
                "and event = {1})", BidTypeEnum.BUILD_APP.getCode(), event);
    }

    private void fillSystemReplaceInfosAndReferBidStatus(List<String> projectCodes,
                                                         Map<String, List<ProjectGovSystemReplaceInfos>> systemReplaceInfoMap,
                                                         Map<Long, Integer> bidStatusMap) {
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.listByProjectCodes(projectCodes);
        if (CollUtil.isNotEmpty(systemReplaceInfos)) {
            systemReplaceInfoMap.putAll(CollUtils.group(systemReplaceInfos, ProjectGovSystemReplaceInfos::getProjectCode));
            List<Long> bidIds = CollUtils.fieldList(systemReplaceInfos, ProjectGovSystemReplaceInfos::getBidId);
            if (CollUtil.isNotEmpty(bidIds)) {
                List<Purchase> purchases = purchaseService.listByIds(bidIds);
                for (Purchase purchase : purchases) {
                    bidStatusMap.put(purchase.getId(), purchase.getStatus());
                }
            }
        }
    }

    private static void buildReferSystemReplaceInfos(Map<String, List<ProjectGovSystemReplaceInfos>> systemReplaceInfoMap, Map<Long, Integer> bidStatusMap, ProjectLibListItemVO item) {
        if (systemReplaceInfoMap.containsKey(item.getProjectCode())) {
            List<ProjectGovSystemReplaceInfos> currSystemReplaceInfos = systemReplaceInfoMap.get(item.getProjectCode());
            currSystemReplaceInfos.sort(Comparator.comparing(ProjectGovSystemReplaceInfos::getBatchNo));
            List<WorkbenchProjectLibSystemReplaceInfoVO> retSystemReplaceInfos = CollUtils.convert(currSystemReplaceInfos,
                    w1 -> WorkbenchProjectLibSystemReplaceInfoVO.builder()
                            .sourceSystem(w1.getSourceSystem())
                            .targetSystem(w1.getTargetSystem())
                            .replaceType(w1.getReplaceType())
                            .sourceSystemId(w1.getSourceSystemId())
                            .isInMhSystemReplaceInfos(StrUtil.isNotBlank(w1.getSourceSystemId()))
                            .referBidStatus(bidStatusMap.get(w1.getBidId()))
                            .build());
            item.setReferSystemReplaceInfos(retSystemReplaceInfos);
        }
    }

    public SystemReplaceInfoVO listSystemReplaceInfos(WorkbenchProjectLibReq req) {
        LambdaQueryWrapper<Project> query = workbenchProjectLibQuery(req);
        if (query == null) {
            return null;
        }
        query.notIn(Project::getStatus, ProjectStatusConstant.CHANGE_STOP_DELETE_PASSED)
                .select(Project::getProjectCode, Project::getProjectName, Project::getId,
                        Project::getBuildOrgCode, Project::getBuildOrgName);
        projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT);
        switch (req.getProcessNode()) {
            case PROJECT_DECLARED:
                break;
            case PROJECT_REVIEW:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                break;
            case APPROVAL_AMOUNT:
                query.isNotNull(Project::getApprovalAmount).gt(Project::getApprovalAmount, BigDecimal.ZERO);
                break;
            case PROJECT_APPROVAL:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT);
                break;
            case PROJECT_PURCHASE:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                break;
            case FIRST_INSPECTED:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                break;
            case PROJECT_ADAPTION:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                break;
            case FINAL_INSPECTED:
                projectQueryExistsProjectStatusChange(query, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS);
                break;
            default:
                return null;
        }
        List<Project> projects = projectService.list(query);
        return buildSystemReplaceInfos(projects);
    }

    private LambdaQueryWrapper<Project> workbenchProjectLibQuery(WorkbenchProjectLibReq req) {
        Assert.notNull(req.getProcessNode(), "节点不能为空");
        Long unitId = req.getUnitId();
        UnitQueryState unitQueryState = mhUnitQueryAuthHelper.listCanViewUnitIds(unitId);
        if (!unitQueryState.isState()) {
            return null;
        }
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(req.getStatus() != null, Project::getStatus, req.getStatus())
                .eq(req.getProjectYear() != null, Project::getProjectYear, req.getProjectYear())
                .like(StrUtil.isNotBlank(req.getBuildOrg()), Project::getBuildOrgName, req.getBuildOrg())
                .like(StrUtil.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .ge(req.getReviewAmountMin() != null, Project::getReviewAmount, req.getReviewAmountMin())
                .le(req.getReviewAmountMax() != null, Project::getReviewAmount, req.getReviewAmountMax())
                .ge(req.getApproveAmountMin() != null, Project::getApprovalAmount, req.getApproveAmountMin())
                .le(req.getApproveAmountMax() != null, Project::getApprovalAmount, req.getApproveAmountMax())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .exists(req.getBidStatus() != null, ExistsSqlConst.PROJECT_EXISTS_PURCHASE
                        + " and status = {0}", req.getBidStatus())
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getUpdateOn);
        List<Long> unitIds = unitQueryState.getUnitIds();
        if (CollUtil.isNotEmpty(unitIds)) {
            query.in(Project::getBuildOrgCode, CollUtils.convert(unitIds, String::valueOf));
        }
        if (req.getReplaceType() != null || StrUtil.isNotBlank(req.getSourceSystem()) || req.getIsInMhSystemReplaceInfos() != null) {
            StringBuilder sql = new StringBuilder();
            if (req.getReplaceType() != null) {
                sql.append(StrUtil.format(" and npgsri.replace_type = '{}'", req.getReplaceType()));
            }
            if (StrUtil.isNotBlank(req.getSourceSystem())) {
                sql.append(StrUtil.format(" and npgsri.source_system like '%{}%'", req.getSourceSystem()));
            }
            if (req.getIsInMhSystemReplaceInfos() != null) {
                char symbol = (Boolean.TRUE.equals(req.getIsInMhSystemReplaceInfos()) ? '>' : '=');
                sql.append(" and NVL(LENGTH(source_system_id),0) ").append(symbol).append(" 0");
            }
            query.exists(ExistsSqlConst.PROJECT_EXISTS_SYSTEM_REPLACE_INFOS + sql);
        }
        return query;
    }

    private SystemReplaceInfoVO buildSystemReplaceInfos(List<Project> projects) {
        if (projects.isEmpty()) {
            return null;
        }
        List<String> projectCodes = CollUtils.fieldList(projects, Project::getProjectCode);
        Wrapper<ProjectGovSystemReplaceInfos> query = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(ProjectGovSystemReplaceInfos::getProjectCode, projectCodes);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(query);
        if (systemReplaceInfos.isEmpty()) {
            return null;
        }
        SystemReplaceInfoVO retDetail = new SystemReplaceInfoVO();
        Map<String, List<ProjectGovSystemReplaceInfos>> projectSystemMap = CollUtils.group(systemReplaceInfos,
                ProjectGovSystemReplaceInfos::getProjectCode);
        Map<String, List<Project>> unitProjectMap = CollUtils.group(projects, Project::getBuildOrgCode);
        List<UnitProjectGovSystemReplaceVO> retList = new ArrayList<>();
        AtomicInteger sourceCount = new AtomicInteger(0);
        AtomicInteger targetCount = new AtomicInteger(0);
        for (Map.Entry<String, List<Project>> entry : unitProjectMap.entrySet()) {
            List<Project> tmpProjects = entry.getValue();
            List<ProjectGovSystemReplaceVO> currOrgReplaceInfos = new ArrayList<>();
            for (Project proj : tmpProjects) {
                List<ProjectGovSystemReplaceInfos> replaceInfos = projectSystemMap.get(proj.getProjectCode());
                if (replaceInfos != null) {
                    ProjectGovSystemReplaceVO projectSystem = new ProjectGovSystemReplaceVO();
                    projectSystem.setProjectId(proj.getId());
                    projectSystem.setProjectCode(proj.getProjectCode());
                    projectSystem.setProjectName(proj.getProjectName());
                    currOrgReplaceInfos.add(projectSystem);
                    Map<Integer, List<ProjectGovSystemReplaceInfos>> replaceInfoGroup = CollUtils.group(replaceInfos,
                            ProjectGovSystemReplaceInfos::getBatchNo);
                    List<GovSystemReplaceInfoDTO> replaceInfoList = replaceInfoGroup.entrySet().stream()
                            .sorted(Comparator.comparingInt(Map.Entry::getKey))
                            .map(w -> {
                                List<ProjectGovSystemReplaceInfos> currBatchValues = w.getValue();
                                ProjectGovSystemReplaceInfos firstReplaceInfo = currBatchValues.get(0);
                                GovSystemReplaceInfoDTO replaceInfo = new GovSystemReplaceInfoDTO();
                                replaceInfo.setReplaceType(firstReplaceInfo.getReplaceType());
                                replaceInfo.setTargetSystemName(firstReplaceInfo.getTargetSystem());
                                List<String> sourceSystemNames = CollUtils.fieldList(currBatchValues, val -> {
                                    String sourceSystem = val.getSourceSystem();
                                    if (IS_VALID_SYSTEM.test(sourceSystem)) {
                                        sourceCount.incrementAndGet();
                                    }
                                    return sourceSystem;
                                });
                                replaceInfo.setSourceSystemNames(sourceSystemNames);
                                if (IS_VALID_SYSTEM.test(replaceInfo.getTargetSystemName())) {
                                    targetCount.incrementAndGet();
                                }
                                return replaceInfo;
                            }).collect(Collectors.toList());
                    projectSystem.setReplaceSystems(replaceInfoList);
                }
            }
            if (!currOrgReplaceInfos.isEmpty()) {
                UnitProjectGovSystemReplaceVO item = new UnitProjectGovSystemReplaceVO();
                item.setUnitName(tmpProjects.get(0).getBuildOrgName());
                item.setProjectSystemReplaces(currOrgReplaceInfos);
                retList.add(item);
            }
        }
        retDetail.setBeforeReplace(sourceCount.get());
        retDetail.setAfterReplace(targetCount.get());
        retDetail.setReplaceDetail(retList);
        return retDetail;
    }

    public WorkbenchProjectEarlyWarnVO projectEarlyWarnings(WorkbenchReq req) {
        Long unitId = Optional.ofNullable(req.getUnitId()).orElseGet(LoginUserUtil::getMhUnitId);
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(unitId);
        List<String> viewUnitIdList = CollUtils.convert(viewUnitIds, String::valueOf);
        WorkbenchProjectEarlyWarnVO earlyWarningStat = new WorkbenchProjectEarlyWarnVO();
        LambdaQueryWrapper<ProjectEarlyWarning> pageQuery = Wrappers.lambdaQuery(ProjectEarlyWarning.class)
                .in(ProjectEarlyWarning::getBuildOrgCode, viewUnitIdList);
        QueryWrapper<ProjectEarlyWarning> countQuery = new QueryWrapper<>();
        countQuery.select("count(1) \"total\", status \"status\"");
        countQuery.in("build_org_code", viewUnitIdList);
        if (req.getUnitStrip() != null) {
            String unitStripSql = String.format("select 1 from nd_project np where " +
                    "np.project_code = nd_project_early_warning.project_code " +
                    " and np.newest = 1 and np.unit_strip = %d", req.getUnitStrip());
            countQuery.exists(unitStripSql);
            pageQuery.exists(unitStripSql);
        }
        if (req.getProjectYear() != null) {
            LocalDate startDate = LocalDate.of(req.getProjectYear(), 1, 1);
            LocalDate endDate = LocalDate.now().plusDays(1);
            pageQuery.between(ProjectEarlyWarning::getCreateOn, startDate, endDate);
            countQuery.between("create_on", startDate, endDate);
        }
        countQuery.groupBy("status");
        List<Map<String, Object>> counts = projectEarlyWarningService.listMaps(countQuery);
        Map<String, Integer> countMap = CollUtils.listToMap(counts,
                w -> MapUtil.getStr(w, "status"), w -> MapUtil.getInt(w, "total"));
        earlyWarningStat.setOvertime(countMap.getOrDefault(ProjectEarlyWarningStatusEnum.OVER_TIME.name(), 0));
        earlyWarningStat.setAdvent(countMap.getOrDefault(ProjectEarlyWarningStatusEnum.ADVENT_TIME.name(), 0));
        earlyWarningStat.setTotal(earlyWarningStat.getAdvent() + earlyWarningStat.getOvertime());
        if (counts.isEmpty()) {
            return earlyWarningStat;
        }
        if (req.getProjectEarlyWarningStatus() == null) {
            pageQuery.in(ProjectEarlyWarning::getStatus, ProjectEarlyWarningStatusEnum.OVER_TIME,
                    ProjectEarlyWarningStatusEnum.ADVENT_TIME);
        } else {
            pageQuery.eq(ProjectEarlyWarning::getStatus, req.getProjectEarlyWarningStatus());
        }
        Page<ProjectEarlyWarning> page = Page.of(1, 500);
        if (req.getProjectEarlyWarningStatus() == null) {
            pageQuery.in(ProjectEarlyWarning::getStatus, ProjectEarlyWarningStatusEnum.OVER_TIME,
                    ProjectEarlyWarningStatusEnum.ADVENT_TIME);
        } else {
            pageQuery.eq(ProjectEarlyWarning::getStatus, req.getProjectEarlyWarningStatus());
        }
        projectEarlyWarningService.page(page, pageQuery);
        if (CollUtil.isNotEmpty(page.getRecords())) {
            List<String> projectCodes = CollUtils.fieldList(page.getRecords(), ProjectEarlyWarning::getProjectCode);

            Wrapper<WflowEarlyWarningRecords> wewQuery = Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .in(WflowEarlyWarningRecords::getProjectCode, projectCodes)
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime);
            List<WflowEarlyWarningRecords> warningRecords = earlyWarningRecordsService.list(wewQuery);
            Set<String> projectCodeSet = Sets.newHashSet();
            Map<String, WflowEarlyWarningRecords> warningMap = warningRecords.stream()
                    .filter(w -> projectCodeSet.add(w.getProjectCode()))
                    .collect(Collectors.toMap(WflowEarlyWarningRecords::getProjectCode, w -> w));

            Wrapper<Project> projectQuery = Wrappers.lambdaQuery(Project.class)
                    .select(Project::getProjectCode, Project::getProjectName)
                    .in(Project::getProjectCode, projectCodes)
                    .eq(Project::getNewest, Boolean.TRUE);
            List<Project> projects = projectService.list(projectQuery);
            Map<String, String> projectNameMap = projects.stream()
                    .collect(Collectors.toMap(Project::getProjectCode, Project::getProjectName));

            List<ProjectEarlyWarningVO> warnings = page.getRecords().stream().map(o -> {
                        ProjectEarlyWarningVO vo = BeanUtil.copyProperties(o, ProjectEarlyWarningVO.class);
                        //取最新一条预警信息
                        if (warningMap.containsKey(vo.getProjectCode())) {
                            WflowEarlyWarningRecords wew = warningMap.get(vo.getProjectCode());
                            vo.setWarningTime(wew.getWarningTime());
                            vo.setInstStart(wew.getInstStart());
                            vo.setWarningUserId(wew.getWarningUserId());
                            vo.setNoticeMethod(wew.getNoticeMethod());
                            vo.setNoticeContent(wew.getNoticeContent());
                            vo.setPath(wew.getPath());
                            vo.setBatchUserIds(wew.getBatchUserIds());
                            vo.setRecordId(wew.getId());
                        }
                        vo.setProjectName(projectNameMap.get(o.getProjectCode()));
                        return vo;
                    }).filter(w -> Objects.nonNull(w.getWarningTime()))
                    .sorted(Comparator.comparing(ProjectEarlyWarningVO::getWarningTime).reversed())
                    .collect(Collectors.toList());
            earlyWarningStat.setProjectEarlyWarnings(warnings);
        }
        return earlyWarningStat;
    }

    public List<WorkbenchProjectMonitorEarlyWarnVO> projectMonitorEarlyWarnings(WorkbenchReq req) {
        UserInfoDetails userInfoDetails = LoginUserUtil.userDetailNotNull();
        // 从预警表中查询当前登录用户所能查看的项目
        LambdaQueryWrapper<ProjectEarlyWarning> queryWrapper = Wrappers.lambdaQuery(ProjectEarlyWarning.class);
        buildPermissionWarningWrapper(queryWrapper, userInfoDetails);
        // 查询状态非正常的预警信息
        queryWrapper.eq(ProjectEarlyWarning::getNormal, Boolean.FALSE);
        queryWrapper.ne(ProjectEarlyWarning::getStatus, ProjectEarlyWarningStatusEnum.NORMAL);
        List<ProjectStatus> hiddenProjectStatus = Arrays.asList(ProjectStatus.STOPPED_PASSED, ProjectStatus.CHANGE_APPLY_PASSED);
        String hiddenStatusInSql = BizUtils.inSqlJoin(CollUtils.convert(hiddenProjectStatus, w -> w.getCode().toString()));
        StringBuilder existsProjectSql = new StringBuilder("select 1 from nd_project np where np.project_code = nd_project_early_warning.project_code" +
                " and np.newest = 1 and np.status not in " + hiddenStatusInSql);
        if (req.getUnitStrip() != null) {
            existsProjectSql.append(" and np.unit_strip = ").append(req.getUnitStrip());
        }
        if (req.getProjectYear() != null) {
            existsProjectSql.append(" and np.project_year = ").append(req.getProjectYear());
        }
        queryWrapper.exists(existsProjectSql.toString());
        List<ProjectEarlyWarning> records = projectEarlyWarningService.list(queryWrapper);
        if (Objects.nonNull(req.getUnitId())) {
            List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
            List<String> queryMhUnitIdList = CollUtils.convert(viewUnitIds, String::valueOf);
            records = records.stream().filter(record -> queryMhUnitIdList.contains(record.getBuildOrgCode())).collect(Collectors.toList());
        }
        if (CollUtil.isNotEmpty(records)) {
            List<String> projectCodes = CollUtils.fieldList(records, ProjectEarlyWarning::getProjectCode);

            Wrapper<WflowEarlyWarningRecords> wewQuery = Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .in(WflowEarlyWarningRecords::getProjectCode, projectCodes)
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime);
            List<WflowEarlyWarningRecords> warningRecords = earlyWarningRecordsService.list(wewQuery);
            Set<String> projectCodeSet = Sets.newHashSet();
            Map<String, WflowEarlyWarningRecords> warningMap = warningRecords.stream()
                    .filter(w -> projectCodeSet.add(w.getProjectCode()))
                    .collect(Collectors.toMap(WflowEarlyWarningRecords::getProjectCode, w -> w));

            return records.stream().map(o -> {
                        WorkbenchProjectMonitorEarlyWarnVO vo = new WorkbenchProjectMonitorEarlyWarnVO();
                        vo.setStepName(o.getStepName());
                        //取最新一条预警信息
                        if (warningMap.containsKey(o.getProjectCode())) {
                            WflowEarlyWarningRecords wew = warningMap.get(o.getProjectCode());
                            vo.setBuildOrgCode(wew.getBuildOrgCode());
                            vo.setBuildOrgName(wew.getBuildOrgName());
                            vo.setInstStart(wew.getInstStart());
                            vo.setProjectId(wew.getProjectId());
                            vo.setProjectCode(wew.getProjectCode());
                            vo.setProjectName(wew.getProjectName());
                            vo.setRecordId(wew.getId());
                        }
                        return vo;
                    }).filter(w -> Objects.nonNull(w.getInstStart()))
                    // 根据预警开始时间自然排序 时间小的在前 历时时间最长
                    .sorted(Comparator.comparing(WorkbenchProjectMonitorEarlyWarnVO::getInstStart))
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    private void buildPermissionWarningWrapper(LambdaQueryWrapper<ProjectEarlyWarning> wrapper, UserInfoDetails user) {
        //如果是超管的话
        if (user.hasSuperAdmin() || user.hasRegionAdmin()) {
            // 可以看所有
        }
        // 如果是单位管理员 查看本单位及下属单位
        else if (user.hasOrgAdmin()) {
            List<String> viewChildIdsRecursion = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId())
                    .stream().map(String::valueOf).collect(Collectors.toList());
            wrapper.in(ProjectEarlyWarning::getBuildOrgCode, viewChildIdsRecursion);
        } else {
            // 否则都只能看自己单位的
            wrapper.eq(ProjectEarlyWarning::getBuildOrgCode, user.getMhUnitIdStr());
        }
    }

    public PageVo<ProjectLibListItemVO> pageMajorProjects(WorkbenchProjectLibReq req) {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(req.getProjectYear() != null, Project::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_STOP_DELETE_PASSED)
                .eq(Project::getIsMajorProject, Boolean.TRUE)
                .orderByDesc(Project::getUpdateOn);
        UnitQueryState state = mhUnitQueryAuthHelper.listCanViewUnitIds(req.getUnitId());
        if (!state.isState()) {
            return PageVo.empty();
        }
        if (CollUtil.isNotEmpty(state.getUnitIds())) {
            query.in(Project::getBuildOrgCode, state.getUnitIds());
        }
        ProjectManageUtil.projectBaseQuery(query);
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> data = page.getRecords().stream().map(w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setBuildOrgName(w.getBuildOrgName());
            item.setBuildOrgCode(w.getBuildOrgCode());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setReviewAmount(w.getReviewAmount());
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

}
